/*
** ** ** ** ** **  Begin header  ** ** ** ** ** ** **
**
** Project : Byggelab i faget TTK4155 Industrielle Datasystemers Konstruksjon 2005
** Filename : I2C_driver.c
** Version : 1
** Date : 23.11.2005
**
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
**
** Copyright (c) 2005, Kristian M. Knausgrd, Gunnar Kjemphol
** All rights reserved.
**
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **

VERSION LOG:
-----------
Version: 1
Date: 23.11.2005
Authors: Kristian M. Knausgrd, Gunnar Kjemphol
Coauthors: <none>
Description : Original version.
Target platform: ATMega128

FILE FUNCTIONALITY DESCRIPTION:
-----------

I2C driver for ATMEGA128

** ** ** ** ** ** ** End header ** ** ** ** ** ** ** **
*/


#include <compat/twi.h>
#include <stdio.h>
#include "I2C_driver.h"


#define WANTED_SCL_FREQ 20000
#define CPU_CLOCK_FREQ 8000000
#define TWPS_VALUE 1	// IF OTHER THAN 1, THE '4' IN THE FORMULA BELOW NEEDS TO BE MULTIPLIED WITH ITSELF A CORRESPONDING NUMBER OF TIMES.
#define TWBR_VALUE 192 //(((CPU_CLOCK_FREQ/WANTED_SCL_FREQ)-16)/(2*4))


/**
* Initialize I2C/TWI
*/
void i2c_init(void) {
	//printf("I2C freq: %d\n\r", TWBR_VALUE);
	//DDRD |= (1<<PD0)|(1<<PD1); // Set to output (NOT NECCESSARY)
	
	
	// Set SCL frequency:
	//TWBR = TWBR_VALUE;//TWBR_VALUE;	// Set bitrate register
	//TWDR = 0xFF;		//Set default content released
	//TWSR &= ~( (1<<TWPS1) | (1<<TWPS0)); // Set TWPS (Bitrate prescaler) to 1
	//TWSR |= ( (0<<TWPS1) | (1<<TWPS0));
	//TWCR = 0;
	//TWCR = (1<<TWEN)|(0<<TWIE)|(0<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC); // Enable TWI, disable interrupt, no signal request

  TWSR &= ~0x03;
  TWBR  = 28;
  TWCR |= _BV(TWEN);
	
}

#define I2C_START     ((1 << TWINT)| (1<<TWSTA) | (1<<TWEN))
#define I2C_MASTER_TX ((1<<TWINT)|(1<<TWEN))
#define I2C_TIMEOUT   1000

/**
* Send data
*/
uint8_t i2c_send(uint8_t receipient_address, char *data, int data_length) {
//	int debugi=0;
//	printf("Debug: %d\n\r",debugi++);

	uint8_t status;
	uint8_t expected_status = 0x08;

	uint16_t ms_count = 0;

	/* send start condition to take control of the bus */
	TWCR = TW_START;

	while (!(TWCR & (1 << TWINT)) && (ms_count < I2C_TIMEOUT))
	;


	if (ms_count >= I2C_TIMEOUT) {
		printf("i2c start timeout error\n\r");
		//i2c_error(s_i2c_start_error, TWCR, TWSR);
		//i2c_error(s_i2c_timeout, TWCR, TWSR);
		return -1;
	}

	/* verify start condition */
	status = TWSR;
	if (status != expected_status) {
		printf("i2c start error\n\r");
		//i2c_error(s_i2c_start_error, TWCR, status);	
		return -1;
	}

//	printf("Debug: %d\n\r",debugi++);
	// 3b: Load slave (SLA+W) address into TWDR register
	TWDR = (receipient_address) & ~(0x01); // Put W  bit as least significant bit
	TWCR = (1<<TWINT) | (1<<TWEN); // Clear TWINT bit in TWCR to start transmission of address.
//	printf("Debug: %d\n\r",debugi++);
	// 4: Wait for TWINT flag set. This indicates that the SLA+w address has been transmitted, and ACK/NACK has been received.
	while(!(TWCR & (1<<TWINT))) {
		;
	}
//	printf("Debug: %d\n\r",debugi++);
	// 5a: Check value of TWI Status register. Mask prescaler bits. If status different from MT_SLA_ACK, print error msg.
	if ((TWSR & 0xF8) != TW_MT_SLA_ACK) {
		printf("Error 2 occured when trying to send with I2C\n\r");
	}
//	printf("Debug: %d\n\r",debugi++);
		
	// *** Data part starts here ***
	int i;
	// Send all data bytes...
	for(i=0;i<data_length;i++) {
		// 5b: Load data into TWDR register. Clear TWINT bit in TWCR to start transmission of data
		TWDR = data[i];
		TWCR = (1<<TWINT) | (1<<TWEN);
	
		// 6: Wait for TWINT flag set. Thies indicates that the data has been transmitted, and ACK/NACK has been received.
		while(!(TWCR & (1<<TWINT))) {
			;
		}
		
		// 7a: Check value of TWI status register. Mask prescaler bits. If status different from MT_DATA_ACK, printf error msg.
		if ((TWSR & 0xF8) != TW_MT_DATA_ACK) {
			printf("Error 2 occured when trying to send with I2C\n\r");
		}
	}
	// *** Data part ends here ***
//	printf("Debug: %d\n\r",debugi++);
	// 7b: Transmit STOP condition
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
	
}


void i2c_receive(uint8_t receipient_address, char *data, int data_length) {
//	int debugi=0;
//	printf("Debug: %d\n\r",debugi++);
	
	// 1: Send START condition
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);//|(1<<TWIE);
//	printf("Debug: %d\n\r",debugi++);
	// 2: Wait for TWINT flag set. This indicates that the START condition has been transmitted.
	while(!(TWCR & (1<<TWINT))) {
		;
	}
//	printf("Debug: %d\n\r",debugi++);
	// 3a: Check value of TWI status register here if error detection is wanted
	if ((TWSR & 0xF8) != TW_START) {
		//error_handeler(); // Call error handeler
		printf("Error 1 occured when trying to receive from I2C\n\r");
	}
//	printf("Debug: %d\n\r",debugi++);
	// 3b: Load slave (SLA+W) address into TWDR register
	TWDR = (receipient_address) | (1 << 0); // Put W  bit as least significant bit
	TWCR = (1<<TWINT) | (1<<TWEN); // Clear TWINT bit in TWCR to start transmission of address.
//	printf("Debug: %d\n\r",debugi++);
	// 4: Wait for TWINT flag set. This indicates that the SLA+w address has been transmitted, and ACK/NACK has been received.
	while(!(TWCR & (1<<TWINT))) {
		;
	}
//	printf("Debug: %d\n\r",debugi++);
	// 5a: Check value of TWI Status register. Mask prescaler bits. If status different from MT_SLA_ACK, print error msg.
	if ((TWSR & 0xF8) != TW_MT_SLA_ACK) {
		printf("Error 2 occured when trying to receive from I2C\n\r");
	}
//	printf("Debug: %d\n\r",debugi++);
		
	// *** Data part starts here ***
	int i;
	// Receive all data bytes...
	for(i=0;i<data_length;i++) {
		// 5b: Clear TWINT bit in TWCR to start transmission of data
		TWCR = (1<<TWINT) | (1<<TWEN);
	
		// 6: Wait for TWINT flag set. Thies indicates that the data has been transmitted, and ACK/NACK has been received.
		while(!(TWCR & (1<<TWINT))) {
			;
		}
	

		// 7a: Check value of TWI status register. Mask prescaler bits. If status different from MT_DATA_ACK, printf error msg.
		if ((TWSR & 0xF8) != TW_MT_DATA_ACK) {
			printf("Error 2 occured when trying to receive from I2C\n\r");
		} else {			
			data[i] = TWDR;	
		}
	}
	// *** Data part ends here ***
//	printf("Debug: %d\n\r",debugi++);
	// 7b: Transmit STOP condition
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);
	
}

/**
 * Receives a byte of data from a TWI slave device.
 *
 * @param sda	the slave address
 * @param data	pointer to where the byte of data should be saved.
 * @return		Zero if success, else an error code
 * @author		Joshua Cantrell
 */
int8_t twi_get_data(uint8_t sla)
{
	uint8_t result = 0;
	
	
	TWCR = (1 << TWINT)           // send START
	| (1 << TWSTA)
	| (1 << TWEN);
	
	while(!(TWCR & (1 << TWINT)));   // wait for START to finish
	
	if((TWSR & 0xF8) != TW_START)
	{
	result = (TWSR & 0xF8);
	printf("Didn't start\n");
	goto error;
	}
	
	TWDR = (sla) | 0x01;        // load slave address into register
	// set the read bit
	//printf("SLA address set to %2X \n\r", sla);
	
	TWCR = (1 << TWINT)
	| (1 << TWEN);              // send address
	
	while(!(TWCR & (1 << TWINT)));   // wait for send to complete
	
	if((TWSR & 0xF8) != TW_MR_SLA_ACK)	{
		result = (TWSR & 0xF8);            // address not acknowledged
		printf("Wrong addr\n\r");
		goto error;
	}
	
	result = TWDR;                    // get data
	
	printf("Data is %2X\n", TWDR);
	
	error:
	TWCR = (1 << TWINT)              // send stop signal to complete transfer
	| (1 << TWEN)
	| (1 << TWSTO);
	
	return result;
}


uint8_t i2c_receive_one(uint8_t receipient_address) {
	return twi_get_data(receipient_address);
	uint8_t data;

//	int debugi=0;
//	printf("Debug: %d\n\r",debugi++);
	
	// 1: Send START condition
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);//|(1<<TWIE);
//	printf("Debug: %d\n\r",debugi++);
	// 2: Wait for TWINT flag set. This indicates that the START condition has been transmitted.
	while(!(TWCR & (1<<TWINT))) {
		;
	}
//	printf("Debug: %d\n\r",debugi++);
	// 3a: Check value of TWI status register here if error detection is wanted
	while ((TWSR & 0xF8) != TW_START) {
		//error_handeler(); // Call error handeler
		printf("Error 1 / receive one from I2C %d\n\r", TWSR);
	}
//	printf("Debug: %d\n\r",debugi++);
	// 3b: Load slave (SLA+W) address into TWDR register
	TWDR = ((receipient_address&0xEF)) | (1 << 0); // Put W  bit as least significant bit
	TWCR = (1<<TWINT) | (1<<TWEN); // Clear TWINT bit in TWCR to start transmission of address.
//	printf("Debug: %d\n\r",debugi++);
	// 4: Wait for TWINT flag set. This indicates that the SLA+w address has been transmitted, and ACK/NACK has been received.
	while(!(TWCR & (1<<TWINT))) {
		;
	}
//	printf("Debug: %d\n\r",debugi++);
	// 5a: Check value of TWI Status register. Mask prescaler bits. If status different from MT_SLA_ACK, print error msg.
	if ((TWSR & 0xF8) != TW_MT_SLA_ACK) {
		printf("Error 2 occured when trying to receive one from I2C\n\r");
	}
//	printf("Debug: %d\n\r",debugi++);
		
	// *** Data part starts here ***
	//int i;
	// Receive one data byte...
	//for(i=0;i<data_length;i++) {
	// 5b: Clear TWINT bit in TWCR to start transmission of data
	TWCR = (1<<TWINT) | (1<<TWEN);

	// 6: Wait for TWINT flag set. Thies indicates that the data has been transmitted, and ACK/NACK has been received.
	while(!(TWCR & (1<<TWINT))) {
		;
	}

	// 7a: Check value of TWI status register. Mask prescaler bits. If status different from MT_DATA_ACK, printf error msg.
	if ((TWSR & 0xF8) != TW_MT_DATA_ACK) {
		printf("Error 3 occured when trying to receive one from I2C\n\r");
	} else {			
		data = TWDR;	
	}
	//}
	// *** Data part ends here ***
//	printf("Debug: %d\n\r",debugi++);
	// 7b: Transmit STOP condition
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);

	return data;
}

