#include <adc/adc.h>
#include <avr/io.h>
#include <util/delay.h>

/**
* Initialize adc system with 8 bit resolution & external reference.
*/
int init_adc( void ){
	// External reference, only ADCH active (8bit).
	ADMUX |= (1 << REFS0) | (1<<ADLAR);

	// ADC Prescaler 4.
	ADCSRA |= (1 << ADPS1);
	
	// Enable ADC.
	ADCSRA |= (1 << ADEN);

	return 0;
}


/**
* Read the given ADC channel. Hangs for 125µs if channel is changed.
*/
unsigned char read_adc( unsigned char channel ){
	static unsigned char last_channel = -1;
	// Select ADC channel 0-7.
	ADMUX = (ADMUX & 0xE0) | channel;
	
	// Wait 125 s if channel is changed.
	if(last_channel!=channel) {
		_delay_us(125);
		last_channel = channel;
	}
	
	// Start a single conversion.
	ADCSRA |= (1 << ADSC);
	//_delay_us(125);
	// Wait for conversion to complete.
	while(ADCSRA & (1 << ADSC)) { ; }
	
	return ADCH;
}


/**
* Read n values and return the average using the above declared functions.
*/
uint16_t adc_filtered_read(uint8_t channel, uint8_t n) {
	uint16_t t;
	uint8_t i;

	//adc_select_channel(channel);

	// Throw away first value
	read_adc(channel);
	//adc_start();
	//adc_wait();
	//adc_start();

	// Sample n times
	t = 0;
	for (i=0; i<n; i++) {
		//adc_wait();
		_delay_us(10);
		t += read_adc(channel); //adc_read();
		//adc_start();
	}

	return t/n;
}

