#ifndef _CONFIG_H
#define _CONFIG_H

// Master if 1, slave if removed.
#define MASTER 1

// Default configuration values
#define DEFAULT_COLLISION_THRESHOLD_CM 40


// Message ID space from BASE_ADDR to BASE_ADDR+9 is required.
#define BASE_ADDR 		0x80
#define CAN_BAUDRATE	500		// kBits

// External messages
#define CANID_COLLISION_DETECTED 				(BASE_ADDR+0x00) // Outbound
#define CANID_RANGE_MEASUREMENTS 				(BASE_ADDR+0x04) // Outbound
#define CANID_REQUEST_MEASUREMENTS 				(BASE_ADDR+0x05) // Inbound
#define CANID_FLASH_SENSOR_BOARD_LED			(BASE_ADDR+0x06) // Inbound
#define CANID_ENABLE_SENSOR_LEDS 				(BASE_ADDR+0x07) // Inbound
#define CANID_SET_COLLISION_THRESHOLDS 			(BASE_ADDR+0x08) // Inbound
#define CANID_SET_INTERRUPT_REFRESH_INTERVAL	(BASE_ADDR+0x09) // Inbound

// Internal messages (between slave and master boards)
#define CANID_COLLISION_STATUS_UPDATE (BASE_ADDR+0x01)		// From slave to master

// i2c address configuration
#define BOARD1_SONAR_I2CADDR 0xEA //nede venstre
#define BOARD2_SONAR_I2CADDR 0xE2 //oppe venstre
#define BOARD3_SONAR_I2CADDR 0xE0
#define BOARD4_SONAR_I2CADDR 0xEC
#define BOARD5_SONAR_I2CADDR 0xE8 //nede hyre
#define BOARD6_SONAR_I2CADDR 0xE4 //oppe hyre
#define BOARD7_SONAR_I2CADDR 0xE6 //bak
#define BOARD8_SONAR_I2CADDR 0xEE

/*
#define BOARD1_SONAR_I2CADDR 0xE0
#define BOARD2_SONAR_I2CADDR 0xE2
#define BOARD3_SONAR_I2CADDR 0xE4
#define BOARD4_SONAR_I2CADDR 0xE6
#define BOARD5_SONAR_I2CADDR 0xE8
#define BOARD6_SONAR_I2CADDR 0xEA
#define BOARD7_SONAR_I2CADDR 0xEC
#define BOARD8_SONAR_I2CADDR 0xEE
*/


// Startup configurations
#define DELAY_STARTUP_FOR_SEC 2 // Delay at startup.

// Engineering configuration (hardware bound)
#define TARGET_FREQ 16000000l // This is defined in the Makefile as well...
#define ATMEGA128_BAUD(baud_rate) (unsigned char)((TARGET_FREQ / (16.0 * (baud_rate))) - 1)
#define MAX_SENSOR_BOARD_COUNT 4



// Debug configuration
//#define BAUD_RATE_DEBUG 57600
//#define DEBUG_PRINTF 1
#define DEBUG_ON 0

// CANbus configuration
// CAN LIB MCU LIB CONFIGURATION
#define FOSC           16000        // 16.000 MHz External cristal


#endif

