// Utestet kode.


// Sonar w/pwm init
void srf_pwm_init(void) {
      SRF_PWM_DDR = 0xFF;
      SRF_PWM_PORT = 0;
}


/**
* Read distance to closest object from the sonar number i.
*
* PWM type SRF sensors is id 1-4, I2C type is 10->.
* Returns distance in cm.
*/
uint8_t read_pwm_sonar(uint8_t i) {
	uint16_t counter = 0; // Count
	uint8_t pin;

	if(i>=1 && i<=8) {
		// Sensor bank A, compute pin no.
		pin = (i-1);

		// Make sure SRF pin is output (should be that already..).
		SRF_PWM_DDR |= (1 << pin);

		// Toggle trig output pin.
		SRF_PWM_PORT = (1 << pin);
		_delay_us(20); 		// SRF05 Datasheet: trigger pulse must be > 10 us.
		SRF_PWM_PORT &= ~(1 << pin);

		// Switch SRF pin to input.
		SRF_PWM_DDR &= ~(1 << pin);

		// Wait for pulse back.
		while (!(SRF_PWM_PIN & (1 << pin)) ) {
		    // Must prevent hanging here if no object detected (delay > 30ms).
			if(counter++>3000) return -1;
		    _delay_us(10);
		}
		counter=0;	// Reset counter for next timing op.

		while (SRF_PWM_PIN & (1 << pin)) {
		    counter++;
		    _delay_us(10); // 10us resolution
		}

		// Make sure SRF pin is output (should be that already..) and low.
		SRF_PWM_DDR |= (1 << pin);
		SRF_PWM_PORT = 0x00;


		return (counter/29);	// ~cm
	}

	printf("Wrong sonar ID\n");
	return -1;
}
