#include <avr/io.h>
#include "srf08.h"
#include "I2C_driver.h"
#include <util/delay.h>



static uint8_t address;


void srf08_init(void) {
	uint16_t range = 0;
	i2c_init();

	address = SRF08_ADDR_2;//SRF08_ADDR_0;
	
	
	// USE TO ChANGE i2c address on SRF08 (make sure only one device on i2c bus
	//srf08_change_i2c_addr(0x80, SRF08_ADDR_0);

	char data[] = {SRF08_COMMAND_WR,SRF08_RANGE_CENTIMETERS};
	printf("Waiting for srf08 on %2X..", address);
	
	i2c_send(address, data, 2);	
	printf("Her\n\r");
	data[0] = address;
	
	while((range = i2c_receive_one(address))!= 0) {
		printf("Range %d ! ", range);
		//i2c_send(address, data, 1);
	}
	printf("..OK\n");
}


void srf08_set_gain(uint8_t gain) {
	// Check that input is in valid range:
    if(gain>31) { gain=31; }
 
	// Send on I2C bus
	char data[] = {SRF08_MAX_GAIN_WR, gain};
	i2c_send(address, &data, 2);
}


void srf08_set_range(uint16_t mm) {
	// Scale & check that input is in valid range:
    mm = (mm/43); 
    if(mm > 0xFF ) { mm = 0xFF; }

	// Send on I2C bus
	char data[] = {SRF08_RANGE_WR, mm};
	i2c_send(address, &data, 2);
}


uint8_t srf08_ping() {
	union rd {
		uint16_t received; 
		uint8_t bytes[2];
	} rd;

	// Sends a ping
	char data[] = {SRF08_COMMAND_WR, SRF08_RANGE_CENTIMETERS};
	i2c_send(address, &data, 2);

	uint8_t received;
	while((received = i2c_receive_one(address)) != 0);

	char data2[] = {SRF08_ECHO_1};
	i2c_send(address, &data2, 1);

	// Receive 2 bytes
	i2c_receive(address, rd.bytes, 2);

	return(rd.received);
}

/*
void srf08_select_unit(unsigned char srf08_address) {
	// New address validity check

	if( (srf08_address<0xE0 || srf08_address>0XFE) && srf08_address != 0 )  { return; }
	if(srf08_address%2) { return; }

	// Make the new i2c address the active one.
	address=srf08_address; 
}*/


uint16_t read_i2c_sonar(uint8_t addr) {
	// Place a check here..
	address = addr;

	return srf08_ping();
}


/**
* Use current addr 0x00 to change address on unknown address SRF08 device.
*/
void srf08_change_i2c_addr(uint8_t current_addr, uint8_t new_addr) {
	// Change address by sending the triple sequence
	char seq1[] = {SRF08_COMMAND_WR, SRF08_CHANGE_I2C_ADDR_SEQ1};
	char seq2[] = {SRF08_COMMAND_WR, SRF08_CHANGE_I2C_ADDR_SEQ2};
	char seq3[] = {SRF08_COMMAND_WR, SRF08_CHANGE_I2C_ADDR_SEQ3};
	char seq4[] = {SRF08_COMMAND_WR, new_addr};
	seq4[1] = new_addr;
	
	i2c_send(current_addr, &seq1, 2);
	i2c_send(current_addr, &seq2, 2);
	i2c_send(current_addr, &seq3, 2);
	i2c_send(current_addr, &seq4, 2);
	
	address = new_addr;
}



/*
unsigned int srf08_read_register(unsigned char srf08_register) {
	union i2c_union {
		unsigned int  rx_word; 
		unsigned char rx_byte[2];
	} i2c;


	I2C_START_TX(address);
	i2c_transmit(srf08_register);
	I2C_START_RX(address);

	// get high byte msb first
	if(srf08_register>=2) { i2c.rx_byte[1]=i2c_receive(I2C_CONTINUE); }                         

	// get low byte msb first 
	i2c.rx_byte[0]=i2c_receive(I2C_QUIT);                          

	i2c_stop();

	return i2c.rx_word;
}









// I added these functions to use in my ISR

void start_ping(unsigned char metric_unit)
{
	I2C_START_TX(address);
	i2c_transmit(0);                    
	i2c_transmit(metric_unit);
}

unsigned char check_ping(void)
{
	unsigned char byte;
	
	i2c_start();
	byte=i2c_transmit(address);               
   i2c_stop();
	
	if (byte==I2C_NO_ERROR)
		return (1);
	else
		return (0);

}

unsigned int read_ping(void)
{
	union i2c_union 
	{
		unsigned int  rx_word; 
		unsigned char rx_byte[2];
	} i2c;
	
	I2C_START_TX(address);
	i2c_transmit(SRF08_ECHO_1);
	I2C_START_RX(address);
	i2c.rx_byte[1]=i2c_receive(I2C_CONTINUE);  // get high byte msb first                        
	i2c.rx_byte[0]=i2c_receive(I2C_QUIT);      // get low byte msb first                
	i2c_stop();

return(i2c.rx_word);
}
*/

