#ifndef SRF08_H
#define SRF08_H

// SRF address on i2c -- 0xE0 is default on new SRF08 sensors.
#define SRF08_ADDR_0	0xE0  
#define SRF08_ADDR_1	0xE2  
#define SRF08_ADDR_2	0xE4  
#define SRF08_ADDR_3	0xE6  
#define SRF08_ADDR_4	0xE8  
#define SRF08_ADDR_5	0xEA  
#define SRF08_ADDR_6	0xEC  
#define SRF08_ADDR_7	0xEE  
#define SRF08_ADDR_8	0xF0  
#define SRF08_ADDR_9	0xF2  
#define SRF08_ADDR_10	0xF4  
#define SRF08_ADDR_11	0xF6  
#define SRF08_ADDR_12	0xF8  
#define SRF08_ADDR_13	0xFA  
#define SRF08_ADDR_14	0xFC  
#define SRF08_ADDR_15	0xFE  

//#define SRF08_I2C_BROADCAST_ADDRESS 0X00

#define SRF08_MIN_GAIN        0
#define SRF08_MAX_GAIN        31
#define SRF08_MIN_RANGE       0      // mm
#define SRF08_MAX_RANGE       11008  // mm



// Registers
// Writable:
#define SRF08_COMMAND_WR	0
#define SRF08_MAX_GAIN_WR	1
#define SRF08_RANGE_WR		2

// Readable:
#define SRF08_SOFTWARE_REV	0
#define SRF08_LIGHT_SENSOR	1
#define SRF08_ECHO_1		2
#define SRF08_ECHO_2		4
#define SRF08_ECHO_3		6
#define SRF08_ECHO_4		8
#define SRF08_ECHO_5		10
#define SRF08_ECHO_6		12
#define SRF08_ECHO_7		14
#define SRF08_ECHO_8		16
#define SRF08_ECHO_9		18
#define SRF08_ECHO_10		20
#define SRF08_ECHO_11		22
#define SRF08_ECHO_12		24
#define SRF08_ECHO_13		26
#define SRF08_ECHO_14		28
#define SRF08_ECHO_15		30
#define SRF08_ECHO_16		32
#define SRF08_ECHO_17		34


// Commands

#define SRF08_RANGE_INCHES			0X50
#define SRF08_RANGE_CENTIMETERS		0X51
#define SRF08_RANGE_MICROSECONDS	0X52

#define SRF08_ANN_INCHES			0X53
#define SRF08_ANN_CENTIMETERS		0X54
#define SRF08_ANN_MICROSECONDS		0X55

#define SRF08_CHANGE_I2C_ADDR_SEQ1	0XA0
#define SRF08_CHANGE_I2C_ADDR_SEQ2	0XAA
#define SRF08_CHANGE_I2C_ADDR_SEQ3	0XA5



void srf08_init(void);
void srf08_set_gain(uint8_t gain);
void srf08_set_range(uint16_t mm);
uint8_t srf08_ping();
uint16_t read_i2c_sonar(uint8_t addr);
void srf08_change_i2c_addr(uint8_t current_addr, uint8_t new_addr);


#endif
