
#include <avr/io.h>

#include "pwm_motor_driver.h"

/*
Supply pwm at 32 khz to motors.
Uses timer1 FAST PWM 8bit Mode

*/
int init_pwm_motor( void ){

	// Set BRAKE and DIRECTION pins as output.
	DIR_BRAKE_PORT |= (1 << MOTOR_DIR)|(1 << MOTOR_BRAKE);
	
	// OC1A as output
	DDRB |= (1<<PB5);

	// Clear OC1A on compare match
	TCCR1A |= (1<<COM1A1);
	
	
	// Waveform generation Fast PWM 9bit
	// TOP is 0x1FF;
	TCCR1A |= (1<<WGM11);
	TCCR1B |= (1<<WGM12);
	
	// Prescaler clk/1 for 32khz on 16Mhz clock
	TCCR1B |= (1<<CS10);
	OCR1A = 0x00;
	OCR1B = 0x00;
	OCR1C = 0x00;
	

	return 0;
}

int motor_direction(char dir){
	switch (dir){
		case RW :
			BRAKE_PORT |= (1 << MOTOR_DIR);
			break;
		case FW :
			BRAKE_PORT &= ~(1 << MOTOR_DIR);
			break;
		default:
			// Do nothing
			return -1;
			break;

	}
	return 0;

}

int motor_brake(char brake_on_off){
	switch (brake_on_off){
		case ON :
			BRAKE_PORT |= (1 << MOTOR_BRAKE);
			break;
		case OFF :
			BRAKE_PORT &= ~(1 << MOTOR_BRAKE);
			break;
		default:
			return -1;
			break;
	}
	return 0;

}


/*
Sets the duty cycle on OCR1

*/

int set_ds_motor(char ds){
	OCR1A = ds;
	
	return 0;

}




