#include "cdma_config.h"

#define CDMA_STATE_OFFLINE 		0
#define CDMA_STATE_SEARCHING 	1
#define CDMA_STATE_ONLINE 		2
#define CDMA_STATE_RECEIVING 	4
#define CDMA_STATE_TRACKING	 	8

#define PHASE_LOCK_THRESHOLD 100
#define SCHMITT 50	// One way "schmitt trigger" threshold value


// Current CDMA receiver state
int receiver_state;
// Correct phase probability
int phase_lock_probability;


uint32_t chipcode = CHIPCODE;
uint32_t received;


void init_cdma() {
	received = 0;

	receiver_state = CDMA_STATE_OFFLINE;

	// Timer0
	DDRB = (1<<PB1)	;									// OC1b output
	TIMSK0 |= (1<<OCIE0B);								// Interrupt on output compare B
	OCR0A = 21; 										// TOP-value to select correct pwm-frequency
	OCR0B = 10;	 										// 50% duty-cycle
	TCCR0A = (1<<COM0B1) | (1<<WGM01) | (1<<WGM00);		// MODE7: Fast pwm with TOP = OCR0A and prescaler = 1
	TCCR0B = (1<<WGM02)  | (1<<CS00);
}


/**
* Current implementation hangs until phase lock...
*/
int sync_channel() {
	receiver_state = CDMA_STATE_SEARCHING;

	while(phase_lock_probability < PHASE_LOCK_THRESHOLD + SCHMITT) {
		correlate(in, chipcode);
	}

	if(phase_lock_probability>PHASE_LOCK_THRESHOLD) {
		receiver_state = CDMA_STATE_HASLOCK;
	}
}


/**
* 32 bit correlation (probably better in hardware..)
*/
uint8_t correlate(uint32_t *a, uint32_t *b) {
	uint8_t ret = 0;
	uint32_t c;
	c = a^b;
	for(int i = 0; i<32; i++) {
		if(!(c&0x01)) ret++;
		c = (c>>1);
	}
	return ret;
}


/**
* cdma_receive() should be triggered with about 2-10 times the chip-frequency
* of the transmission.
*/
void cdma_receive() {

	switch(receiver_state) {
		case CDMA_STATE_OFFLINE:
		
		break;
		case CDMA_STATE_SEARCHING:
			// Wait for haslock state...
		break;
		case CDMA_STATE_TRACKING:
			// Correlate expected, early & late
			ea = correlate(&received, &chipcode);
			ex = correlate(&received, &chipcode);
			la = correlate(&received, &chipcode);

			if(ex>ea && ex>la) {
				// Do nothing...
				if(ex>PHASE_LOCK_THRESHOLD) {

				}
			}
		break;
		case CDMA_STATE_RECEIVING:
			// Sample
			received = received << 1;
			received |= (PINB & 0x01);

			// Transition
			receiver_state = CDMA_STATE_TRACKING;
		break;
		default:
			// Do nothing
			printf("This can't happen!");
		break;
	}
}


/**
*
*/
int main(void) {
	// Init
	init_cdma();	

	// Enable global interrupts
	sei();

	// Update on receiver
	for(EVER) {
		cdma_receive();
	}
}


/**
* Interrupt service routine
*/
ISR(TIM0_COMPB_vect){
	if(receiver_state == CDMA_STATE_HASLOCK) receiver_state = CDMA_STATE_RECEIVING;
}




