#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>





#include "config.h"
#include "SPI/spi_driver.h"
#include "USART/usart0_driver.h"
#include "CAN/can_lib.h"
#include "DCMOTOR/motorcontrol.h"

#define EVER ;;
#define TRUE 1
#define FALSE 0


/**
* Busy-wait s seconds
*/
void _delay_s(int s) {
	int i;
	s*=10;
	for(i=0;i<s;i++)
			_delay_ms(100);
}




/**
* Set up input capture pin for IR signal reading
*/
void init_input_capture() {
	//
	TCCR1B &= ~(1<<ICES1);				// Input-Capture edge detect
	TCCR1B |= (1<<CS11) | (1<<CS10);	// Timer prescaler = 1
	//PORTD |= (1<<PD4);				// Enable pull-up on IC1
	TIMSK1 |= (1<<ICIE1);				// Input Capture interrupt enable
}


void init_infrared() {
	init_input_capture();
}

/**
* Initialize all subsystems
*/
int init( void ) {
	// Return value
	int retval = 0;
	
	// Set up USART for debugging
	usart0_init(51);

	// Delayed startup
	if(DELAY_STARTUP_FOR_SEC) _delay_s(DELAY_STARTUP_FOR_SEC);
	
	// Bind stdout to use uart0_putc on USART (printf is active after this point)
	fdevopen(usart0_putc, NULL);
	printf("Starting up positioning system..\n\r");
	
	// Initialize motor
	printf("Initializing motor subsystem..");
	//init_motor();
	printf("..OK\n\r");

	// Set up SPI
	printf("Initializing SPI.. ");
	spi_master_init();
	printf("..OK\n\r");
	
	// Set up CAN interface
	printf("Initializing CANbus.. ");
	init_can();
	printf("..OK\n\r");
	
	// Set up infrared interface
	printf("Initializing IR.. ");
	init_infrared();
	printf("..OK\n\r");

	// Enable interrupts
	sei();

	// Return retval
	return retval;
}


#define MIN_BEACON1_TIME	5
#define MAX_BEACON1_TIME	12
#define MIN_BEACON2_TIME	20
#define MAX_BEACON2_TIME	30
#define MIN_BEACON3_TIME	40
#define MAX_BEACON3_TIME	60

// Used for reading IR signals
unsigned int ir_pulselength = 0;

unsigned char beacon1_count = 0;
unsigned char beacon2_count = 0;
unsigned char beacon3_count = 0;

unsigned int angle1 = 0;
unsigned int angle2 = 0;
unsigned int angle3 = 0;

unsigned char last_read_beacon = 0;



/**
* Check beacon (if on)
*/
void check_beacon(unsigned int time)
{
	if((time > MIN_BEACON1_TIME) && (time < MAX_BEACON1_TIME))	{
		
		beacon1_count ++;
		beacon2_count = 0;
		beacon3_count = 0;
		
		if(beacon1_count == 3) {
			angle1 = read_angle();
			last_read_beacon = 1;
		}
		
	} else if((time > MIN_BEACON2_TIME) && (time < MAX_BEACON2_TIME)) {
		
		beacon1_count = 0;
		beacon2_count ++;
		beacon3_count = 0;
		if(beacon2_count == 3) {
			angle2 = read_angle();
			last_read_beacon = 2;
		}
		
	} else if((time > MIN_BEACON3_TIME) && (time < MAX_BEACON3_TIME)) {
		beacon1_count = 0;
		beacon2_count = 0;
		beacon3_count ++;
		
		if(beacon3_count == 3) {
			angle3 = read_angle();
			last_read_beacon = 3;
		}
		
	}
}


/**
* Reads the current tower angle.
*/
int read_angle() {
	/*
	PORTB &= ~(1<<PB0);
	test1 = spi_master_tranceive_NO_CS(0xFF);
	test2 = spi_master_tranceive_NO_CS(0xFF);
	test3 = spi_master_tranceive_NO_CS(0xFF);
	ringbuffer[i++] = (test1 << 4)|((test2>>4)&0xFF);
	PORTB |= (1<<PB0);
	result = 0;
	for(j=0;j<10;j++)
		result += ringbuffer[j]/10;
	
	
	//if(i>=10){
		i=0;
		//printf("Test: %d\n\r", test1);
		printf("Position: %d deg %d/2048\n\r", (int)(360.0*((float)result/(float)2048)), result);
	*/
	return 10;
}

/**
* Atmel appnote
*/
int init_can() {
	U8 txt_can_ana[]="======= UART in CAN ANALYZER =======";
	U8 txt_ech_mod[]="======== UART in ECHO MODE =========";
	
	U8 u8_temp;
	U8 buffer[8];
	st_cmd_t message;

	//- Pull-up on TxCAN & RxCAN one by one to use bit-addressing
    CAN_PORT_DIR &= ~(1<<CAN_INPUT_PIN );
    CAN_PORT_DIR &= ~(1<<CAN_OUTPUT_PIN);
    CAN_PORT_OUT |=  (1<<CAN_INPUT_PIN );
    CAN_PORT_OUT |=  (1<<CAN_OUTPUT_PIN);
	
	printf("Waiting for CAN msg (autobaud conf)..\n\r");
	// --- Init CAN (special AUTOBAUD)  
        //- Wait until activity on RxCAN
    //while ((CAN_PORT_IN & (1<<CAN_INPUT_PIN)) != 0);
        //- Reset CAN peripheral
    Can_reset();
        //- Set CAN Bit-timming
    can_init((U16)CAN_BAUDRATE);        // c.f. macro in "can_drv.h"
        //- Set CAN Timer Prescaler
    //CANTCON = CANBT1;                   // Why not !
}

/**
* Main method
*/
int main( void ) {
	// State variable
	int running = TRUE;

	// Run initialization
	init();

	// Test motor
	init_motor();

	/*
	// Activate progress LED
	char led_red = 0;
	char led_green = 0;
	
	DDRB |= (1 << PB6);
	DDRB |= (1 << PB7);
	
	#define LED_RED PB7
	#define LED_GREEN PB6
	
	PORTB &= ~(1 << LED_GREEN);
	PORTB |= (1 << LED_RED);
	*/


	
	
	// Outer main-loop
	char test1 = 0xFF;
	char test2 = 0xFF;
	char test3 = 0xFF;
	int result = 0;
	int ringbuffer[10];
	int i=0,j=0;
	for(i=0;i<10;i++);
		ringbuffer[i] = 0;
	DDRB |= (1<<PB0);
	PORTB |= (1<<PB0);
	printf("Starting up..\n\r");

	char speed = 0;
	char increasing = 1;

	while(running) {
		
/*	
		if(i>=10){
			i=0;
			
			
			//- CAN ECHO: WAIT FOR RECEIVED
			// --- Init Rx data
			message.pt_data = &buffer[0];
			for(i=0; i<8; i++) buffer[i]=0;
	
			
			
			// --- Init Tx data
			buffer[0] = 0x01;
			buffer[1] = 0x05;
	
			// --- Tx Command
			message.id.ext++;   // Incrementation of ID to revome possible clashes
			message.dlc = 8;
			message.cmd = CMD_TX_DATA;
			
			// --- Enable Tx
			while(can_cmd(&message) != CAN_CMD_ACCEPTED);
			// --- Wait for Tx completed        
			while(1)
			{
				u8_temp = can_get_status(&message);
				if (u8_temp != CAN_STATUS_NOT_COMPLETED) break; // Out of while
			}
			
			// ---- Exit if CAN error(s)
			if (u8_temp == CAN_STATUS_ERROR) printf("CAN err 1\n\r"); // Out of function
			
		}
		
		if(increasing) {
			speed++;
			
			if(speed<100) {
				set_speed(speed);
			} else {
				increasing = 0;
				set_dir(MOTOR_CW);
			}
		} else {
			speed--;
			if(speed>0) {
				set_speed(speed);
			} else {
				increasing = 1;
				set_dir(MOTOR_CCW);
			}
		}
		*/
		printf("Last read beacon: %d\n\r", last_read_beacon);
		
		_delay_ms(200);
		_delay_ms(200);
		_delay_ms(200);
		//_delay_s(2);
	}
	
	printf("Some error..\n\r");
	// Hang here to stop in case of running == FALSE
	for(EVER) {;}

	return 0;
}



/**
* Interrupt from ICP (for reading infrared signal)
*/
ISR(TIMER1_CAPT_vect) {
	
	if(!(TCCR1B && (1<<ICES1))) {
		// Falling Edge
		TCCR1B |= (1<<ICES1);			// Set trigger to rising edge
		ICR1H = 0;
		ICR1L = 0;
	} else {
		// Rising edge
		TCCR1B &= ~(1<<ICES1);			// Set trigger to falling edge
		ir_pulselength = ICR1; 
		check_beacon(ir_pulselength);
		TIFR1 |= (1<<ICF1);				// Reset interrupt flag
		TCNT1 = 0;
	}
	
}



