#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

#include "usart0_driver.h"


#define TARGET_FREQ 16000000UL // This is defined in the Makefile as well...
#define ATMEGA8_BAUD(baud_rate) ((TARGET_FREQ / (8UL * (baud_rate))) - 1)


// IRDA ENC/DEC Card
#define RX 0
#define TX 1
#define ED_MODE PC0
#define ED_EN PC1
#define TR_MODE PC2
#define ED_RST PD3	
#define TR_SD PC3



/**
* Initialize
*/
int8_t init(void) {

	cli();

	// Set up USART for IRDA
	usart0_init(ATMEGA8_BAUD(57600UL));//8);
	fdevopen(usart0_putc, NULL);



	// Set control pins as output
	DDRC |= (1 << ED_MODE) | (1 << ED_EN) | (1 << TR_MODE) |  (1 << TR_SD);
	DDRD |= (1 << ED_RST);

	// Set data mode + reset on + mode + tranceiver shutdown off
	PORTC = (0 << ED_MODE) | (1 << ED_RST) | (1 << ED_EN) | (0 << TR_MODE);
	PORTD |= (1 << ED_RST);
	PORTC &= ~(1 << TR_SD);



	//Pulse length measuring
	
	//enable overflow and input capture interrupts

	TIMSK = (1 << TICIE1)|(1 << TOIE1);

	/*Noise canceller, without prescaler, rising edge*/

	TCCR1B = (1 << ICNC1)|(0 << ICES1)|(1 << CS10)|(0 << CS12);
	
}

uint32_t pulseCount = 0;
uint32_t pulseCount2 = 0;
uint32_t pulseCount3 = 0;
uint32_t pulseDetected = 0;

int main(void) {
	// Init
	init();
	
	// Status led
	DDRB |= (1<<PB1);
	PORTB &= ~(1<<PB1);
	
	// Enable global interrupts
	sei();
	
	// Hang here
	while(1){
		//printf("%d", pulseDetected);
		if(pulseDetected ==1){
			pulseCount++;
			if(pulseCount>10){
				printf("1");
				pulseCount=0;
			}
			pulseDetected=0;
			printf(".");
		}
		else if(pulseDetected ==2){
			pulseCount2++;
			if(pulseCount2>10){
				printf("2");
				pulseCount2=0;
			}
			pulseDetected=0;
			printf("-");
		}
		else if(pulseDetected ==3){
			pulseCount3++;
			if(pulseCount3>10){
				printf("3");
				pulseCount3=0;
			}
			pulseDetected=0;
			printf("*");
		}

	}
}





//define ovrflow counter

uint16_t ov_counter;

//define times for start and end of signal

uint16_t rising, falling;

//define overall counts

uint32_t counts;



//overflow counter interrupts service routine

ISR(TIMER1_OVF_vect){

  ov_counter++;

}

//Timer1 capture interrupt service subroutine

ISR(TIMER1_CAPT_vect){

	

	if ((PINB&(1 << PB0))){
		//save falling time

		falling=ICR1;

		//rising edge triggers next

		TCCR1B=TCCR1B&0xBF;

		counts=(uint32_t)falling-(uint32_t)rising+(uint32_t)ov_counter;
		
		//printf("%d\n\r", counts);
		if(counts>1200&&counts<1300){
			//printf("%d\n\r",counts);
			pulseDetected = 1;
		}
		else if(counts>2500&&counts<2600){
			pulseDetected = 2;
		}
		else if(counts>3550&&counts<3650){
			pulseDetected = 3;
			//printf("%d\n\r", counts);
		}

	}
	else{
	
		//save start time

		rising=ICR1;
		

		//set to trigger on falling edge

		
		TCCR1B=TCCR1B|0x40;
		
		
		//reset overflow counter

		ov_counter=0;



	}


}
