#include <avr/io.h>
#include <avr/interrupt.h>

#include "motorcontrol.h"



/**
* Ikke i bruk nå
*/
void timer16_wr_oc1ab(unsigned int i) {
  unsigned char sreg;
  // Save global interrupt flag
  sreg = SREG;
  // Disable interrupts
  cli();
  // Set TCNTn to i
  TCNT1 = i;
  // Restore global interrupt flag
  SREG = sreg;
}


/**
* Set up all necessary stuff for motor contro (PWM, IO ports etc.)
*
* Starts the motor with default "speed" somewhat below 50%
*/
/*
int init_motor() {
	// Set pins as output
	DDRB |= (1<<LEFTBRIDGE)|(0<<RIGHTBRIDGE)|(1<<HBRIDGE_ENABLE);
	
	// Disable H-bridge until PWM is on. Left bridge on.
	motor_off();
	
	// Set default direction clockwise
	set_dir(MOTOR_CW);
	
	// Start 35kHz PWM on both channels (16MHz/457)
	// Toggle on compare match both channels, using P&F correct PWM
	TCCR1A = (1<<COM1A0)|(1<<COM1B0)|(0<<WGM11)|(0<<WGM10);
	TCCR1B = (1<<WGM13)|(0<<WGM12)|(1<<CS10); // 1 is clk/1
	ICR1 = PWM_TOP;	// This is the TOP value (sets frequency ~35kHz)
	
	// Default is somewhat below 50% duty cycle
	OCR1A = 200;	// This is the duty cycle value (TOP is 100%)
	OCR1B = 200;	// This is the duty cycle value (TOP is 100%)
	
	// Turn on motor
	motor_on();
}
*/
int init_motor( void ){

	// Set pins as output
	DDRB |= (1<<LEFTBRIDGE)|(1<<RIGHTBRIDGE)|(1<<HBRIDGE_ENABLE);

	// Disable H-bridge until PWM is on. Left bridge on.
	motor_off();
	
	// Set default direction clockwise
	set_dir(MOTOR_CW);
	
	// Waveform generation Fast PWM 8bit
	// TOP is 0xFF;
	TCCR1A |= (1<<WGM10);
	TCCR1B |= (1<<WGM12);
	
	// Prescaler clk/1 for 32khz on 8Mhz clock
	TCCR1B |= (1<<CS10);
	
	// Initial values
	OCR1A = 0xFF;
	OCR1B = 0xFF;
	OCR1C = 0x00;

	// Turn on motor
	motor_on();

	return 0;
}


/**
* Turn off halfbridge circuitry
*/
void motor_off() {
	PORTB &= ~(1<<HBRIDGE_ENABLE);
}


/**
* Turn on halfbridge circuitry
*/
void motor_on() {
	PORTB |= (1<<HBRIDGE_ENABLE);
}


/**
* Sets the motor speed
*/
void set_speed(char percent) {
	char val = (char)(((unsigned int)percent*(unsigned int)PWM_TOP)/(unsigned int)100);
	
	// Both PWM channels needs to be set, as we can change direction at any moment.
	OCR1A = val;	// This is the duty cycle value (TOP is 100%)
	OCR1B = val;	// This is the duty cycle value (TOP is 100%)
}


/**
* Sets the motor direction
*/
void set_dir(char dir) {
	switch(dir){
		case MOTOR_CW:
			DDRB &= ~(0<<RIGHTBRIDGE);
			DDRB |= (1<<LEFTBRIDGE);
			// Clear OC1A on compare match
			TCCR1A &= ~(1<<COM1B1);
			TCCR1A |= (1<<COM1A1);
		break;
		case MOTOR_CCW:
			DDRB &= ~(0<<LEFTBRIDGE);
			DDRB |= (1<<RIGHTBRIDGE);
			// Clear OC1A on compare match
			TCCR1A &= ~(1<<COM1A1);
			TCCR1A |= (1<<COM1B1);
		break;
		default:
			// Do nothing
		break;
	}
}
