#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "at90can128_driver.h"
#include "../hovedkort_main.h"

/*
Initialisation for CAN bus Recieve Frames on MOb channel 0, and send frames at
MOb channel 1

*/


int init_can( void ){
	CANSTMOB = 0x00;

	int mob_number;
	int num_data;
	// Reset CAN
	CANGCON |= (1<<SWRES);

	// Reset all mail boxes, the MOb registers has no initial value after RESET
	for ( mob_number = 0; mob_number < 15; mob_number++ ){
		CANPAGE = mob_number << 4; 		// MOb numbers form 0 to 14, 4 MSB in CANPAGE regiser.
										// the rest in this for loop is for the coresponding MOb number.
		CANCDMOB = CONMOB_DISABLE;		// MOb Configuration
		CANSTMOB = 0;					// Clear MOb Status register.
		CANIDT1 = 0;					// Clear CAN Identifier Tag Registers
		CANIDT2 = 0;
		CANIDT3 = 0;
		CANIDT4 = 0;
		CANIDM1 = 0;					// Clear CAN Identifier Mask Registers
		CANIDM2 = 0;
		CANIDM3 = 0;
		CANIDM4 = 0;
		for ( num_data = 0; num_data < 8; num_data++ )
			CANMSG = 0;
	}
	// Bit timing at 125kbps with 16 MHz external crystal (recommanded).
//	CANBT1  = 0x0E; 					// from Table 20-2 in Datasheet
//	CANBT2  = 0x0C;
//	CANBT3  = 0x37;

	// Bit timing at 500kbps with 16 MHz external crystal (recommanded).
	CANBT1  = 0x02; 					// from Table 20-2 in Datasheet
	CANBT2  = 0x0C;
	CANBT3  = 0x37;

	// Bit timing at 1 Mbps with 16 MHz external crystal (recommanded).
//	CANBT1  = 0x00; 					// from Table 20-2 in Datasheet
//	CANBT2  = 0x0C;
//	CANBT3  = 0x37;

	// Start CAN / Enable
	CANGCON |=  0x02;

	// Chanel 0 MOb page 0 enable for recieving
	CANPAGE = (0 << 4);
	CANSTMOB = 0x00;					// Clear MOb Status register.
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration

	//CANIDT1 = 0x26;						// Identifier TAG (IDT10:0) 11 bits. CANIDT 0x130 - 0x13F;
	//CANIDT2 &= ~0xE0;

	//CANIDM1 = 0xFE;						// Set mask
	//CANIDM2 &= ~0xE0;
	//CANIDM4 = 0;

	// Channel 0 configuration
//	CANIDT4 &= ~(1 << RTRMSK);			// Comparison True Forced
	CANIDT4 &= ~0x04;

	CANCDMOB |= NUM_BYTES_DATA;			// Reception 8 bytes
	CANCDMOB |= CONMOB_ENABLE_RESEP;	// Recieve

// Channel 1
	// Channel 1 MOb page 1 enable for transmitting
	CANPAGE = (1 << 4);
	CANSTMOB = 0x00;						// Clear MOb Status register.
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration

	// Interrupt configuration
	CANIE2|=0x01;						// IECH0=1
	CANGIE = ((1<<ENRX)|(1<<ENIT));	// Can_Rx & IT enable
	return 0;
}


int send_can_message( can_message *can_message_1){

	CANPAGE |= (1 << MOBNB0); // Message object 1

	CANCDMOB &= 0xE0;
	CANCDMOB |= can_message_1->length;


	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = can_message_1->id << 5;
	CANIDT1 = can_message_1->id >> 3;

	for(int i = 0; i < can_message_1->length; i++){
		CANMSG = can_message_1->bytes[i];
	}

	CANSTMOB = 0x00;


	CANCDMOB |= CONMOB_ENABLE_TRANS;	// Enable transmission
	CANGIT = CANGIT;					// Reset all flags
	CANSTMOB = 0x00;					// Clear MOb Status register.
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
	return 0;
}


#define CAN_ADDR_POSITIONING_BEACON_ANGLES 0x120
#define CAN_ADDR_POSITIONING_BEACON_REQUEST 0x121
#define CAN_ADDR_POSITIONING_UPDATE_ANGLES 0x122


can_message cmsg;

void send_beacon_angles(){
	cmsg.id = CAN_ADDR_POSITIONING_BEACON_ANGLES;
	cmsg.length = 6;
	cmsg.bytes[0] = ((int)t1Average)&0xFF;
	cmsg.bytes[1] = ((int)t1Average)>>8;
	cmsg.bytes[2] = ((int)t2Average)&0xFF;
	cmsg.bytes[3] = ((int)t2Average)>>8;
	cmsg.bytes[4] = ((int)t3Average)&0xFF;
	cmsg.bytes[5] = ((int)t3Average)>>8;
	send_can_message(&cmsg);
}

void update_beacon_angles(){

}

can_message rmsg;

ISR(SIG_CAN_INTERRUPT1){
	PORTC = 0x55;
	CANPAGE = (0 << 4);

	if(CANSTMOB << 6){
		rmsg.id = (((int)(CANIDT2))>>5) + (((int)(CANIDT1))<<3);
		rmsg.length = CANCDMOB&0x0F;
		for(int ii = 0; ii < rmsg.length; ii++){
			rmsg.bytes[ii] = CANMSG;
			//printf(" - %x - ", rmsg.bytes[ii]);
		}
		//printf("CAN-ID: %d\n\r", rmsg.id);
		switch(rmsg.id){
			case(CAN_ADDR_POSITIONING_BEACON_REQUEST):
				send_beacon_angles();
				break;
			case(CAN_ADDR_POSITIONING_UPDATE_ANGLES):
				t1Average = (rmsg.bytes[0])|(rmsg.bytes[1]<<8);
				for(int i = 0; i<TOWERBUFSIZE; i++){
					tower1[i] = t1Average;
				}
				t2Average = (rmsg.bytes[2])|(rmsg.bytes[3]<<8);
				for(int i = 0; i<TOWERBUFSIZE; i++){
					tower2[i] = t2Average;
				}
				t3Average = (rmsg.bytes[4])|(rmsg.bytes[5]<<8);
				for(int i = 0; i<TOWERBUFSIZE; i++){
					tower3[i] = t3Average;
				}
		//		printf("Fikk: %f, %f\n\r", rmsg.bytes[0], rmsg.bytes[1]);
				send_beacon_angles();
				break;
			default:
				break;
		}
	}
	CANPAGE = (0 << 4);
	CANSTMOB = 0x00;
	CANEN2 |= (1 << 0);
	CANCDMOB = NUM_BYTES_DATA;			// Reception 8 bytes
	CANCDMOB |= CONMOB_ENABLE_RESEP;	// Recieve
	CANGIT = CANGIT;
}


