/**
* Kristian I. Øien 2008
*/

#include <avr/io.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

#include "config.h"
#include "usart0_driver.h"
#include "usart1_driver.h"
#include "ls7366_quadrature.h"
#include "can/at90can128_driver.h"
#include "DCMOTOR/motorcontrol.h"
#include "hovedkort_main.h"

#define TRUE 1
#define FALSE 0
#define TARGET_FREQ 16000000UL // This is defined in the Makefile as well...
#define ATMEGA8_BAUD(baud_rate) ((TARGET_FREQ / (16UL * (baud_rate))) - 1)


// IRDA ENC/DEC Card
#define RX 0
#define TX 1
#define ED_MODE PE5
#define ED_EN PE6
#define ED_RST PE7
#define TR_MODE PE3
#define TR_SD PE4




/**
* Busywait s seconds
*/
void _delay_s(uint8_t s) {
	uint8_t i;
	s*=1;
	for(i=0;i<s;i++) {
		_delay_ms(100);
	}
}


/**
* Initialize
*/
int8_t init(void) {

	cli();


	// Set up USART0 for IRDA
	usart0_init(ATMEGA8_BAUD(57600UL));

	// Set up USART1 for debugging
	usart1_init(ATMEGA8_BAUD(57600UL));//8);

	// Bind stdout to use uart1_putc (Only used for debugging).
	fdevopen(usart1_putc, NULL);
	printf("\n\nStarter hovedkort for posisjonering... BAUD: %d , %d\n\r",ATMEGA8_BAUD(57600UL), ATMEGA8_BAUD(115200UL));


	// Set control pins for MCP 2120
	DDRE |= (1 << ED_MODE) | (1 << ED_EN) | (1 << TR_MODE) | (1 << ED_RST) | (1 << TR_SD);

	// Set data mode + reset on + mode + tranceiver shutdown off
	PORTE = (0 << ED_MODE) | (1 << ED_RST)| (1 << ED_EN) | (0 << TR_MODE) | (0 << TR_SD);


	// Set up CAN interface
	printf("Initializing CANbus... ");
	init_can();
	printf("..OK\n\r");

	// Initialize quadrature counter
	printf("Init quadrature.. ");
	init_quadrature();
	printf("..OK\n\r");

	// Initialize motor
	printf("Initializing DC motor control..");
	init_motor();
	printf("..OK\n\r");

	// Enable interrupts
	sei();

}

/**
* Atmel appnote
*/
/*int init_can() {
	//- Pull-up on TxCAN & RxCAN one by one to use bit-addressing
    CAN_PORT_DIR &= ~(1<<CAN_INPUT_PIN );
    CAN_PORT_DIR &= ~(1<<CAN_OUTPUT_PIN);
    CAN_PORT_OUT |=  (1<<CAN_INPUT_PIN );
    CAN_PORT_OUT |=  (1<<CAN_OUTPUT_PIN);

	printf("Waiting for CAN msg (autobaud conf)..\n\r");
	// --- Init CAN (special AUTOBAUD)
        //- Wait until activity on RxCAN
    //while ((CAN_PORT_IN & (1<<CAN_INPUT_PIN)) != 0);
        //- Reset CAN peripheral
    Can_reset();
        //- Set CAN Bit-timming
    can_init((U16)CAN_BAUDRATE);        // c.f. macro in "can_drv.h"
        //- Set CAN Timer Prescaler
    //CANTCON = CANBT1;                   // Why not !
}*/

// Find average value from towerBuf
float average(int * towerBuf){
	if(towerBuf[0]>=(360-TOWERMARGIN)){
		int sum = towerBuf[0];
		for(int i = 1; i<TOWERBUFSIZE;i++){
			if(towerBuf[i]<(360-2*TOWERMARGIN)){
				sum += towerBuf[i]+360;
			}
			else{
				sum += towerBuf[i];
			}
		}
		float avg = (float)sum/(float)TOWERBUFSIZE;
		if(avg>360){
			return avg-360;
		}
		else{
			return avg;
		}

	}
	else if(towerBuf[0]<=TOWERMARGIN){
		int sum = towerBuf[0];
		for(int i = 1; i<TOWERBUFSIZE;i++){
			if(towerBuf[i]>(2*TOWERMARGIN)){
				sum += towerBuf[i]-360;
			}
			else{
				sum += towerBuf[i];
			}
		}
		float avg = (float)sum/(float)TOWERBUFSIZE;
		if(avg<0){
			return avg+360;
		}
		else{
			return avg;
		}
	}
	else{
		int sum = 0;
		for(int i = 0; i<TOWERBUFSIZE;i++){
			sum += towerBuf[i];
		}
		return (float)sum/(float)TOWERBUFSIZE;

	}
}


char quadbuf[4];

/*void updateTower(int * tower, float * towAverage, int * towerPt){

	read_ls7366(quadbuf);
	unsigned int quadCnt = 16*256*(int)(quadbuf[1]&0x0F)+256*(int)quadbuf[2]+(int)quadbuf[3];
	unsigned int angle = (int)(((float)quadCnt/(float)8191)*(float)360);

	*towAverage = average(&tower);

	if(angle<(*towAverage+TOWERMARGIN) && angle > (*towAverage-TOWERMARGIN)){
		tower[*towerPt] = angle;
		*towerPt++;
		if(*towerPt >=TOWERBUFSIZE){
			*towerPt = 0;
		}
		printf(".");
	}
	else if(*towAverage>=(360-TOWERMARGIN)&&(angle>(*towAverage-TOWERMARGIN) || angle<(*towAverage-(360-TOWERMARGIN)))){
		tower[*towerPt] = angle;
		*towerPt++;
		if(*towerPt >=TOWERBUFSIZE){
			*towerPt = 0;
		}
		printf("*");

	}
	else if(*towAverage<TOWERMARGIN && (angle<(*towAverage+TOWERMARGIN)||angle>(*towAverage+(360-TOWERMARGIN)))){
		tower[*towerPt] = angle;
		*towerPt++;
		if(*towerPt >=TOWERBUFSIZE){
			*towerPt = 0;
		}
		printf("+");
	}
	printf("1-%d\n\r", angle);
	_delay_s(1);


}*/








int main(void) {
	// Init
	init();
	DDRC |= 0x01;
	PORTC |= 0x01;

	can_message cmsg;

	//int tower1[TOWERBUFSIZE];
	int tower1Pt = 0;
	//float t1Average;
	//int tower2[TOWERBUFSIZE];
	int tower2Pt = 0;
	//float t2Average;
	//int tower3[TOWERBUFSIZE];
	int tower3Pt = 0;
	//float t3Average;



	int t1start = 310;
	for(int i = 0; i<TOWERBUFSIZE; i++){
		tower1[i] = t1start;
	}
	t1Average = t1start;

	int t2start = 62;
	for(int i = 0; i<TOWERBUFSIZE; i++){
		tower2[i] = t2start;
	}
	t2Average = t2start;

	int t3start = 349;
	for(int i = 0; i<TOWERBUFSIZE; i++){
		tower3[i] = t3start;
	}
	t3Average = t3start;


	// Main loop

	set_dir(MOTOR_CW);
	motor_on();
	set_speed(50);

	int whileCount = 0;

	while(1) {
		if(whileCount > 5){
			whileCount = 0;
			cmsg.id = 0x120;
			cmsg.length = 6;
			cmsg.bytes[0] = ((int)t1Average)&0xFF;
			cmsg.bytes[1] = ((int)t1Average)>>8;
			cmsg.bytes[2] = ((int)t2Average)&0xFF;
			cmsg.bytes[3] = ((int)t2Average)>>8;
			cmsg.bytes[4] = ((int)t3Average)&0xFF;
			cmsg.bytes[5] = ((int)t3Average)>>8;

			send_can_message(&cmsg);
			printf("\n\r\n\rTowers: %f, %f, %f\n\r\n\r",t1Average, t2Average, t3Average);
		}


		//read IR
		char tower = usart0_getc();
		//printf("%c", tower);
		//char tower = 0;

		//update Towers

		/*read_ls7366(quadbuf);
		unsigned int quadCnt = 16*256*(int)(quadbuf[1]&0x0F)+256*(int)quadbuf[2]+(int)quadbuf[3];
		unsigned int angle = (int)(((float)quadCnt/(float)8191)*(float)360);
		printf("%d\n\r", angle);
		_delay_s(30);*/

		if(tower=='1'){
			whileCount++;
			//updateTower(&tower1, &t1Average, &tower1Pt);
			read_ls7366(quadbuf);
			unsigned int quadCnt = 16*256*(int)(quadbuf[1]&0x0F)+256*(int)quadbuf[2]+(int)quadbuf[3];
			unsigned int angle = (int)(((float)quadCnt/(float)8191)*(float)360);


			t1Average = average(&tower1);

			if(angle<(t1Average+TOWERMARGIN) && angle > (t1Average-TOWERMARGIN)){
				tower1[tower1Pt] = angle;
				tower1Pt++;
				if(tower1Pt >=TOWERBUFSIZE){
					tower1Pt = 0;
				}
				printf(".");
			}
			else if(t1Average>=(360-TOWERMARGIN)&&(angle>(t1Average-TOWERMARGIN) || angle<(t1Average-(360-TOWERMARGIN)))){
				tower1[tower1Pt] = angle;
				tower1Pt++;
				if(tower1Pt >=TOWERBUFSIZE){
					tower1Pt = 0;
				}
				printf("*");

			}
			else if(t1Average<TOWERMARGIN && (angle<(t1Average+TOWERMARGIN)||angle>(t1Average+(360-TOWERMARGIN)))){
				tower1[tower1Pt] = angle;
				tower1Pt++;
				if(tower1Pt >=TOWERBUFSIZE){
					tower1Pt = 0;
				}
				printf("+");
			}
			printf("1-%d\n\r", angle);
			_delay_s(1);
		}
		else if(tower=='2'){
			whileCount++;
			read_ls7366(quadbuf);
			unsigned int quadCnt = 16*256*(int)(quadbuf[1]&0x0F)+256*(int)quadbuf[2]+(int)quadbuf[3];
			unsigned int angle = (int)(((float)quadCnt/(float)8191)*(float)360);

			t2Average = average(&tower2);

			if(angle<(t2Average+TOWERMARGIN) && angle > (t2Average-TOWERMARGIN)){
				tower2[tower2Pt] = angle;
				tower2Pt++;
				if(tower2Pt >=TOWERBUFSIZE){
					tower2Pt = 0;
				}
				printf(".");
			}
			else if(t2Average>=(360-TOWERMARGIN)&&(angle>(t2Average-TOWERMARGIN) || angle<(t2Average-(360-TOWERMARGIN)))){
				tower2[tower2Pt] = angle;
				tower2Pt++;
				if(tower2Pt >=TOWERBUFSIZE){
					tower2Pt = 0;
				}
				printf("*");

			}
			else if(t2Average<TOWERMARGIN && (angle<(t2Average+TOWERMARGIN)||angle>(t2Average+(360-TOWERMARGIN)))){
				tower2[tower2Pt] = angle;
				tower2Pt++;
				if(tower2Pt >=TOWERBUFSIZE){
					tower2Pt = 0;
				}
				printf("+");
			}
			printf("2-%d\n\r", angle);
			_delay_s(1);

		}
		else if(tower=='3'){
			whileCount++;
			read_ls7366(quadbuf);
			unsigned int quadCnt = 16*256*(int)(quadbuf[1]&0x0F)+256*(int)quadbuf[2]+(int)quadbuf[3];
			unsigned int angle = (int)(((float)quadCnt/(float)8191)*(float)360);

			t3Average = average(&tower3);

			if(angle<(t3Average+TOWERMARGIN) && angle > (t3Average-TOWERMARGIN)){
				tower3[tower3Pt] = angle;
				tower3Pt++;
				if(tower3Pt >=TOWERBUFSIZE){
					tower3Pt = 0;
				}
				printf(".");
			}
			else if(t3Average>=(360-TOWERMARGIN)&&(angle>(t3Average-TOWERMARGIN) || angle<(t3Average-(360-TOWERMARGIN)))){
				tower3[tower3Pt] = angle;
				tower3Pt++;
				if(tower3Pt >=TOWERBUFSIZE){
					tower3Pt = 0;
				}
				printf("*");

			}
			else if(t3Average<TOWERMARGIN && (angle<(t3Average+TOWERMARGIN)||angle>(t3Average+(360-TOWERMARGIN)))){
				tower3[tower3Pt] = angle;
				tower3Pt++;
				if(tower3Pt >=TOWERBUFSIZE){
					tower3Pt = 0;
				}
				printf("+");
			}
			printf("3-%d\n\r", angle);
			_delay_s(1);
		}


	}
}



