#include <avr/io.h>
#include "spi.h"
#include "ls7366_quadrature.h"


#define DUMMY_BYTE 0x55

#define WRITE_MDR0 0x88
#define WRITE_MDR1 0x90
#define WRITE_DTR 0x98

#define READ_MDR0 0x48
#define READ_MDR1 0x50
#define READ_CNTR 0x60
#define READ_OTR 0x68
#define READ_STR 0x30

#define CLEAR_MDR0 0x08
#define CLEAR_MDR1 0x10
#define CLEAR_CNTR 0x20
#define CLEAR_STR 0x30

#define LOAD_CNTR 0xE0
#define LOAD_OTR 0xE8


int init_quadrature(char cs){
	// SS, SCK and MOSI
	DDRB |= (1 << PB0) | (1 << PB1) | (1 << PB2);
	// Count 
	DDRC |= (1 << PC2);
	
	char cData[2];
	char readData[3];
	SPI_MasterInit();


	while(readData[2] != 0b01100011){

		// MDR0 init
		cData[0] = WRITE_MDR0;
	//	cData[1] = 0x01;  	// 0b00000001
		// Set mode: negative index pulse + reset on index pulse + 4x quad (four counts per quadrature cycle)
		cData[1] = 0b01100011;//(1 << 6) | (0x02 << 4) | (0x03 << 0);  	//= 0b01100011
		SPI_MasterTransmit(&cData, 2, PB0);
		
		
		readData[0] = READ_MDR0;
		readData[1] = DUMMY_BYTE;
		readData[2] = SPI_MasterTransmit(&readData,2,PB0);
		printf("read0: - %X\n\r", readData[2]);
	
	}
	

	

	// MDR1 init
	cData[0] = WRITE_MDR1;
	cData[1] = 0b00;  	// 0b00000000
	SPI_MasterTransmit(&cData, 2, PB0);
	
	/*cData[0] = READ_MDR1;
	cData[1] = DUMMY_BYTE;
	cData[0] = SPI_MasterTransmit(&cData,2,PB0);
	printf("read1: - %X\n\r", cData[0]);*/
	
	
	// Clear STR
	cData[0] = CLEAR_CNTR;
	SPI_MasterTransmit(&cData, 1, PB0);

	PORTC |= (1 << PC2);	// Turn on counting
	
	
	return 0;
	
}


void read_ls7366(char *quadrature_buffer){
	char cData[1];

	// Chip select on
	PORTB &= ~(1 << PB0);
	
	cData[0] = READ_CNTR;	// Read command
	SPI_MasterTransmit_no_cs(&cData, 1);
	cData[0] = DUMMY_BYTE;
	quadrature_buffer[0] = SPI_MasterTransmit_no_cs(&cData, 1);
	cData[0] = DUMMY_BYTE;
	quadrature_buffer[1] = SPI_MasterTransmit_no_cs(&cData, 1);
	cData[0] = DUMMY_BYTE;
	quadrature_buffer[2] = SPI_MasterTransmit_no_cs(&cData, 1);
	cData[0] = DUMMY_BYTE;
	quadrature_buffer[3] = SPI_MasterTransmit_no_cs(&cData, 1);

	// chip select off
	PORTB |= (1 << PB0);
}
