#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#include "at90can128_driver.h"
#include "uart.h"
/*
Initialisation for CAN bus Recieve Frames on MOb channel 0, and send frames at 
MOb channel 1

*/

can_message can_message_1;

int init_can( void ){
	CANSTMOB = 0x00;

	int mob_number;
	int num_data;
	// Reset CAN
	CANGCON |= (1<<SWRES);
	
	// Reset all mail boxes, the MOb registers has no initial value after RESET
	for ( mob_number = 0; mob_number < 15; mob_number++ ){
		CANPAGE = mob_number << 4; 		// MOb numbers form 0 to 14, 4 MSB in CANPAGE regiser.
										// the rest in this for loop is for the coresponding MOb number.
		CANCDMOB = CONMOB_DISABLE;		// MOb Configuration
		CANSTMOB = 0;					// Clear MOb Status register. 
		CANIDT1 = 0;					// Clear CAN Identifier Tag Registers
		CANIDT2 = 0;
		CANIDT3 = 0;
		CANIDT4 = 0;
		CANIDM1 = 0;					// Clear CAN Identifier Mask Registers
		CANIDM2 = 0;
		CANIDM3 = 0;
		CANIDM4 = 0;
		for ( num_data = 0; num_data < 8; num_data++ ) 
			CANMSG = 0;
	}
	// Bit timing at 125kbps with 16 MHz external crystal (recommanded).
//	CANBT1  = 0x0E; 					// from Table 20-2 in Datasheet
//	CANBT2  = 0x0C;
//	CANBT3  = 0x37;
	
	// Bit timing at 500kbps with 16 MHz external crystal (recommanded).
	CANBT1  = 0x02; 					// from Table 20-2 in Datasheet
	CANBT2  = 0x0C;
	CANBT3  = 0x37;
	
	// Bit timing at 1 Mbps with 16 MHz external crystal (recommanded).
//	CANBT1  = 0x00; 					// from Table 20-2 in Datasheet
//	CANBT2  = 0x0C;
//	CANBT3  = 0x37;
	
	// Start CAN / Enable
	CANGCON |=  0x02;
	
	// Chanel 0 MOb page 0 enable for recieving
	CANPAGE = (0 << 4); 		
	CANSTMOB = 0x00;					// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration

	CANIDT1 = 0x26;						// Identifier TAG (IDT10:0) 11 bits. CANIDT 0x130 - 0x13F;
	CANIDT2 &= ~0xE0;

	CANIDM1 = 0xFE;						// Set mask
	CANIDM2 &= ~0xE0;
	CANIDM4 = 0;

	// Channel 0 configuration
//	CANIDT4 &= ~(1 << RTRMSK);			// Comparison True Forced
	CANIDT4 &= ~0x04;
	
	CANCDMOB |= NUM_BYTES_DATA;			// Reception 8 bytes
	CANCDMOB |= CONMOB_ENABLE_RESEP;	// Recieve
	
// Channel 1
	// Channel 1 MOb page 1 enable for transmitting
	CANPAGE = (1 << 4); 		
	CANSTMOB = 0x00;						// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
	
	// Interrupt configuration
	CANIE2|=0x01;						// IECH0=1
	CANGIE = ((1<<ENRX)|(1<<ENIT));	// Can_Rx & IT enable
	return 0;
}


//int send_can_message( can_message *can_message_1){
int send_can_message( ){
	
	CANPAGE |= (1 << MOBNB0); // Message object 1
	
	CANCDMOB &= 0xE0;
	CANCDMOB |= can_message_1.length;            

	
	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = 0x40;
	CANIDT1 = 0x0E; // 0x71

	CANMSG = can_message_1.bytes[7];
	CANMSG = can_message_1.bytes[6];
	CANMSG = can_message_1.bytes[5];
	CANMSG = can_message_1.bytes[4];

	CANMSG = can_message_1.bytes[3];
	CANMSG = can_message_1.bytes[2];
	CANMSG = can_message_1.bytes[1];
	CANMSG = can_message_1.bytes[0];
	CANSTMOB = 0x00;
	
   
	CANCDMOB |= CONMOB_ENABLE_TRANS;	// Enable transmission
	CANGIT = CANGIT;					// Reset all flags
	CANSTMOB = 0x00;					// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
	return 0;
}

int send_can_message_2( void ){
//	CONMOB = 0x01; // Tx
	can_message can_message_1;
	
	can_message_1.id = 0x123;
	can_message_1.length = 8;
	can_message_1.bytes[0] = 0x00;//read_adc( ADC0 );
	can_message_1.bytes[1] = 0x00;
	can_message_1.bytes[2] = 0xAA;
	can_message_1.bytes[3] = 0x55;
	can_message_1.bytes[4] = 0x43;
	can_message_1.bytes[5] = 0xBB;
	can_message_1.bytes[6] = 0xCC;
	can_message_1.bytes[7] = 0x44;
	
	CANPAGE |= (1 << MOBNB0); // Message object 1
	
	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = 0x10;
	CANIDT1 = 0x50;
	CANMSG = can_message_1.bytes[0];
	CANMSG = can_message_1.bytes[1];
	CANMSG = can_message_1.bytes[2];
	CANMSG = can_message_1.bytes[3];
	CANSTMOB = 0x00;
	CANCDMOB = 0x04;            /* transmit 2 bytes */
	CANCDMOB |= CONMOB_ENABLE_TRANS;     // Enable transmission
                         //CANEN2 |= (1 << 1);       /* channel 1 enable */
    //CANPAGE = (0 << 4);    // select chanel 0
	CANGIT = CANGIT;                            /* reset all flags */

   
	return 0;
}

int send_can_start_message( void ) {
	CANCDMOB &= 0xE0;
	CANCDMOB |= 0;	// Length

	CANPAGE |= (1 << 4); // Message object 1
	
	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = 0x00; // ID 0x10
	CANIDT1 = 0x02; 
	
	CANSTMOB = 0x00;
	CANCDMOB |= CONMOB_ENABLE_TRANS;     // Enable transmission
	CANGIT = CANGIT;                            /* reset all flags */
	CANSTMOB = 0x00;					// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
	//printf("Start message sent!\n\r");
	return 0;

}

int send_can_reset_message( void ) {
	CANCDMOB &= 0xE0;
	CANCDMOB |= 0;	// Length

	CANPAGE |= (1 << 4); // Message object 1
	
	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = 0x20; // ID 0x11
	CANIDT1 = 0x02; 
	
	CANSTMOB = 0x00;
	CANCDMOB |= CONMOB_ENABLE_TRANS;     // Enable transmission
	CANGIT = CANGIT;                            /* reset all flags */
	CANSTMOB = 0x00;					// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
   
	return 0;

}

int send_can_team_color_red_message( void ) {
	CANCDMOB &= 0xE0;
	CANCDMOB |= 0;	// Length

	CANPAGE |= (1 << 4); // Message object 1
	
	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = 0xA0; // ID 0x15
	CANIDT1 = 0x02; 
	
	CANSTMOB = 0x00;
	CANCDMOB |= CONMOB_ENABLE_TRANS;     // Enable transmission
	CANGIT = CANGIT;                            /* reset all flags */
	CANSTMOB = 0x00;					// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
	//printf("Red team color sent!\n\r");
	return 0;

}

int send_can_team_color_blue_message( void ) {
	CANCDMOB &= 0xE0;
	CANCDMOB |= 0;	// Length

	CANPAGE |= (1 << 4); // Message object 1
	
	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = 0xC0; // ID 0x16
	CANIDT1 = 0x02; 
	
	CANSTMOB = 0x00;
	CANCDMOB |= CONMOB_ENABLE_TRANS;     // Enable transmission
	CANGIT = CANGIT;                            /* reset all flags */
	CANSTMOB = 0x00;					// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
	//printf("Blue team color sent!\n\r");
	return 0;
}

int send_low_battery_message( void ) {
	CANCDMOB &= 0xE0;
	CANCDMOB |= 0;	// Length

	CANPAGE |= (1 << 4); // Message object 1
	
	CANPAGE = (1 << 4);   // select chanel 1
	CANIDT2 = 0xC0; // ID 0x06
	CANIDT1 = 0x00; 
	
	CANSTMOB = 0x00;
	CANCDMOB |= CONMOB_ENABLE_TRANS;     // Enable transmission
	CANGIT = CANGIT;                            /* reset all flags */
	CANSTMOB = 0x00;					// Clear MOb Status register. 
	CANCDMOB = CONMOB_DISABLE;			// MOb Configuration
	printf("Low battery sent!\n\r");
	return 0;
}

ISR(SIG_CAN_INTERRUPT1){
	unsigned int id;

	PORTC = 0x55;
	
	CANPAGE = (0 << 4);
	
	if(CANSTMOB << 6){
		id = (((int)(CANIDT2))>>5) + (((int)(CANIDT1))<<3);
		switch(id){
			case(0x134):
				set_bit(PORTB,PB4); //set red led
				//printf("SET RED LED!\n\r");
				break;
			case(0x135):
				set_bit(PORTB,PB5); //set blue led
				//printf("SET BLUE LED!\n\r");
				break;		
			default:
				break;
		}
	}
	CANPAGE = (0 << 4);
	CANSTMOB = 0x00;
	CANEN2 |= (1 << 0);
	CANCDMOB = NUM_BYTES_DATA;			// Reception 8 bytes
	CANCDMOB |= CONMOB_ENABLE_RESEP;	// Recieve
	CANGIT = CANGIT;
}


