/***************************************************************************
 *   Copyright (C) 2007 by Lars Vråle   *
 *   lars.vrale@gmail.com   *
 *                                                                         *
 ***************************************************************************/

#include "position.h"
#include <math.h>

CvMat M_l= cvMat(3,3,CV_32FC1,new float[9]);
CvMat t_l= cvMat(3,1,CV_32FC1,new float[9]);


void initCameraMatrix(){
	
	int temp;
	/*
Focal Length:          fc = [ 546.77136   540.95011 ] ± [ 3.53208   2.80428 ]
Principal point:       cc = [ 315.44049   254.32692 ] ± [ 5.41923   4.74782 ]
Skew:             alpha_c = [ 0.00000 ] ± [ 0.00000  ]   => angle of pixel axes = 90.00000 ± 0.00000 degrees
Distortion:            kc = [ -0.19278   0.22373   -0.00042   0.00295  0.00000 ] ± [ 0.02006   0.04579   0.00233   0.00259  0.00000 ]
Pixel error:          err = [ 0.52454   0.38386 ]
	
	Intrisik Matrise (beskriver krumning på linsen osv):
	
	CvMat KK_l = cvMat(3,3,CV_32FC1,new float[9]);
	KK_l.data.fl[0] = 546.56936;         KK_l.data.fl[1] = 0.0;               KK_l.data.fl[2] = 319.50000;
	KK_l.data.fl[3] = 0.0;               KK_l.data.fl[4] = 539.37032;         KK_l.data.fl[5] =239.50000;
	KK_l.data.fl[6] = 0.0;               KK_l.data.fl[7] = 0.0;               KK_l.data.fl[8] = 1.0;
*/
	
	CvMat intrisic_invertert = cvMat(3,3,CV_32FC1,new float[9]);
	
	intrisic_invertert.data.fl[0] = 0.001791;     intrisic_invertert.data.fl[1] = 0;         intrisic_invertert.data.fl[2] = -0.565929;
	intrisic_invertert.data.fl[3] = 0;            intrisic_invertert.data.fl[4] = 0.001804;  intrisic_invertert.data.fl[5] = -0.444123;
	intrisic_invertert.data.fl[6] = 0;            intrisic_invertert.data.fl[7] = 0;         intrisic_invertert.data.fl[8] = 1;
	/* ------------------------------------ */
	
	/*
	Eksterne parametre:
	Translation vector: Tc_ext = [ -57.185790 	 -112.820336 	 259.975128 ]
	Rotation vector:   omc_ext = [ 1.700049 	 1.729702 	 0.783064 ]
	Rotation matrix:    Rc_ext = [ -0.015296 	 0.656442 	 0.754221
                               0.999873 	 0.013346 	 0.008662
                               -0.004380 	 0.754258 	 -0.656563 ]
	Pixel error:           err = [ 0.77645 	 0.39189 ]	
	*/
	
	CvMat Rc_ext_l = cvMat(3,3,CV_32FC1,new float[9]);
	Rc_ext_l.data.fl[0] =  -0.015296;   Rc_ext_l.data.fl[1] =  0.656442;    Rc_ext_l.data.fl[2] =  0.754221;
	Rc_ext_l.data.fl[3] =   0.999873;   Rc_ext_l.data.fl[4] =  0.013346;    Rc_ext_l.data.fl[5] =  0.008662;
	Rc_ext_l.data.fl[6] =  -0.004380;   Rc_ext_l.data.fl[7] =  0.754258;    Rc_ext_l.data.fl[8] = -0.656563;
	
	CvMat Tc_ext_l = cvMat(3,1,CV_32FC1,new float[9]);
	Tc_ext_l.data.fl[0] = -57.185790;
	Tc_ext_l.data.fl[1] = -112.829336;
	Tc_ext_l.data.fl[2] = 259.975128;
	
	
	/* Calculations combining extrinsic and intrinsic for left camera*/
	CvMat Rc_inv_l = cvMat(3,3,CV_32FC1,new float[9]);
	temp = (int)cvInvert(&Rc_ext_l, &Rc_inv_l,CV_LU);
	
	M_l = cvMat(3,3,CV_32FC1,new float[9]);
	cvMatMulAdd(&Rc_inv_l, &intrisic_invertert, 0, &M_l);
	
	t_l = cvMat(3,1,CV_32FC1,new float[3]);
	cvMatMulAdd(&Rc_inv_l, &Tc_ext_l, 0, &t_l);
	/* ----------------------------------------------- */

}


CvPoint getWorldPosFromPixel(CvPoint pixel, int obj){
	int px = pixel.y;
	int py = pixel.x;
	CvPoint result;
	int zw;
		zw = 40;  //radiusen til ballen. ca.
	int zp =(int) ((zw+t_l.data.fl[2])/(M_l.data.fl[6]*px + M_l.data.fl[7]*py + M_l.data.fl[8]));
	int xw =(int) (M_l.data.fl[0]*px*zp + M_l.data.fl[1]*py*zp + M_l.data.fl[2]*zp-t_l.data.fl[0]);
	int yw = (int) (M_l.data.fl[3]*px*zp + M_l.data.fl[4]*py*zp + M_l.data.fl[5]*zp-t_l.data.fl[1]);
	
	
	result.x = yw; //y er fremover i rotasjonsmatrisens koordinatsystem
	result.y = xw;

	/* litt justering i forhold til robotsenter */
	result.x +=177;
	result.y -=110;
	
	return result;
}

