
#include "gatewayMotorControl.h"

unsigned int lastQuadLeft = 0;
unsigned int lastQuadRight = 0;

int Qright = 0;
int Qleft = 0;

int quad_init(){
	TPCANMsg canSendStruct;
	canSendStruct.ID = 0x400;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 0x00;
	return 0;

}



void gatewayUpdateBeaconAngles(pmq_message *pm){


	int n;
	TPCANMsg canSendStruct;
	beacon_angles angles, *angles_pt;
	angles_pt = &angles;
	memcpy((void *)(angles_pt),( void * )pm->data, sizeof(beacon_angles));

	if(angles.b1 < 0){
		angles.b1 += 180;
	}
	if(angles.b2 < 0){
		angles.b2 += 180;
	}
	if(angles.b3 < 0){
		angles.b3 += 180;
	}

	canSendStruct.ID = CAN_ADDR_POSITIONING_UPDATE_ANGLES;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 6;
	canSendStruct.DATA[0] = (int)angles.b1&0xFF;
	canSendStruct.DATA[1] = ((int)angles.b1)>>8;
	canSendStruct.DATA[2] = (int)angles.b2&0xFF;
	canSendStruct.DATA[3] = ((int)angles.b2)>>8;
	canSendStruct.DATA[4] = (int)angles.b3&0xFF;
	canSendStruct.DATA[5] = ((int)angles.b3)>>8;
	printf("BeaconUpdate1: %d, %d, %d\n\r", (int)angles.b1, ((int)angles.b1&0xFF)>>8, (int)angles.b3);

	n = can_send_message( &canSendStruct );

}


void gatewayReceiveMotorControl(pmq_message *pm){

	int n;
	TPCANMsg canSendStruct;
	motor_control_message motor_control_m, *motor_control_mes;
	motor_control_mes = &motor_control_m;
	memcpy((void *)(motor_control_mes),( void * )pm->data, sizeof(motor_control_message));

	canSendStruct.ID = CAN_ADDR_MOTOR_MAIN;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 0x06;
//printf("MOTOR: %d, %d\n", motor_control_mes->m1, motor_control_mes->m2);
//motor_control_mes->m1 = -255;
//motor_control_mes->m2 = -255;

	if(motor_control_mes->m1 > 0){
		canSendStruct.DATA[0] = 0xFF;
		canSendStruct.DATA[1] = (0xFF & motor_control_mes->m1);
		canSendStruct.DATA[2] = (motor_control_mes->m1 >> 8);
	}
	else{
		canSendStruct.DATA[0] = 0x00;
		canSendStruct.DATA[1] = -(0xFF & motor_control_mes->m1);
		canSendStruct.DATA[2] = ((-motor_control_mes->m1) >> 8);
	}


	if(motor_control_mes->m2 > 0){
		canSendStruct.DATA[3] = 0x00;
		canSendStruct.DATA[4] = (0xFF & motor_control_mes->m2);
		canSendStruct.DATA[5] = (motor_control_mes->m2 >> 8);
	}
	else{
		canSendStruct.DATA[3] = 0xFF;
		canSendStruct.DATA[4] = (0xFF & (-motor_control_mes->m2));
		canSendStruct.DATA[5] = ((-motor_control_mes->m2) >> 8);
	}

	//printf("speed: %d\n\r", (int)((canSendStruct.DATA[4])|(canSendStruct.DATA[5]<<8)));
	//printf("CAN-motor: %d , %d , %d , %d, %d, %d \n", canSendStruct.DATA[0], canSendStruct.DATA[1], canSendStruct.DATA[2], canSendStruct.DATA[3], canSendStruct.DATA[4], canSendStruct.DATA[5], canSendStruct.DATA[6]);

	n = can_send_message( &canSendStruct );

}

int gatewayRightReturnQuad(__u8 *mdata, MessageHandler *navsysMessageHandler){
	int data;
	odometry_observation obs_struct;

	data = ((mdata[0]<<24)|(mdata[1]<<16)|(mdata[2]<<8)|mdata[3]);
	if(data < 0x0000FFFF){
	}
	else{
		data = 0xFFFFFFFF - data;
	}
	obs_struct.right = data;

	obs_struct.left = 0;

	Qright += data;
#ifdef PRINT_DEBUG
	cout << " Quadrature RIGHT " << data << "\n";
	cout << " Quadrature RIGHT total " << Qright << "\n";
#endif
	//printf("Quadrature RIGHT total: %i\n", Qright);

	navsysMessageHandler->sendMessage((char *)&obs_struct, (int)sizeof(odometry_observation), MINOR_ODOMETRY_OBSERVATION, PRIORITY_VERY_HIGH );

return 0;

}

int gatewayLeftReturnQuad(__u8 *mdata, MessageHandler *navsysMessageHandler){
	int data;
	odometry_observation obs_struct;

	data = ((mdata[0]<<24)|(mdata[1]<<16)|(mdata[2]<<8)|mdata[3]);

	if(data > 0x0000FFFF){
		// Fremmover OK
		data = 0xFFFFFFFF - data;

	}
	else{
		data = -data;
	}
	obs_struct.right = 0;
	obs_struct.left = data;


	Qleft += data;
#ifdef PRINT_DEBUG
	cout << " Quadrature LEFT " << data << "\n";
	cout << " Quadrature LEFT total " << Qleft << "\n";
#endif

	//printf("Quadrature LEFT total: %i\n", Qleft);

	navsysMessageHandler->sendMessage((char *)&obs_struct, (int)sizeof(odometry_observation), MINOR_ODOMETRY_OBSERVATION, PRIORITY_VERY_HIGH );

	return 0;

}



int gatewayReturnDualQuad(__u8 *mdata, MessageHandler *navsysMessageHandler){
	int dataLeft, dataRight;
	odometry_observation obs_struct;

	dataLeft = ((mdata[0]<<24)|(mdata[1]<<16)|(mdata[2]<<8)|mdata[3]);
	dataRight = ((mdata[4]<<24)|(mdata[5]<<16)|(mdata[6]<<8)|mdata[7]);

	if(dataLeft > 0x0000FFFF){
		// Fremmover OK
		dataLeft = 0xFFFFFFFF - dataLeft;
	}
	else{
		dataLeft = -dataLeft;
	}

	if(dataRight > 0x0000FFFF){
		// Fremmover OK
		dataRight = 0xFFFFFFFF - dataRight;
	}
	else{
		dataRight = -dataRight;
	}
	dataRight = -dataRight;
	obs_struct.right = dataRight;

	dataLeft = -dataLeft;
	obs_struct.left = dataLeft;



	navsysMessageHandler->sendMessage((char *)&obs_struct, (int)sizeof(odometry_observation), MINOR_ODOMETRY_OBSERVATION, PRIORITY_VERY_HIGH );


	return 0;
}

