// INCLUDES
#include <pthread.h>
#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <ctype.h>
#include <unistd.h>   // exit
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>    // O_RDWR

#include <libpcan.h>
#include <pcan.h>

#include <math.h>

#include "main.h"
#include "can_bus.h"
#include "gatewayMotorControl.h"
#include "gatewayCollisionDetection.h"
#include "gatewayCommands.h"
#include "gatewayBallSorting.h"


MessageHandler navsysMessageHandler(PROXY_ID, PMQ_PROXY, PMQ_NAVSYS, true);
MessageHandler plansysMessageHandler(PROXY_ID, PMQ_PROXY, PMQ_PLANSYS, true);


int dirleft = 0;
int dirright = 0;
int running = 1;


//Method for CAN server thread
extern "C" void* can_server(void *arg){
	TPCANMsg canReceiveStruct;
	while(running){
		can_recieve_message( &canReceiveStruct, &navsysMessageHandler, &plansysMessageHandler );

	}
	return arg;
}
int flag = 0;
			TPCANMsg canSendStruct2;
//Method for CAN server thread
extern "C" void* can_send(void *arg){



			canSendStruct2.ID = 0x180;
			canSendStruct2.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct2.LEN = 3;
			canSendStruct2.DATA[0] = 0x00;
			canSendStruct2.DATA[1] = 0;
			canSendStruct2.DATA[2] = 0;
			while(1){
				flag = 1;
				if(canSendStruct2.DATA[0]){
					canSendStruct2.DATA[0] = 0x00;
				}
				else{
					canSendStruct2.DATA[0] = 0x58;
				}
				usleep(500000);
			}
		return arg;
}


//Startup method
void gatewayReceiveGoCommand(pmq_message *pm) {
	speed_limit max_speed_limit;
    	max_speed_limit.u_b = 10.0;
	max_speed_limit.r_b = 10.0;
    	navsysMessageHandler.sendMessage( (char *)&max_speed_limit, (int)sizeof(speed_limit), MINOR_NAVSYS_SET_SPEED_LIMIT, PRIORITY_HIGHEST );

	//Initializing voltage alarm
	TPCANMsg canSendStruct;
	canSendStruct.ID = CAN_RESET_VOLTAGE_ALARM;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 0x00;
	can_send_message( &canSendStruct );
	usleep(10000);

	canSendStruct.ID = CAN_ADDR_RESET_SORTER;
	//can_send_message( &canSendStruct );

        setStartTimerCollision();
}


//Shutdown method
void gatewayReceiveSTOPCommand(pmq_message *pm) {
	speed_limit max_speed_limit;
    	max_speed_limit.u_b = 0.0;
    	max_speed_limit.r_b = 0.0;
    	navsysMessageHandler.sendMessage( (char *)&max_speed_limit, (int)sizeof(speed_limit), MINOR_NAVSYS_SET_SPEED_LIMIT, PRIORITY_HIGHEST );

	//Disabling ball sorter
	TPCANMsg canSendStruct;
	canSendStruct.ID = CAN_ADDR_STOP_SORTER;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 0x00;
	can_send_message( &canSendStruct );

	cout << " STOPPING ALL ACTUATORS " << endl;
}

void gatewayReceiveShutdownCommand(pmq_message *pm) {
	speed_limit max_speed_limit;
    	max_speed_limit.u_b = 0.0;
    	max_speed_limit.r_b = 0.0;
    	navsysMessageHandler.sendMessage( (char *)&max_speed_limit, (int)sizeof(speed_limit), MINOR_NAVSYS_SET_SPEED_LIMIT, PRIORITY_HIGHEST );

	//Disabling ball sorter
	TPCANMsg canSendStruct;
	canSendStruct.ID = CAN_ADDR_STOP_SORTER;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 0x00;
	can_send_message( &canSendStruct );

	cout << " STOPPING ALL ACTUATORS " << endl;
	running = 0;
	usleep(1000000);
	system("sudo shutdown -h now");
}

int main() {

	int n;
	cout << "Starting PROXY " << endl;
	errno = 0;
	can_init();
	quad_init();

	signal(SIGINT, catch_int);

	signal(SIGHUP, catch_sighup);

	TPCANMsg canSendStruct;

	//Create message queues
	navsysMessageHandler.addSubMessageHandler(gatewayReceiveMotorControl, MAJOR_STRUCT, MINOR_MOTOR_CONTROL_MESSAGE);
	navsysMessageHandler.addSubMessageHandler(gatewayUpdateBeaconAngles, MAJOR_STRUCT, MINOR_NAVSYS_UPDATE_BEACON_ANGLES);

	plansysMessageHandler.addSubMessageHandler(gatewayReceiveCommands, MAJOR_STRUCT, MINOR_PROXY_COMMAND);
	plansysMessageHandler.addSubMessageHandler(gatewayReceiveCommand, MAJOR_COMMAND, MINOR_SET_LED_TEAM_COLOR_RED);
	plansysMessageHandler.addSubMessageHandler(gatewayReceiveCommand, MAJOR_COMMAND, MINOR_SET_LED_TEAM_COLOR_BLUE);
	plansysMessageHandler.addSubMessageHandler(gatewayReceiveGoCommand, MAJOR_COMMAND, MINOR_CMD_SYSTEM_GO);
	plansysMessageHandler.addSubMessageHandler(gatewayReceiveSTOPCommand, MAJOR_COMMAND, MINOR_CMD_SYSTEM_STOP);
	plansysMessageHandler.addSubMessageHandler(gatewayRequestDistanceMeasurement, MAJOR_COMMAND, MINOR_REQUEST_MEASUREMENTS);
	plansysMessageHandler.addSubMessageHandler(gatewayFlashSensorBordLeds, MAJOR_STRUCT, MINOR_FLASH_SENSOR_BOARD_LED);
	/**
	 * NEW CODE STARTS HERE
	 */
	plansysMessageHandler.addSubMessageHandler(gatewayReceiveShutdownCommand, MAJOR_COMMAND, MINOR_CMD_SHUTDOWN_SYSTEM);
	plansysMessageHandler.addSubMessageHandler(gatewayReleaseBalls, MAJOR_STRUCT, MINOR_RELEASE_BALLS);
	plansysMessageHandler.addSubMessageHandler(gatewayColorAck, MAJOR_STRUCT, MINOR_COLOR_ACK);
	plansysMessageHandler.addSubMessageHandler(gatewaySetArms, MAJOR_STRUCT, MINOR_SET_ARMS);
	/**
	 * NEW CODE ENDS HERE
	 */
	pthread_t thread2, thread3;

	// Start CAN-server
	pthread_create(&thread2, NULL, can_server, NULL);
	//pthread_create(&thread3, NULL, can_send, NULL);

	/*cout << "Enter color (1,2): 1. Red, 2. Blue: " << endl;
	int color = 0;
	cin >> color;
	switch (color)
	{
		case 1:
			gatewaySendCommand(&plansysMessageHandler, MINOR_CMD_SET_TEAM_COLOR_RED, PRIORITY_HIGHEST);
			cout << "CAN_CMD_SET_TEAM_COLOR_RED" << endl;
			break;
		case 2:
			gatewaySendCommand(&plansysMessageHandler, MINOR_CMD_SET_TEAM_COLOR_BLUE, PRIORITY_HIGHEST);
			cout << "CAN_CMD_SET_TEAM_COLOR_BLUE" << endl;
			break;
		default:
			cout << "Wrong color!" << endl;
	}
	usleep(1000000);*/
	gatewayInitTimeMessage(&plansysMessageHandler);








/*			canSendStruct.ID = 0x180;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 3;
			canSendStruct.DATA[0] = 0x58;
			canSendStruct.DATA[1] = 0;
			canSendStruct.DATA[2] = 0;
			n = can_send_message( &canSendStruct );
*/



			canSendStruct.ID = CAN_ADDR_POSITIONING_UPDATE_ANGLES;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 6;
			canSendStruct.DATA[0] = (195&0xFF);
			canSendStruct.DATA[1] = (195>>8);
			canSendStruct.DATA[2] = (200&0xFF);
			canSendStruct.DATA[3] = (200>>8);
			canSendStruct.DATA[4] = (100&0xFF);
			canSendStruct.DATA[5] = (100>>8);
			canSendStruct.DATA[6] = 0xE7;
			//n = can_send_message( &canSendStruct );




	//main loop
	while(running){
		//Read from quadrature
		if( MOTORDRIVER_QUAD ) {

			usleep(5000);
			usleep(5000);

			canSendStruct.ID = CAN_ADDR_MOTOR_LEFT_READ_QUAD; //0x200;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 0x00;
			n = can_send_message( &canSendStruct );

			usleep(5000);
			usleep(5000);

			canSendStruct.ID = CAN_ADDR_MOTOR_RIGHT_READ_QUAD; //0x201;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 0x00;
			n = can_send_message( &canSendStruct );

			/*if(flag){
				usleep(5000);
				flag = 0;
				can_send_message( &canSendStruct2 );
				usleep(5000);

			}*/


			/*canSendStruct.ID = CAN_ADDR_POSITIONING_BEACON_REQUEST;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 0x00;
			n = can_send_message( &canSendStruct );
			usleep(5000);
			usleep(5000);
			canSendStruct.ID = CAN_ADDR_POSITIONING_UPDATE_ANGLES;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 6;
			canSendStruct.DATA[0] = 100;
			canSendStruct.DATA[1] = 0;
			canSendStruct.DATA[2] = 254;
			canSendStruct.DATA[3] = 1;
			canSendStruct.DATA[4] = 0xFF;
			canSendStruct.DATA[5] = 0x0;
			canSendStruct.DATA[6] = 0xE7;
			canSendStruct.DATA[7] = 0xA8;
			n = can_send_message( &canSendStruct );*/

		}
		else{
			usleep(50000);
			canSendStruct.ID = CAN_ADDR_READ_DUAL_QUAD; //0x202;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 0x00;

			n = can_send_message( &canSendStruct );

		}

	}

} // main


void catch_sighup(int signum){
	signal(SIGHUP, catch_sighup);

}

//signal handler
void catch_int(int sig_num){

	signal(SIGINT, catch_int);
	TPCANMsg canSendStruct;

	cout << "Ctrl+c stopping robot " << endl;

	canSendStruct.ID = CAN_ADDR_MOTOR_MAIN;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 0x04;
	canSendStruct.DATA[0] = 0x00;
	canSendStruct.DATA[1] = 0x00;
	canSendStruct.DATA[2] = 0x00;
	canSendStruct.DATA[3] = 0x00;

	can_send_message( &canSendStruct );

	exit(1);

}

