#ifndef LOGSYS_OSTREAM_H
#define LOGSYS_OSTREAM_H

// From qdebug.h (Trolltech forums).

#include <iostream>
#include <streambuf>
#include <string>

#include "MessageHandler.h"

class LogSysOstream : public std::basic_streambuf<char>
{
public:
	LogSysOstream(std::ostream &stream, void (*sendString)(std::string)) : m_stream(stream)
	{
		this->sendString = sendString;
		m_old_buf = stream.rdbuf();
		stream.rdbuf(this);
	}
	~LogSysOstream()
	{
		// output anything that is left
		if (!m_string.empty())
			sendString(m_string.c_str());

		m_stream.rdbuf(m_old_buf);
	}

protected:
	virtual int_type overflow(int_type v)
	{
		if (v == '\n')
		{
			sendString(m_string.c_str());
			m_string.clear();
		}
		else
			m_string.push_back(v);

		return v;
	}

	virtual std::streamsize xsputn(const char *p, std::streamsize n) 
	{
		m_string.append(p, p + n);

		int pos = 0;
		while (pos != std::string::npos)
		{
			pos = m_string.find('\n');
			//if(pos >10) pos = 10;
			if (pos != std::string::npos)
			{
				std::string tmp(m_string.begin(), m_string.begin() + pos);
				sendString(tmp.c_str());
				m_string.erase(m_string.begin(), m_string.begin() + pos + 1);
			}
		}

		return n;
	}

private:
	void (*sendString)(std::string);
	std::ostream &m_stream;
	std::streambuf *m_old_buf;
	std::string m_string;
};

#endif
