#ifndef MESSAGE_HANDLER_H
#define MESSAGE_HANDLER_H

#include <iostream>
#include <string>
#include <stdexcept>
#include <errno.h>
#include <vector>
#include <ostream>

#ifdef __cplusplus
extern "C"
{
#endif
#include "posix_message_queue.h"
#include "posix_message_config.h"
#ifdef  __cplusplus
}
#endif

#include "observation_structs.h"
//#include "LogSysOstream.h"


using namespace std;


// Handle messages to all MessageHandler objects in this scope.
static void handleMessages(int i);


// Debug message method (does the same as << std:string )
		void sendDebugMessage(const string s);


class MessageHandler {
    public:
		/**
		* Creates a MessageHandler object that receives messages on receiveFromQueue and
		* sends messages on sendToQueue. The debug ID is important, as it identifies the
		* sender in the log system.
		*
		* Queue names (that is the files) must start with prefix / and be >=3 characters
		* in length.
		*
		* Two modes of operation is available; interrupt driven and simply use of the
		* handleAllMessagesNow() method.
		*/
        MessageHandler(int debugId, string receiveQueueName, string sendQueueName, bool interruptDrivenOperation) throw(invalid_argument);

		// Add a message handler subclass to handle a specific message id.
		void addSubMessageHandler(void(*handlingfun)(pmq_message *), int major, int minor);
		
		// Send a simple text string.
		void sendMessage(string s);

		// Send a simple object
		void sendMessage(char *s, int size, int minor, int priority);

		// Send a command
		void sendCommand(int command, int priority);

		// Send a voice message via log system
		void say(string s);

		// Handle messages for this object instance
		void handleMessage();

		// Do sequentially receive and message handling with respect to priority
		void handleAllMessagesNow();

		// Returns true if interrupt driven operation is chosen.
		bool isInterruptDriven();


		// Debug message shiftout
		const MessageHandler& operator << ( const std::string& ) const;
		// Debug message shiftout int
		const MessageHandler& operator << ( const int ) const;
		// Debug message shiftout double
		const MessageHandler& operator << ( const double ) const;
		// Debug message shiftout char
		const MessageHandler& operator << ( const char ) const;
		
		// Log message (sends struct data to log system).
		void sendLogMessage(char *s, int size, int minor, int priority);
		
		int getDebugId() {
			return debugId;
		}

		posix_message_queue logQueueDesc;

    private:
		string logQueueName;
		//LogSysOstream *logSysOstream;

		//int type;
		bool connectionActive;
		bool interruptDriven;		
		posix_message_queue receiveQueueDesc, sendQueueDesc;
		string receiveQueueName, sendQueueName;
		int debugId;
		int handlerId;
};



#endif
