/**
* Coordinate transformation factory class.
*
* Kristian M. Knausgård <kmk@wait.no> - 2007
*/

#ifndef COORDINATE_H
#define COORDINATE_H


//template <class double>	// Using templates to accept both double and int

#include <iostream>
#include <ostream>

using namespace std;

class Coordinate {
public:
	Coordinate() {};
	~Coordinate() {};
};


class AngleEuler
{
public:
	AngleEuler(const double phi, const double theta, const double psi) {
		this->phi = phi;
		this->theta = theta;
		this->psi = psi;
	}

	// Overloaded << operator using friend for private data access.
	friend ostream& operator<<(ostream& output, const AngleEuler& ae) {
    	output << "Euler angle: " << "(" << ae.phi << ", " << ae.theta <<", " << ae.psi <<")";
    	return output;  // for multiple << operators.
	}
	const double getPhi() { return phi; }
	const double getTheta() { return theta; }
	const double getPsi() { return psi; }
	void setPhi(const double phi) { this->phi = phi; }
	void setTheta(const double theta) { this->theta = theta; }
	void setPsi(const double psi) { this->psi = psi; }
	void getValue(double *ae) {
		ae[0] = phi;
		ae[1] = theta;
		ae[2] = psi;
	}
	
private:
	double phi, theta, psi;	// = Roll, Pitch, Yaw
};


class CoordLocal{ //  : public Coordinate 
public:
	CoordLocal() {
		x = 0;
		y = 0;
		z = 0;
	}
	CoordLocal(const double x, const double y, const double z) {
		this->x = x;
		this->y = y;
		this->z = z;
	}
	CoordLocal(const double *cl) {
		this->x = cl[0];
		this->y = cl[1];
		this->z = cl[2];
	}
	// Overloaded << operator using friend for private data access.
	friend ostream& operator<<(ostream& output, const CoordLocal& cl) {
    	output << "Local coordinate: " << "(" << cl.x << ", " << cl.y <<", " << cl.z <<")";
    	return output;  // for multiple << operators.
	}
/*
	CoordLocal(const CoordLocal &cl) {
		x=cl->x;
		y=cl->y;
		z=cl->z;
	}*/
	const double getX() { return x; }
	const double getY() { return y; }
	const double getZ() { return z; }
	void setX(const double x) { this->x = x; }
	void setY(const double y) { this->y = y; }
	void setZ(const double z) { this->z = z; }
	void getValue(double *cl) {
		cl[0] = x;
		cl[1] = y;
		cl[2] = z;
	}
private:
	double x,y,z;
};


class CoordBody {
public:
	CoordBody() {
		x = 0;
		y = 0;
		z = 0;
	}
	CoordBody(const double x, const double y, const double z) {
		this->x = x;
		this->y = y;
		this->z = z;
	}
	CoordBody(const double *cb) {
		this->x = cb[0];
		this->y = cb[1];
		this->z = cb[2];
	}
	// Overloaded << operator using friend for private data access.
	friend ostream& operator<<(ostream& output, const CoordBody& cb) {
    	output << "BODY coordinate: " << "(" << cb.x << ", " << cb.y <<", " << cb.z <<")";
    	return output;  // for multiple << operators.
	}
	/*CoordBody(const CoordBody &cb) {
		x=cb->x;
		y=cb->y;
		z=cb->z;
	}*/
	const double getX() { return x; }
	const double getY() { return y; }
	const double getZ() { return z; }
	void setX(const double x) { this->x = x; }
	void setY(const double y) { this->y = y; }
	void setZ(const double z) { this->z = z; }
	void getValue(double *cb) {
		cb[0] = x;
		cb[1] = y;
		cb[2] = z;
	}
private:
	double x,y,z;
};


#endif
