#include "NavsysXMLParser.h"

#include "Controller.h"

#include <string>
#include <sstream>
#include <iostream>
#include <exception>

template <class T>
bool convertString(T& t, 
                 const std::string& s, 
                 std::ios_base& (*f)(std::ios_base&))
{
  std::istringstream iss(s);
  return !(iss >> f >> t).fail();
}


NavsysXMLParser::NavsysXMLParser(string fileName) {
	parsed = false;
	this->fileName = fileName;

	// Make sure correct shared library is used (from libxml doc).
    LIBXML_TEST_VERSION
}

NavsysXMLParser::~NavsysXMLParser() {
	// Clean up.
	//if(doc) xmlFreeDoc(doc); Hvorfor krasjer der her??? TODO!

	parsed = false;
}



int NavsysXMLParser::parse(string rootNodeName) {
	//int ret = 0;

	doc = xmlParseFile(fileName.c_str());


	if (doc == NULL ) {
		cout << "Document not parsed successfully -- does the " << fileName << " exist? If it does, clean up your XML!" << endl;
		parsed = false;
		return -1;
	}

	rootNode = xmlDocGetRootElement(doc);
	if (rootNode == NULL) {
		cout << "The XML document " << fileName << " is empty." << endl;
		xmlFreeDoc(doc);
		parsed = false;
		return -1;
	}

	if (xmlStrcmp(rootNode->name, (const xmlChar *) rootNodeName.c_str())) {
		cout << "Wrong document type, root node != " << rootNodeName << endl;
		xmlFreeDoc(doc);
		parsed = false;
		return -1;
	}

	parsed = true;
	return 1;	
}


xmlNodePtr NavsysXMLParser::getFirst(string key, xmlNodePtr rootNode) {
	xmlNodePtr iNode;
	//cout << "getFirst(\"" << key << "\", " << (int)rootNode << ")" << endl;

	if(rootNode == NULL) return NULL;
	

	for (iNode = rootNode; iNode; iNode = iNode->next) {
        switch(iNode->type) {
			case XML_ELEMENT_NODE:
				//cout << "Element node: " << iNode->name << endl; // << ", " << xmlNodeGetContent(iNode) << endl;
				if(strcmp((char *)(iNode->name), key.c_str())==0) {
	            	cout << "\nFound first "<< key <<" element." << endl;
					cout << "<" << iNode->name << ">" << xmlNodeGetContent(iNode) << "</" << iNode->name << ">" << endl;
					return iNode;
				} else {
					xmlNodePtr tmp = getFirst(key, iNode->children);
					if(tmp!=NULL)return tmp;
				}
			break;
			case XML_TEXT_NODE:
				//cout << "Text node: " << iNode->name << ", " << (char *)xmlNodeListGetString(doc, rootNode, 1) << endl;
				if((!xmlStrcmp(rootNode->name, (const xmlChar *)key.c_str()))) {
	            	cout << "\nFound first "<< key <<" element." << endl;
					cout << "<" << iNode->name << ">" << xmlNodeGetContent(iNode) << "</" << iNode->name << ">" << endl;
					return iNode;
				} else {
					xmlNodePtr tmp = getFirst(key, iNode->children);
					if(tmp!=NULL)return tmp;
				}
			break;
			default:
				cout << "NavsysXMLParser: Ignoring node type: " << (int)iNode->type << endl;
		}
	}
	return NULL;
}

/*
xmlNodePtr NavsysXMLParser::getFirst(const char *str, xmlNode *rootNode) {
	xmlNodePtr cur;
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		if ((!xmlStrcmp(cur->name, (const xmlChar *)"controller"))){
		        parseStory (doc, cur);
		}
		cur = cur->next;
	}
}*/

/*
xmlNodePtr NavsysXMLParser::getFirst(string str, xmlDocPtr doc, xmlNodePtr cur) {
	xmlChar *key;
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		if ((!xmlStrcmp(cur->name, (const xmlChar *)str.c_str()))) {
			key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
			printf("keyword: %s\n", key);
			xmlFree(key);
			return cur;			
		}
		cur = cur->next;
	}
	return NULL;
}

*/

int NavsysXMLParser::getAsInt(string key, xmlNode *rootNode) {
	int value = 0;

	if(rootNode == NULL) {
		std::cout << "NavsysXMLParser: Can't get integer value from a node pointer that's NULL!" << endl;
		//throw exception();
		return 0;
	}
	//cout << "Looking for " << key << " at " << (int)rootNode << endl;
	xmlNodePtr theNode = getFirst(key.c_str(),  rootNode);
	//cout << "Nothing.. " << (int)theNode << endl;
	if(convertString<int>(value, (const char * )xmlNodeGetContent(theNode), std::dec)) {
		return value;
	} else {
		std::cout << "NavsysXMLParser: convertString() failed to get controller " << key << "! Check your XML controller definition!" << std::endl;
		//throw exception();
		return 0;
	}
}

vector<Feature> NavsysXMLParser::extractFeatures() {
	vector<Feature> features;
	if(parsed) {
		//xmlChar *tmp;
		xmlNodePtr featureNode;
		//xmlNodePtr tmpNode;
		
		// Fetch first feature type node.
		featureNode = getFirst("feature", rootNode);

		// Walk through all feature nodes, and populate features vector with new Feature objects:
		/*for (featureNode; featureNode; featureNode = featureNode->next) {
			//Feature f("Uten navn");
			printf("<%s>%s</%s>\n", featureNode->name, xmlNodeGetContent(featureNode), featureNode->name);
		}*/


	} else {
		return features;
	}
	printf("ERROR (NavsysXMLParser.cpp:177): Should never get here. \n\r");
	return features;
}

vector<Controller *> NavsysXMLParser::extractControllers() {
	vector<Controller *> controllers;
	if(parsed) {
		//xmlChar *tmp;
		xmlNodePtr controllerNode;
		//xmlNodePtr tmpNode;
		



		// Fetch first feature type node.
		controllerNode = getFirst("controller", rootNode);

		// Walk through all feature nodes, and populate features vector with new Feature objects:
		int i = 0;
		for (controllerNode; controllerNode; controllerNode = controllerNode->next) {
			if(!xmlStrcmp(controllerNode->name, (const xmlChar *) "controller")) {
				// OK
			} else {
				continue;
			}
			cout << "--------- Parsing controller no: " << i++ << " ---------" << endl;			

			// Parse data
			int id, p, i, d, saturation;

			const char *tmp = (const char *)xmlNodeGetContent(getFirst("name", controllerNode));
			string name = "Unnamed";
			
			try	{
				if(tmp!=NULL) name = tmp;
				id = getAsInt("id", controllerNode);
				p = getAsInt("p", controllerNode);
				i = getAsInt("i", controllerNode);
				d = getAsInt("d", controllerNode);
				saturation = getAsInt("antiwindup", controllerNode);
			} catch (exception& e) {
				cout << "Controller XML specification is not valid: " << e.what() << endl;
				//exit(-1);
			}

			// Create a PIDController instance
			PIDController *pid = new PIDController(name, id, 0, p, i, d, saturation);

			cout << endl << endl << "--------- Completed: ---------" << endl << pid << endl << endl;

			// Store in controller vector.
			controllers.push_back(pid);
		}


	} else {
		return controllers;
	}

	printf("ERROR (NavsysXMLParser.cpp): Should never get here. \n\r");
	//return controllers;
}

