#ifndef ODOMETRY_OBSERVATION_H
#define ODOMETRY_OBSERVATION_H

#include "stdio.h"
#include "Observation.h"
#include "config.h"
#include "Mutex.h"


class OdometryObservation : Observation {
    public:
		// Observer is always observing in a coordinate sys. defined from BODY.
        OdometryObservation(const double left, const double right) {
			this->right = right;
			this->left = left;
		}

		/**
		* Thread-safe.
		*/
		const double getLeft() {
			leftMutex.lock();
			double tmp = left;
			leftMutex.unlock();
			
			return tmp;
		}

		/**
		* Thread-safe.
		*/
		const double getRight() {
			rightMutex.lock();
			double tmp = right;
			rightMutex.unlock();

			return tmp;
		}

		/**
		* Thread-safe.
		*/
		const double getAndClearLeft() {
			leftMutex.lock();
			double l = left;
			left = 0;
			leftMutex.unlock();

			return l;
		}

		/**
		* Thread-safe.
		*/
		const double getAndClearRight() {
			rightMutex.lock();
			double r = right;
			right=0;
			rightMutex.unlock();

			return r;
		}

		/*const int getTimeStamp() {
			
		}*/

		/**
		* Thread-safe.
		*/
		void setObservation(const double left, const double right) {
			leftMutex.lock();
			rightMutex.lock();
			
			// Both assignments executed without interruption.
			this->right = right;
			this->left = left;

			leftMutex.unlock();
			rightMutex.unlock();
		}

		/**
		* Thread-safe.
		*/
		void addToLeft(const double left) {
			leftMutex.lock();
			this->left+=left;
			leftMutex.unlock();
		}

		/**
		* Thread-safe.
		*/
		void addToRight(const double right) {
			rightMutex.lock();
			this->right+=right;
			rightMutex.unlock();
		}

    private:
		Mutex leftMutex, rightMutex;

		void setTimeStamp() {
			; // Nothing yet..
		}
		double left;
		double right;
		int timestamp;
};
#endif

