#ifndef PID_CONTROLLER_H
#define PID_CONTROLLER_H

#import "Controller.h"


class PIDController : public Controller {
    public:

        PIDController(string name, int id, int group, double Kp, double Ti, double Td, double integralSaturation) : Controller(name, id, group) {
			this->Kp = Kp;
			this->Ti = Ti;
			this->Td = Td;
			this->integralSaturation = integralSaturation;

			i = 0;
			p_last = 0;
		}

		PIDController(string name, int id, double Kp, double Ti, double Td, double integralSaturation) : Controller(name, id) {
			this->Kp = Kp;
			this->Ti = Ti;
			this->Td = Td;
			this->integralSaturation = integralSaturation;

			i = 0;
			p_last = 0;
		}

		~PIDController() {

		}

		double update(const double measurement, const double reference, const double dt) {
			double err = (reference-measurement);
			double p = Kp*err;
			i += p/Ti;
			double d = p*Td;
			p_last = p;

			if(integralSaturation>0) {
				if(i>integralSaturation) i = integralSaturation;
				else if(i<-integralSaturation) i = -integralSaturation;
			}

			//cout << "p: " << p << endl;
			//printf("SurgeController: err: %f, p %f, i %f, d %f\n\r", err, p, i, d);
			return (p  + d);

		}

		void setParameters(double Kp, double Ti, double Td, double integralSaturation) {
			this->Kp = Kp;
			this->Ti = Ti;
			this->Td = Td;
			this->integralSaturation = integralSaturation;
		}

		// Overloaded << operator using friend for private data access.
		friend ostream& operator<<(ostream& output, const PIDController& pidc) {
	    	output << "PID Controller \"" << pidc.name << "\" group:id "<< pidc.group << ":" << pidc.id << ":" << "\nKp: " << pidc.Kp << "\nTi: " << pidc.Ti << "\nTd: " << pidc.Td << "\nSaturation: " << pidc.integralSaturation << endl;
	    	return output;  // for multiple << operators.
		}

    private:
		double integralSaturation;
		double Kp;
		double Ti;
		double Td;

		double p_last;
		double i;
};
#endif

