#include "RobotModel.h"


LOSGuidance *los;
extern RobotModel *robotModel;
extern MessageHandler *messageHandler;
extern MessageHandler *messageHandlerPlan;

// Model:
double RobotModel::F[3][3] = {{1, 0, 0},
					{0, 1, 0},
					{0, 0, 1}};
double RobotModel::Gamma[3][3] = {{1, 0, 0},
				 	{0, 1, 0},
					{0, 0, 1}
				};


ofstream fout;


RobotModel::~RobotModel() {
	fout.close();
	system("pause");

}

RobotModel::RobotModel(double T, MessageHandler *messageHandler, MessageHandler *messageHandlerPlan, vector<Controller *> controllerList) {
		this->messageHandler = messageHandler;
		this->messageHandlerPlan = messageHandlerPlan;

		// Assign controllers
		surgeRateController = controllerList.at(SURGE_RATE_CONTROLLER);
		yawRateController = controllerList.at(YAW_RATE_CONTROLLER);

		// Open textual log
		fout.open("robotlog.txt");
		if (!fout.good()) {
			cout << "Error creating log file.";
		}

		x.x_l = 0;
		x.y_l = 0;
		x.u_b = 0;
		x.v_b = 0;
		x.psi_l = 0;	// Starting angle
		x.r_b = 0;

		adjustPosition = 0;
		adjustLeft = 0;
		adjustRight = 0;
		measuring = 0;

		// Max force
		maxForceLeft = MAX_FORCE;
		maxForceRight = MAX_FORCE;

		// Max speed
		setMaxSpeed(10.0, 10.0);

		// Zero out observations:
		odometry = new OdometryObservation(0,0);


		// Multiply Gamma with sample time.
		matrix_mult((double *)Gamma, 3, 3, T);


		// LOS (for guidance)
		los = new LOSGuidance(messageHandlerPlan, controllerList);//&wp0); // new LOSGuidance();

		//this->setPosition(2.55, 0.33, PI);
		//this->setPosition(2.75, 0.25, 135*(PI/180));
		this->setPosition(0, 0, 0);

		// Starting point











		los_waypoint wprc, wplc;

		// Set id on local coordinate (the one we transformes TO)
		wplc.id = -1;
		wplc.circle_of_acceptance = 0.01;
		// Get the robocentric wp
		wprc.x_l = -0.4;
		wprc.y_l = 0;	// From mm to m
		wprc.psi_l = 0;		// Already in rad -- but increasing precision


		// Translate from robot to local coordinates:
		CoordLocal result(0,0,0);
		CoordBody body(wprc.x_l, wprc.y_l, 0);
		CoordLocal bodyRef(this->getX(),this->getY(),0);
		AngleEuler eulerRef(0, 0, this->getPsi());
		body2Local(body, bodyRef, eulerRef, &result);
		wplc.x_l = result.getX();
		wplc.y_l = result.getY();
		wplc.psi_l = angleWrap(this->getPsi() + wprc.psi_l);	// Don't twist too much.
		//printf("Oversatt: %f, %f, %f",wplc.x_l,wplc.y_l,wplc.psi_l);

		los_waypoint wp00;

		wp00.id = -1;
		wp00.x_l = wplc.x_l;
		wp00.y_l = wplc.y_l;
		wp00.psi_l = wplc.psi_l;
		wp00.circle_of_acceptance = 0.01;




		los_waypoint wp0;
		wp0.id = -1;
		wp0.x_l = -0.700;
		wp0.y_l = 0;
		wp0.psi_l = PI/2;
		wp0.circle_of_acceptance = 0.01;


		los_waypoint wp1;
		wp1.id = 1;
		wp1.x_l = -0.700;
		wp1.y_l = 0.700;
		wp1.psi_l = PI/2;
		wp1.circle_of_acceptance = 0.01;

		los_waypoint wp2;
		wp2.id = -1;
		wp2.x_l = 0.25;
		wp2.y_l = 0;
		wp2.psi_l = 0;
		wp2.circle_of_acceptance = 0.01;

		los_waypoint wp3;
		wp3.id = -1;
	    wp3.x_l = 0;
		wp3.y_l = 0;
		wp3.psi_l = 0;
		wp3.circle_of_acceptance = 0.01;


		//los->addWaypoint(&wp0);
		//los->addWaypoint(&wp1);
		//los->addWaypoint(&wp2);
		//los->addWaypoint(&wp3);
		//los->addWaypoint(&wp1);
		//printf("Waypoint added\n");

		//messageHandler->sendMessage((char *)&wp1, sizeof(los_waypoint), MINOR_WAYPOINT_MESSAGE, PRIORITY_HIGH);
//los->addWaypoint(&wp1);

		// Clear filter buffer
		int i = 0;
		for(i=0;i<FILTER_LENGTH;i++) ringbuf[i] = 0;

		timeStamp.tv_sec = 0;
		timeStamp.tv_usec = 0;

		messageHandler->say("I am Legend of Norway");
		messageHandler->say("Navigation Ready..");
}


void handleCommands(pmq_message *pm) {
	switch(pm->minor) {
		case MINOR_CMD_SYSTEM_STOP:
			printf("UNHANDLED MESSAGE IN handleCommands!!");
			cout << "UNHANDLED MESSAGE IN handleCommands!!" << endl;
		break;
		case MINOR_CMD_SYSTEM_GO:
			printf("UNHANDLED MESSAGE IN handleCommands!!");
			cout << "UNHANDLED MESSAGE IN handleCommands!!" << endl;
		break;
		case MINOR_CMD_REQUEST_POSITION:
			posix_robot_state_3dof x;
			x.x_l = robotModel->getX()*1000;
			x.y_l = robotModel->getY()*1000;
			x.psi_l = robotModel->getPsi();
			messageHandlerPlan->sendMessage((char *)&x, sizeof(posix_robot_state_3dof), MINOR_NAVSYS_POSITION, PRIORITY_HIGH);
			//cout<< "Position sent" << endl;
		break;
		case MINOR_CMD_CLEAR_WAYPOINTS:
			los->clearAllWaypoints();
		break;
		case MINOR_CMD_SYSTEM_INITIALIZE:
			los->clearAllWaypoints();			
			robotModel->setPosition(0, 0, 0);
			robotModel->stop();			
		break;
		default:
			printf("UNHANDLED MESSAGE IN handleCommands!!");
			cout << "UNHANDLED MESSAGE IN handleCommands!!" << endl;
		break;
	}

}


/**
* Thread safe because Odometry::addToLeft() etc. is thread-safe.
*/
void handleOdometryMessage(pmq_message *pm) {
	odometry_observation *oo = (odometry_observation *)pm->data;

	static double left = 0;
	static double right = 0;

	if(DEBUG_OUTPUT) cout << "Proxy/CAN: l: " << dec << oo->left << " r: " << dec << oo->right << "\n";


	odometry->addToLeft((double)(oo->left));
	odometry->addToRight((double)(oo->right));


	left += (double)(oo->left);
	right += (double)(oo->right);

}


/**
* Checks if the waypoint is inside our feasable region.
*
* Current implementation assumes a square table...
*/
bool checkWPFeasibility(los_waypoint *wp) {
	if((wp->x_l > TABLE_LENGTH_X - ROBOT_RADIUS) && (wp->y_l > TABLE_WIDTH_Y - ROBOT_RADIUS) && (wp->x_l < 0 + ROBOT_RADIUS) && (wp->y_l < 0  + ROBOT_RADIUS)) {
		printf("WP outside table");
		cout << "WP is outside feasible region." << endl;
		return false;
	}
	return true;
}


/**
* Add a waypoint in local (table) coordinates
*/
void handleWPLocalCoordinates(pmq_message *pm) {
	ai_waypoint * awp = (ai_waypoint *)pm->data;
	//printf("%d",(short)(robotModel->getX()*1000));
	//short jalla = awp->y_l;

	//printf("WaypointLocal message: %i , %d , %d\n", 2, &jalla, jalla);
	printf("WaypointLocal message: %f , %f , %f, %f\n\r", (double)awp->x_l, (double)awp->y_l, (double)awp->psi_l, (double) awp->tag);
//printf("AddLocalWP:");
	los_waypoint wp;
	wp.id = awp->id;
	wp.tag = awp->tag;
	wp.x_l = ((double)awp->x_l)/1000.0;	// From mm to m
	wp.y_l = ((double)awp->y_l)/1000.0;	// From mm to m
	wp.psi_l = (double)awp->psi_l;		// Already in rad -- but increasing precision
	wp.circle_of_acceptance = ((double)awp->circle_of_acceptance)/1000.0;	// From mm to m

	if(awp->id == 3 || awp->id == -3){
		robotModel->setMaxForce(100, 100);
	}
	else{
		robotModel->setMaxForce(MAX_FORCE, MAX_FORCE);
	}

//	if(checkWPFeasibility(&wp)) {
		los->addWaypoint(&wp);
//		messageHandler->say("Added");

/*	} else {
		messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_WAYPOINT_UNFEASIBLE, PRIORITY_HIGH);
		messageHandler->say("abs waypoint unfeasible.");
	}*/

}


/**
* Add a waypoint in robot-coordinates
*/
void handleWPRobotCoordinates(pmq_message *pm) {
	ai_waypoint *awp = (ai_waypoint *)pm->data;
	los_waypoint wprc, wplc;
	printf("WaypointRobot message: %f , %f , %f\n", (double)awp->x_l, (double)awp->y_l, (double)awp->psi_l, (double)awp->tag);
	// Set id on local coordinate (the one we transformes TO)
	wplc.id = awp->id;
	wplc.tag = awp->tag;
	wplc.circle_of_acceptance = ((double)awp->circle_of_acceptance)/1000.0;	// From mm to m

	// Get the robocentric wp
	wprc.x_l = ((double)awp->x_l)/1000.0;	// From mm to m
	wprc.y_l = ((double)awp->y_l)/1000.0;	// From mm to m
	wprc.psi_l = (double)awp->psi_l;		// Already in rad -- but increasing precision


	// Translate from robot to local coordinates:
	CoordLocal result(0,0,0);
	CoordBody body(wprc.x_l, wprc.y_l, 0);
	CoordLocal bodyRef(robotModel->getX(),robotModel->getY(),0);
	AngleEuler eulerRef(0, 0, robotModel->getPsi());
	body2Local(body, bodyRef, eulerRef, &result);
	wplc.x_l = result.getX();
	wplc.y_l = result.getY();
	wplc.psi_l = angleWrap(robotModel->getPsi() + wprc.psi_l);	// Don't twist too much.

	if(awp->id == 3 || awp->id == -3){
		robotModel->setMaxForce(100, 100);
	}
	else{
		robotModel->setMaxForce(MAX_FORCE, MAX_FORCE);
	}

	if(wplc.id>1){

		if(wplc.x_l < 0.2){
			printf("WP outside table (going forward)\n\r");
		//	if(robotModel->getX() < 0.2){
				printf("Rejected!\n\r");
				messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
				return;
		//	}
		//	else{
		//		wplc.x_l = 0.2;
		//	}
		}
		else if(wplc.x_l > 2.8){
				printf("WP outside table (going forward)\n\r");
			//	if(robotModel->getX() > 2.8){
					printf("Rejected!\n\r");
					messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
					return;
			//	}
			//	else{
			//		wplc.x_l = 2.8;
			//	}
		}

		printf("WP x accepted! \n\r");


		if(wplc.y_l < 0.2){
			printf("WP outside table (going forward)\n\r");
			//if(robotModel->getY() < 0.2){
				printf("Rejected!\n\r");
				messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
				return;
			//}
			//else{
			//	wplc.y_l = 0.2;
			//}
		}
		else if(wplc.y_l > 1.9){
				printf("WP outside table (going forward)\n\r");
			//	if(robotModel->getY() > 1.9){
					printf("Rejected!\n\r");
					messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
					return;
			//	}
			//	else{
			//		wplc.y_l = 1.9;
			//	}
		}

		printf("WP y accepted! \n\r");



	}
	else if(wplc.id < -1){

		if(wplc.x_l < 0.3){
			printf("WP outside table (going backward)\n\r");
			//if(robotModel->getX() < 0.3){
				printf("Rejected!\n\r");
				messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
				return;
			//}
			//else{
			//	wplc.x_l = 0.3;
			//}
		}
		else if(wplc.x_l > 2.7){
				printf("WP outside table (going backward)\n\r");
			//	if(robotModel->getX() > 2.7){
					printf("Rejected!\n\r");
					messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
					return;
			//	}
			//	else{
			//		wplc.x_l = 2.7;
			///	}
		}

		printf("WP x accepted! \n\r");


		if(wplc.y_l < 0.3){
			printf("WP outside table (going backward)\n\r");
		//	if(robotModel->getY() < 0.3){
				printf("Rejected!\n\r");
				messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
				return;
		//	}
		//	else{
		//		wplc.y_l = 0.3;
		//	}
		}
		else if(wplc.y_l > 1.8){
				printf("WP outside table (going backward)\n\r");
		//		if(robotModel->getY() > 1.9){
					printf("Rejected!\n\r");
					messageHandlerPlan->sendMessage((char *)awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
					return;
		//		}
		//		else{
		//			wplc.y_l = 1.8;
		//		}
		}

		printf("WP y accepted! \n\r");

	}

	los->addWaypoint(&wplc);

}

/**
* Compute local coordinates from robot
*/
void handleRobotToLocalRequest(pmq_message *pm) {
	ai_waypoint *awp = (ai_waypoint *)pm->data;
	los_waypoint wprc;
	posix_robot_state_3dof wplc;
	//printf("Got RobotToLocalMessage: %f , %f , %f\n", (double)awp->x_l, (double)awp->y_l, (double)awp->psi_l);

	// Get the robocentric wp
	wprc.x_l = ((double)awp->x_l)/1000.0;	// From mm to m
	wprc.y_l = ((double)awp->y_l)/1000.0;	// From mm to m
	wprc.psi_l = (double)awp->psi_l;		// Already in rad -- but increasing precision


	// Translate from robot to local coordinates:
	CoordLocal result(0,0,0);
	CoordBody body(wprc.x_l, wprc.y_l, 0);
	CoordLocal bodyRef(robotModel->getX(),robotModel->getY(),0);
	AngleEuler eulerRef(0, 0, robotModel->getPsi());
	body2Local(body, bodyRef, eulerRef, &result);
	wplc.x_l = result.getX()*1000;
	wplc.y_l = result.getY()*1000;
	wplc.psi_l = angleWrap(robotModel->getPsi() + wprc.psi_l);	// Don't twist too much.

	printf("Sending RobotToLocalMessage: %f, %f, %f\n\r", (double)wplc.x_l, (double)wplc.y_l, (double)wplc.psi_l);

	messageHandlerPlan->sendMessage((char *)&wplc, sizeof(posix_robot_state_3dof), MINOR_LOCAL_WAYPOINT, PRIORITY_HIGH);

}


/**
* Not thread-safe (the error will be fixed automatically)
*/
void handleSetSpeedLimit(pmq_message *pm) {
	speed_limit *sl = (speed_limit *)pm->data;

	robotModel->setMaxSpeed(sl->u_b, sl->r_b);
}

/**
* Not thread-safe (the error will be fixed automatically)
*/
void handleSetSpeed(pmq_message *pm) {
	speed *s = (speed *)pm->data;
	cout << s->u_b << endl;
	robotModel->setSpeed(s->u_b, s->r_b);
}

/**
* Not thread-safe
*/
void handleSetPosition(pmq_message *pm) {
	posix_robot_state_3dof *prs3d = (posix_robot_state_3dof *)pm->data;
	printf("setPos: %f, %f, %f\n\r", ((double)(prs3d->x_l))/1000, ((double)(prs3d->y_l))/1000, (180/PI)*((double)(prs3d->psi_l)));
	robotModel->setPosition(((double)(prs3d->x_l))/1000, ((double)(prs3d->y_l))/1000, ((double)(prs3d->psi_l)));
}


/**
* Not thread-safe
*/
int beaconBuffer = 0;
int buffCnt = 0;
void handleBeaconMeasurement(pmq_message *pm){
	beacon_angles *beaconAngles = (beacon_angles *)pm->data;

	if(robotModel->measuring == BEACON_MEASURE_BLUE_CONTAINER){
		buffCnt++;
		beaconBuffer = (beaconBuffer+beaconAngles->b1)/buffCnt;
		beaconAngles->b1 = beaconBuffer;
		printf("Measuring blue container: %d\n\r", beaconAngles->b1);
		double newX = 300 - tan((beaconAngles->b1-180+4)*PI/180)*2.045;
		printf("x: %f \n\r", newX);
		beaconAngles->b2 = 400;
		beaconAngles->b3 = 400;
		messageHandler->sendMessage((char *)beaconAngles, sizeof(beacon_angles), MINOR_NAVSYS_UPDATE_BEACON_ANGLES, PRIORITY_HIGH);
		robotModel->setPosition(newX, robotModel->getY(), robotModel->getPsi());
	}
	else if(robotModel->measuring == BEACON_MEASURE_STOP){
		printf("Stopping beacon measurements\n\r");
		buffCnt = 0;
	}



	//printf("BeaconMeasurement received: %d",pm->data[4]);
	/*printf("BEACON ANGLES RECEIVED: %d, %d, %d\n", (m.DATA[0])|(m.DATA[1]<<8), (m.DATA[2])+(m.DATA[3]<<8),(m.DATA[4])+(m.DATA[5]<<8));
	if(measuring){
		calculatePositionFromAngles(1, -45, 90, -135);
		robotModel->setPosition(((double)(prs3d->x_l))/1000, ((double)(prs3d->y_l))/1000, ((double)(prs3d->psi_l)));
	}*/

	//printf("Position calculated from beacons:\n");
}

/**
* Not thread-safe
*/
void handleBeaconMeasureRequest(pmq_message *pm){
	beacon_measure *measureStruct = (beacon_measure *)pm->data;
	robotModel->measuring = measureStruct->type;

	/*if(measureStruct->type == BEACON_MEASURE_STOP){
		robotModel->measuring = 0;
	}*/


	if(robotModel->measuring == BEACON_MEASURE_BLUE_CONTAINER){
		beacon_angles beaconAngles;
		beaconAngles.b1 = (double)200;
		beaconAngles.b2 = (double)400;
		beaconAngles.b3 = (double)400;
		messageHandler->sendMessage((char *)&beaconAngles, sizeof(beacon_angles), MINOR_NAVSYS_UPDATE_BEACON_ANGLES, PRIORITY_HIGH);
	}
	else if(robotModel->measuring == BEACON_MEASURE_RED_CONTAINER){
		beacon_angles beaconAngles;
		beaconAngles.b1 = (double)160;
		beaconAngles.b2 = (double)400;
		beaconAngles.b3 = (double)400;
		messageHandler->sendMessage((char *)&beaconAngles, sizeof(beacon_angles), MINOR_NAVSYS_UPDATE_BEACON_ANGLES, PRIORITY_HIGH);
	}


	printf("Got beaconMessurementMessage, type: %d\n\r", robotModel->measuring);

}

/**
* Not thread-safe
*/
void handleAdjustPosition(pmq_message *pm){
	robotModel->adjustLeft = pm->data[0]-pm->data[2];
	robotModel->adjustRight = pm->data[1]-pm->data[3];
	if(pm->data[4] == 1){
		robotModel->adjustPosition = 1;
		printf("Start adjustPosition\n\r");
	}
	else if(pm->data[4] == 2){
		robotModel->adjustPosition = 0;
		printf("Abort adjustPosition\n\r");
	}
	//printf("Adjusting position: lf: %d, rf: %d\n\r", robotModel->adjustLeft, robotModel->adjustRight);
}

/**
* Not thread-safe
*/
void RobotModel::setPosition(double x_l, double y_l, double psi_l) {
	this->x.x_l = x_l;
	this->x.y_l = y_l;
	this->x.psi_l = psi_l;

	messageHandler->say("Position set");
/*	beacon_angles beaconAngles;
	beaconAngles.b1 = 360;
	beaconAngles.b2 = 330;
	beaconAngles.b3 = 355;
	messageHandler->sendMessage((char *)&beaconAngles, sizeof(beaconAngles), MINOR_NAVSYS_UPDATE_BEACON_ANGLES, PRIORITY_HIGH);
*/

}

/**
* Not thread-safe (but not likely to cause serious error)
*/
void RobotModel::setMaxForce(int left, int right) {
	maxForceLeft = left;
	maxForceRight = right;
}


/**
* Not thread-safe (but not likely to cause serious error)
*/
void RobotModel::setMaxSpeed(double u_b, double r_b) {
	max_u_b = u_b;
	max_r_b = r_b;

	if(u_b==0 && r_b==0) {
		stop();
	} else {
		setMaxForce(MAX_FORCE,MAX_FORCE);	// Revert to default value
	}
}


/**
* Not thread-safe (but not likely to cause serious error)
*/
void RobotModel::setSpeed(double u_b, double r_b) {
	this->u_b = u_b;
	this->r_b = r_b;
	printf("-------------------------------\n\r");
	printf("u_b: %f, r_b: %f \n\r",u_b,r_b);
	printf("-------------------------------\n\r");
	if(u_b==0 && r_b==0) {
		stop();
	}
}


/**
* Not thread-safe (but not likely to cause serious error)
*/
void RobotModel::stop() {
	// Set max force to motors
	setMaxForce(0,0);
	// Send stop command to motor driver
	setMotorForce(0,0);
}


ofstream myfile ("example.txt");
int filecount = 0;


/**
* Run one step.. (like number k in the equation....)
*/
void RobotModel::update() {
	static int ii=0;
	ii++;
	if(ii==11)ii=0;



	int lf=0,rf=0; // Actuator outputs.


	// Check for new sensor messages in sensor data queue:
	// Get measurements


	// Filter out outliers, clearly corrupt measurements and measurements
	// outside feasible region.
	//filterMeasurements(measurements);


	// Compute model inputs (from odometry):
	// Compute angle-movement on wheels
	double dl = 2*PI*(odometry->getAndClearLeft())/(ODOMETRY_PULSES_PER_REVOLUTION * ODOMETRY_GEAR_RATIO);	// Should returned rotated angle on left wheel
	double dr = 2*PI*(odometry->getAndClearRight())/(ODOMETRY_PULSES_PER_REVOLUTION * ODOMETRY_GEAR_RATIO);


	// Remember scaling here!
	double L = ODOMETRY_WHEEL_DISTANCE;
	double r = (ODOMETRY_WHEEL_DIAMETER/2);

	// Old calculation
	// a = 2*r*(dl-dr)/L;
	//double dx_b = (r*(dl+dr)*0.5);		// Exact: c*sin(a);
	double dy_b = 0; 							// Exact: (1-cos(a))*c;
	//double dpsi = (r*(dl-dr))/L;		// Exact: a (maybe 2*r?)


	//Exact calculation of dx_b and dpsi
	double dpsi = (r*(dl-dr))/L;
	double dx_b = (r*(dl+dr)*0.5)*cos(dpsi);

	//printf("Calculation: %f , %f \n",dpsi, dx_b);

	static int i=0;
	ringbuf[i++] = dx_b;
	if(i>=FILTER_LENGTH) i=0;


	//dx_b = 0;	???????????????


	//int j;
	//for(j=0;j<FILTER_LENGTH;j++){ dx_b+=ringbuf[j]; }
	//dx_b=dx_b/FILTER_LENGTH;


	// Har denne if'en noen funksjon???
	if(dx_b == 0 && dpsi == 0) {
		// Keep same speed values as last time...
	}
	else {
		x.u_b = (dx_b/1000);//sampleTimeMs)*1000;	// *1000 to convert to m/s from m/ms
		x.r_b = (dpsi/1000);//sampleTimeMs)*1000;
	}

	x.u_b = dx_b;
	x.r_b = dpsi;

	//printf("Calculation2: %f , %f \n",x.u_b,x.r_b);


	// Then compose transformation:
	CoordLocal result(0,0,0);
	CoordBody body(dx_b, 0, 0);
	CoordLocal bodyRef(x.x_l,x.y_l,0);
	AngleEuler eulerRef(0, 0, x.psi_l);
	body2Local(body, bodyRef, eulerRef, &result);
	double x_pri_l = result.getX();
	double y_pri_l = result.getY();
	double psi_pri_l = angleWrap(x.psi_l + dpsi);	// Don't twist too much.

	// Initial estimate:
	x.x_l = x_pri_l;
	x.y_l = y_pri_l;
	x.psi_l = psi_pri_l;

	//printf("Position: %f , %f, %f \n", x.x_l*100, x.y_l*100, (x.psi_l*180)/PI);
	//printf("current beaconangle: %f\n\r",calculateAnglesFromPosition(2,x.x_l, x.y_l, (x.psi_l*180)/PI));
	//beacon_angles beaconAngles;


	//calculatePositionFromAngles(1, 30, -90, 130);
	//printf("%f, %f, %f \n\r", beaconAngles.b1, beaconAngles.b2, beaconAngles.b3);
	//beaconAngles.b1 = (180/PI)*calculateAnglesFromPosition(getX(), getY(), getPsi());
	//messageHandler->sendMessage((char *)&beaconAngles, sizeof(beaconAngles), MINOR_NAVSYS_UPDATE_BEACON_ANGLES, PRIORITY_HIGH);

	//Save position to file
	filecount++;
	//printf("%d\n\r", filecount);
	if(filecount >= 100){
		//printf("Update triangulation\n\r");
		filecount = 0;
		//messageHandler->sendMessage((char *)&beaconAngles, sizeof(beaconAngles), MINOR_NAVSYS_UPDATE_BEACON_ANGLES, PRIORITY_HIGH);
	}




	// Use Estimator class to get new estimate:

	// Compute new reference speeds.
	robot_state_3dof x_d;
	if(adjustPosition){
		if(adjustLeft <= 2 && adjustRight <= 2 && adjustLeft >= -2 && adjustRight >= -2){
			printf("Adjust completed\n\r");

			/*ai_waypoint awp;
			awp.id = 1;
			awp.x_l = 100; // From m to mm
			awp.y_l = 100; // From m to mm
			awp.psi_l = 10;
			awp.circle_of_acceptance = 10;
			messageHandlerPlan->sendMessage((char *)&awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);*/

			adjustPosition = 0;
			x_d.u_b = 0;
			x_d.r_b = 0;
		}
		else{
			x_d.u_b = 0.07*(adjustLeft+adjustRight);
			x_d.r_b = 0.07*(adjustLeft-adjustRight);
			printf(".");
			//printf("Adjusting: u=%f, r=%f\n\r", x_d.u_b, x_d.r_b);

		}

	}
	else{
		//printf("***");
		los->update(&x, &x_d);


	}

	max_u_b = 10;
	max_r_b = 10;

	// Limit to current limit
	/*if(x_d.u_b>max_u_b) x_d.u_b = max_u_b;
	else if(x_d.u_b<-max_u_b) x_d.u_b = -max_u_b;
	if(x_d.r_b>max_r_b) x_d.r_b = max_r_b;
	else if(x_d.r_b<-max_r_b) x_d.r_b = -max_r_b;*/

	//printf("Desired: %f, %f\n\r", x_d.r_b, x_d.u_b);
	//	x_d.r_b = 10;
	//	x_d.u_b = 10;
	if((x_d.r_b || x_d.u_b)) {

		// This is not real force, but it's an OK approximation.
		double surgeForce, swayForce;

		int dt = 1;

		// Using two independent controllers to control fwd. speed and rot. speed:
		surgeForce = surgeRateController->update(x.u_b, x_d.u_b, dt);	// measured fwd speed, reference fwd speed, dt
		swayForce = yawRateController->update(x.r_b, x_d.r_b, dt);	// measurement rot speed, reference rot speed, dt

		// Distribute forces among actuators (our two motors):
		double leftForce = surgeForce + swayForce;
		double rightForce = surgeForce - swayForce;

		lf = (int)leftForce;
		rf = (int)rightForce;

		//printf("Motorpdrag: lf: %d, rf: %d, u %f, r %f, u_d %f, r_d %f\n\r", lf, rf, x.u_b, x.r_b, x_d.u_b,x_d.r_b);
	}
	else {
		lf = 0;
		rf = 0;
	}




	if(lf>maxForceLeft) lf = maxForceLeft;
	else if(lf<-maxForceLeft) lf = -maxForceLeft;
	if(rf>maxForceRight) rf = maxForceRight;
	else if(rf<-maxForceRight) rf = -maxForceRight;

	//printf("Motorpdrag: lf: %d, rf: %d\n\r", lf, rf );

	// Set control force (voltage proportional) on motor.

	setMotorForce(lf, rf);
	//setMotorForce(200, 200);


}


/* //original update
void RobotModel::update() {
	static int ii=0;
	ii++;
	if(ii==11)ii=0;



	int lf=0,rf=0; // Actuator outputs.


	// Check for new sensor messages in sensor data queue:
	// Get measurements


	// Filter out outliers, clearly corrupt measurements and measurements
	// outside feasible region.
	//filterMeasurements(measurements);


	// Compute model inputs (from odometry):
	// Compute angle-movement on wheels
	double dl = 2*PI*(odometry->getAndClearLeft())/(ODOMETRY_PULSES_PER_REVOLUTION * ODOMETRY_GEAR_RATIO);	// Should returned rotated angle on left wheel
	double dr = 2*PI*(odometry->getAndClearRight())/(ODOMETRY_PULSES_PER_REVOLUTION * ODOMETRY_GEAR_RATIO);

	printf("Rotated angles: %f , %f \n",dl,dr);

	// Remember scaling here!
	double L = ODOMETRY_WHEEL_DISTANCE;
	double r = (ODOMETRY_WHEEL_DIAMETER/2);

	// Old calculation
	// a = 2*r*(dl-dr)/L;
	double dx_b = (r*(dl+dr)*0.5);		// Exact: c*sin(a);
	double dy_b = 0; 							// Exact: (1-cos(a))*c;
	double dpsi = (r*(dl-dr))/L;		// Exact: a (maybe 2*r?)


	//Exact calculation of dx_b and dpsi
	//double dpsi = (r*(dl-dr))/L;
	//double dx_b = (r*(dl+dr)*0.5)*cos(dpsi);


	static int i=0;
	ringbuf[i++] = dx_b;
	if(i>=FILTER_LENGTH) i=0;


	//dx_b = 0;	???????????????


	int j;
	for(j=0;j<FILTER_LENGTH;j++){ dx_b+=ringbuf[j]; }
	dx_b=dx_b/FILTER_LENGTH;


	// Har denne if'en noen funksjon???
	if(dx_b == 0 && dpsi == 0) {
		// Keep same speed values as last time...
	}
	else {
		x.u_b = (dx_b);//sampleTimeMs)*1000;	// *1000 to convert to m/s from m/ms
		x.r_b = (dpsi);//sampleTimeMs)*1000;
	}

	x.u_b = dx_b;
	x.r_b = dpsi;


	// Then compose transformation:
	CoordLocal result(0,0,0);
	CoordBody body(dx_b, 0, 0);
	CoordLocal bodyRef(x.x_l,x.y_l,0);
	AngleEuler eulerRef(0, 0, x.psi_l);
	body2Local(body, bodyRef, eulerRef, &result);
	double x_pri_l = result.getX();
	double y_pri_l = result.getY();
	double psi_pri_l = angleWrap(x.psi_l + dpsi);	// Don't twist too much.

	// Initial estimate:
	x.x_l = x_pri_l;
	x.y_l = y_pri_l;
	x.psi_l = psi_pri_l;

	//printf("Position: %f , %f, %f \n", x.x_l*100, x.y_l*100, (x.psi_l*180)/PI);
	fout << "[ " << x.x_l << " " << x.y_l << " ]" << endl; // " << x.psi_l << " " << " ]" << endl;

	//Save position to file
	filecount++;
	if(filecount = 1000){
		if (myfile.is_open())
		{
			char buffer [200];
			sprintf(buffer, "%f, %f, %f\n", x.x_l*100, x.y_l*100, (x.psi_l*180)/PI);
			//sprintf (buffer, "%f, %f, %f \n", filecount, x.x_l*100, x.y_l*100, (x.psi_l*180)/PI);
			myfile << buffer;
		}
		else cout << "Unable to open file";
		filecount = 0;
	}



	// Use Estimator class to get new estimate:

	// Compute new reference speeds.
	robot_state_3dof x_d;
	los->update(&x, &x_d);

//	x_d.u_b = u_b;
//	x_d.r_b = r_b;



	// Limit to current limit
	if(x_d.u_b>max_u_b) x_d.u_b = max_u_b;
	else if(x_d.u_b<-max_u_b) x_d.u_b = -max_u_b;
	if(x_d.r_b>max_r_b) x_d.r_b = max_r_b;
	else if(x_d.r_b<-max_r_b) x_d.r_b = -max_r_b;


	if((x_d.r_b || x_d.u_b)) {

		// This is not real force, but it's an OK approximation.
		double surgeForce, swayForce;

		int dt = 1;

		// Using two independent controllers to control fwd. speed and rot. speed:
		surgeForce = surgeRateController->update(x.u_b, x_d.u_b, dt);	// measured fwd speed, reference fwd speed, dt
		swayForce = yawRateController->update(x.r_b, x_d.r_b, dt);	// measurement rot speed, reference rot speed, dt

		// Distribute forces among actuators (our two motors):
		double leftForce = surgeForce + swayForce;
		double rightForce = surgeForce - swayForce;

		lf = (int)leftForce;
		rf = (int)rightForce;
	}
	else {
		lf = 0;
		rf = 0;
	}


	if(lf>maxForceLeft) lf = maxForceLeft;
	else if(lf<-maxForceLeft) lf = -maxForceLeft;
	if(rf>maxForceRight) rf = maxForceRight;
	else if(rf<-maxForceRight) rf = -maxForceRight;

	// Set control force (voltage proportional) on motor.
	setMotorForce(lf, rf);
	//setMotorForce(0xff, -0xff);


}*/

void RobotModel::setMotorForce(int left, int right) {


	motor_control_message mcm;
	static int i = 0;
	static int up = 1;
	if(up)i+=20;
	else i-=20;
	if(i>0xFF){up=0; i=0xFF; }
	else if(i<-0xFE){ up=1; i=-0xFE; }


	mcm.m1 = left;//0xDF;//left;
	mcm.m2 = right;//0x3F;//right;

	if(DEBUG_OUTPUT) cout << "ALforce: " << dec << left << " ARforce: " << dec << right << "\n";
	if(DEBUG_OUTPUT) cout << "ALforce: " << hex << left << " ARforce: " << hex << right << "\n";

	//printf("Output force: Right: %i , Left: %i\n", right, left);
	messageHandler->sendMessage((char *)&mcm, sizeof(motor_control_message), MINOR_MOTOR_CONTROL_MESSAGE, PRIORITY_VERY_HIGH);



	posix_robot_state_3dof x;
	x.x_l = (short)(robotModel->getX()*1000);
	x.y_l = (short)(robotModel->getY()*1000);
	x.psi_l = robotModel->getPsi();

	//printf("x: %d , y: %d , psi: %f\n\r", x.x_l, x.y_l, x.psi_l);
	messageHandler->sendLogMessage((char *)&x, sizeof(posix_robot_state_3dof), MINOR_NAVSYS_POSITION, PRIORITY_HIGH);
}


/**
* Run one step.. (like number k in the equation....)
*/
//Observation *RobotModel::getNextMeasurement() {
	// Check for new sensor messages in sensor data queue:
	//measurements.get(i);

	// Do something useful with them ;)

	// Use Estimator class to get new estimate:

//}


/**
* Add a sensor to the robot model.
*/
void RobotModel::addSensor(Sensor s) {	// Kanskje også "SensorModel s" her..
	// Add to sensor list.

}


