#include "mathutils.h"
#include <math.h>
#include <iostream>
#include <cstdlib>
#include <sys/time.h>

using namespace std;

#define TEAM_COLOR_RED 1
#define TEAM_COLOR_BLUE 2


/**
* Returns the same angle (in radians) wrapped to [-PI, PI]
*
* Works only for angles <= |2*PI| away from [-PI, PI]. Larger
* angles will be reduced with 2*PI, smaller increased with 2*PI.
*/
double angleWrap(double angle) {
	return (angle>PI)?angle-2*PI:( (angle<-PI)?angle+2*PI:angle );
}

void matrix_mult(double *matrix, const int n, const int m, const double c) {
	int size = n*m-1;
	int i;
	for(i=0;i<size;i++) matrix[i] *= c;
}


/**
*
* Insert beacon angles in degrees
*
*/
void calculatePositionFromAngles(int teamColor, double beacon1, double beacon2, double beacon3){
	double table_x, table_y, length_x,length_y;
	double beacon1_xr,beacon1_yr,beacon2_xr,beacon2_yr,beacon3_xr,beacon3_yr;
	double L12,L31,L32,L1;
	double x1,y1;
	double lambda1,lambda2,lambda3,lambda12,lambda31;
	double alpha,beta,sigma,gamma,num,den,tau,phi;
	double x_R,y_R,theta_R;
	double beacon_placement = 0;

	table_x = 3000;
	table_y = 2100;


	length_x = table_x + 2*beacon_placement;
	length_y = table_y + 2*beacon_placement;


	beacon1_xr = length_x;
	beacon1_yr = length_y;
	beacon2_xr = 0;
	beacon2_yr = length_y/2;
	beacon3_xr = length_x;
	beacon3_yr = 0;

	L12 = sqrt(pow(beacon1_xr-beacon2_xr,2)+pow(beacon1_yr-beacon2_yr,2));
	L31 = sqrt(pow(beacon1_xr-beacon3_xr,2)+pow(beacon1_yr-beacon3_yr,2));
	L32 = sqrt(pow(beacon2_xr-beacon3_xr,2)+pow(beacon2_yr-beacon3_yr,2));



	lambda1 = -beacon1;
	lambda2 = -beacon2;
	lambda3 = -beacon3;


	beta = acos((pow(L31,2)+pow(L12,2)-pow(L32,2))/(2*L31*L12))*180/PI;
	alpha = atan2(-beacon1_xr+beacon3_xr,L31)*180/PI;

	x1 = beacon1_xr;
	y1 = beacon1_yr;

	phi = (90-beta-alpha);


	lambda12 = lambda2-lambda1;

	if(lambda1>lambda2){
	    lambda12 = 360+(lambda2-lambda1);
	}

	lambda31 = lambda1-lambda3;

	if(lambda3>lambda1){
	    lambda31 = 360+(lambda1-lambda3);
	}


	sigma = 180-beta;
	gamma = sigma-lambda31;
	num = sin(lambda12*PI/180)*(L12*sin(lambda31*PI/180)-L31*sin(gamma*PI/180));
	den = L31*sin(lambda12*PI/180)*cos(gamma*PI/180)-L12*cos(lambda12*PI/180)*sin(lambda31*PI/180);
	tau = atan2(num,den)*180/PI;



	if(lambda12<180 && tau<0){
	    tau = tau+180;
	}
	if(lambda12>180 && tau>0){
    	tau = tau-180;
	}
	if(abs( (int)sin(lambda12*PI/180) ) >  abs( (int)sin(lambda31*PI/180) )){
	    L1 = (L12*sin((tau+lambda12)*PI/180))/(sin(lambda12*PI/180));
	}
	else{
	    L1 = (L31*sin((tau+sigma-lambda31)*PI/180))/(sin(lambda31*PI/180));
	}


	theta_R = phi + tau - lambda1;


	if(teamColor == TEAM_COLOR_BLUE){
		theta_R = theta_R +180;
	}


	if(theta_R <= -180){
		theta_R = theta_R + 360;
	}
	if(theta_R > 180){
		theta_R = theta_R - 360;
	}


	x_R = x1 - L1*cos((phi+tau)*PI/180) - beacon_placement;
	y_R = y1 - L1*sin((phi+tau)*PI/180) - beacon_placement;

	if(teamColor == TEAM_COLOR_BLUE){
		x_R = length_x-x_R;
		y_R = length_y-y_R;
	}

	printf("xR: %f, yR: %f, thetaR: %f\n",x_R,y_R,theta_R);
	//calculateAnglesFromPosition(teamColor, x_R,  y_R, theta_R);



/*
	x_pos = 1500 + beacon_placement;
	y_pos = 1050 + beacon_placement;
	theta = 0;

	table_x = 3000;
	table_y = 2100;

	length_x = table_x + 2*beacon_placement;
	length_y = table_y + 2*beacon_placement;

	switch(teamColor){
		case TEAM_COLOR_RED:
			beacon1_x = length_x;
			beacon1_y = length_y;
			beacon2_x = 0;
			beacon2_y = length_y/2;
			beacon3_x = length_x;
			beacon3_y = 0;
			break;
		case TEAM_COLOR_BLUE:
			beacon1_x = 0;
			beacon1_y = 0;
			beacon2_x = length_x;
			beacon2_y = length_y/2;
			beacon3_x = 0;
			beacon3_y = length_y;
			break;
		default:
			break;
	}

	beacon1_xr = length_x;
	beacon1_yr = length_y;
	beacon2_xr = 0;
	beacon2_yr = length_y/2;
	beacon3_xr = length_x;
	beacon3_yr = 0;

	lambda1 = -beacon1;
	lambda2 = -beacon2;
	lambda3 = -beacon3;

	L12 = sqrt(pow(beacon1_xr-beacon2_xr,2)+pow(beacon2_yr,2));
	L31 = beacon1_y-beacon3_y;
	L32 = sqrt(pow(beacon2_x-beacon3_x,2)+pow(beacon2_yr-beacon3_yr,2));
	//beta = atan2(beacon1_x-beacon2_x,beacon1_y-beacon2_y)*180/M_PI;
	beta = acos((pow(L31,2)+pow(L12,2)-pow(L32,2))/(2*L31*L12))*180/M_PI;
	y1 = beacon1_y;
	x1 = beacon1_x;
	phi = 90-beta;


	lambda12 = lambda2-lambda1;
	if(lambda1>lambda2) lambda12 = 360+(lambda2-lambda1);
	lambda31 = lambda1-lambda3;
	if(lambda3>lambda1) lambda31 = 360+(lambda1-lambda3);

	sigma = 180-beta;
	gamma = sigma-lambda31;
	num = sin(lambda12*M_PI/180)*(L12*sin(lambda31*M_PI/180)-L31*sin(gamma*M_PI/180));
	den = L31*sin(lambda12*M_PI/180)*cos(gamma*M_PI/180)-L12*cos(lambda12*M_PI/180)*sin(lambda31*M_PI/180);
	tau = atan2(num,den)*180/M_PI;

	if(lambda12<180 && tau<0) tau = tau+180;
	if(lambda12>180 && tau>0) tau = tau-180;

	if(abs((int)sqrt(pow(sin(lambda12*M_PI/180),2)))>abs((int)sqrt(pow(sin(lambda31*M_PI/180),2))))
		L1 = (L12*sin((tau+lambda12))*M_PI/180)/(sin(lambda12*M_PI/180));
	else
		L1 = (L31*sin((tau+sigma-lambda31)*M_PI/180))/(sin(lambda31*M_PI/180));

	thetaR = phi+tau-lambda1+180;

	if(thetaR <= -180) thetaR = thetaR + 360;
	if(thetaR > 180) thetaR = thetaR - 360;

	cout << "Robot coordinates (in metres) and heading (in degrees)" << endl;
	xR = x1 - L1*cos((phi+tau)*M_PI/180) - beacon_placement;
	yR = y1 - L1*sin((phi+tau)*M_PI/180) - beacon_placement;
	cout << "xR: " << xR << " yR: " << yR << " thetaR: " << thetaR << endl;
	//printf("xR: %f, yR: %f, thetaR: %f\n",xR,yR,thetaR);

	//calculatePositionFromAngles(teamColor,xR,yR);*/
}

/**
*
* Insert beacon angles in degrees
*
*/
void calculateAnglesFromPosition(int teamColor,double x_R, double y_R, double theta_R){
	double table_x, table_y, length_x,length_y;
	double vinkel1, vinkel2, vinkel3;
	double beacon1_x,beacon1_y,beacon2_x,beacon2_y,beacon3_x,beacon3_y;
	double beacon_placement = 0;

	x_R = x_R*100;
	y_R = y_R*100;
	theta_R = 0;
	table_x = 300;
	table_y = 210;


	length_x = table_x + 2*beacon_placement;
	length_y = table_y + 2*beacon_placement;


	switch(teamColor){
		case TEAM_COLOR_RED:
			beacon1_x = length_x;
			beacon1_y = length_y;
			beacon2_x = 0;
			beacon2_y = length_y/2;
			beacon3_x = length_x;
			beacon3_y = 0;
			break;
		case TEAM_COLOR_BLUE:
			beacon1_x = 0;
			beacon1_y = 0;
			beacon2_x = length_x;
			beacon2_y = length_y/2;
			beacon3_x = 0;
			beacon3_y = length_y;
			break;
		default:
			break;
	}

	vinkel1 = -(atan2(beacon1_y-y_R,beacon1_x-x_R)*180/PI-theta_R);
	vinkel1 = vinkel1+360*(vinkel1<0);
	vinkel2 = -(atan2(beacon2_y-y_R,beacon2_x-x_R)*180/PI-theta_R);
	vinkel2 = vinkel2+360*(vinkel2<0);
	vinkel3 = -(atan2(beacon3_y-y_R,beacon3_x-x_R)*180/PI-theta_R);
	vinkel3 = vinkel3+360*(vinkel3<0);

	printf("Vinkler: %d, %d, %d\n\r", (int)vinkel1, (int)vinkel2, (int)vinkel3);

	//return angleWrap(atan2(-x,-y)- psi);
}


