#ifndef ANTI_COLLISION_STRATEGY_H
#define ANTI_COLLISION_STRATEGY_H

#include <math.h>

#include "Observer.h"
#include "PlansysObservation.h"
#include "planner.h"
#include "waypoint.h"
#include "posix_message_config.h"
#include "init.h"
#include "observation_structs.h"

class AntiCollisionStrategy: public Observer {
  public:
	AntiCollisionStrategy( PlansysObservation * , int theTeamColor );
	~AntiCollisionStrategy();
	void Reset( int theTeamColor );
	void Init();
	void Update( int typeOfObservation );
	void UpdatePriority();
	void CheckForCollision(MessageHandler *navsysMessageHandler);
	int GetPriority();
	ai_waypoint CalculateWaypoint();
	void CalculateOpponentWaypoint( MessageHandler *navsysMessageHandler, int where);
	void AdjustPosition( MessageHandler *navsysMessageHandler );
	void PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  );
	
  private:
	int lastWaypointSent;
	int lastWaypointReached;
	int positionReceived;
	int opponentPositionReceived;
	
	int strategyDone;
	int collision;
	int collisionStop;
	int adjustCount;

	int board1, board2, board3;
	int boards[NUMBER_OF_BOARDS];

	collision_detection_message collision_message;
	
	double internalState;
	double distance;
	double distance_x;
	double distance_y;
	int timeLeft;
	int priority;
	
	int state;
	int where;

	enum{
		ADJUST,
		COLLISION,
		};
	
	PlansysObservation *_subject;
	ai_waypoint ai_waypoint_3d;
	ai_waypoint last_ai_waypoint;
	ai_waypoint activeWaypoint;
	ai_waypoint opponentWaypoint;
	ai_waypoint relativeOpponentWaypoint;
	
	cv_observation my_cv_observation;
	proxy_message my_proxy_message;
	time_message local_time_message;
};

#endif
