#include <math.h>

#include "AntiCollisionSupportStrategy.h"
#include "StartStrategy.h"
#include "waypoints.h"
#include "PlansysObservation.h"
#include "observation_structs.h"
#include "posixCommunication.h"
#include "planner.h"

AntiCollisionSupportStrategy::AntiCollisionSupportStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	priority = 5;
	internalState = 1;
	strategyDone = FALSE;
	timeLeft = 90;
	Init();
	balls.InitializeBalls();
	_subject = s;
	_subject->Attach(this);
	_subject->SetCollision(FALSE);
	_subject->InitCollisionMessage();
	cout << "Strategy added" << endl;
}

AntiCollisionSupportStrategy::~AntiCollisionSupportStrategy (){
 	_subject->Detach(this);
}

void AntiCollisionSupportStrategy::Reset( int theTeamColor ){
	_subject->SetCollision(FALSE);
	teamColor = theTeamColor;
	priority = 5;
	internalState = 1;
	strategyDone = FALSE;
	timeLeft = 90;
	Init();	
	balls.InitializeBalls();
	_subject->InitCollisionMessage();
	cout << "Strategy reset!" << endl;
}

void AntiCollisionSupportStrategy::Init(){
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	collision = FALSE;
	collisionStop = FALSE;
	state = INIT;
	adjustCount = 0;
	waypointCounter = 0;
}

void AntiCollisionSupportStrategy::UpdatePriority(){
	local_time_message = _subject->GetTimeObservation();
	timeLeft = local_time_message.timeLeft;	
	//cout << "Time left: " << timeLeft << endl;
	if(_subject->GetSupportNeeded()){
		priority = 9;
	}
	else{
		priority = 0;
	}
}

void AntiCollisionSupportStrategy::Update( int typeOfObservation ){
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision = TRUE;
	}
	else if(typeOfObservation == MINOR_NAVSYS_POSITION){
		ai_waypoint_3d = _subject->GetPosObservation();
		cout <<  ai_waypoint_3d.x_l << ", " << ai_waypoint_3d.y_l << ", " << ai_waypoint_3d.psi_l << endl;
		positionReceived = TRUE;
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;
		waypointCounter++;
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		cout << "System reset request" << endl;
		_subject->SetCollision(FALSE);
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
}

int AntiCollisionSupportStrategy::GetPriority(){
	return priority;
}

void AntiCollisionSupportStrategy::CheckForCollision(MessageHandler *navsysMessageHandler){
	if(collision){
		_subject->GetCollisionValues(navsysMessageHandler,boards);
		
		cout << "board1: " << boards[0] << ", board2: " << boards[1] << ", board3: " << boards[2] << endl;

		if(boards[0]<=COLLISION_THRESHOLD || boards[1]<=COLLISION_THRESHOLD){
			#ifdef ANTI_COLLISION_ACTIVE
				if(_subject->GetAntiCollisionActive()){
					//_subject->SetCollision(TRUE);
					collisionStop = TRUE;
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		else if(boards[2]<=COLLISION_THRESHOLD){
			#ifdef REAR_ANTI_COLLISION_ACTIVE
				if(_subject->GetRearAntiCollisionActive()){
					_subject->SetCollision(TRUE);
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
			
		}
		collision = FALSE;
	}
}

void AntiCollisionSupportStrategy::CalculateSupportWaypoint(){
	int oppX, oppY, numberOfWaypoints;
	int lastX, lastY;
	double lastPsi;
	int robotX, robotY;
	double robotPsi;
	int newX, newY;
	double newPsi;
	int radius;

	numberOfWaypoints = NUMBER_OF_SUPPORT_WAYPOINTS;

	ai_waypoint allWaypoints[numberOfWaypoints + 1];
	ai_waypoint lastSupportWaypoint;
	lastActiveWaypoint = _subject->GetLastWaypointSent();
	lastCollision = _subject->GetLastCollision();
	cout << "COLLISION_RIGHT: " << RIGHT << endl;
	opponentWaypoint = _subject->GetLocalWaypoint();
	cout << "Opponent waypoint: x: " << opponentWaypoint.x_l << ", y: " << opponentWaypoint.y_l << ", psi: " << opponentWaypoint.psi_l << endl;
	oppX = opponentWaypoint.x_l;
	oppY = opponentWaypoint.y_l;
	lastX = lastActiveWaypoint.x_l;
	lastY = lastActiveWaypoint.y_l;
	lastPsi = lastActiveWaypoint.psi_l;
	robotX = ai_waypoint_3d.x_l;
	robotY = ai_waypoint_3d.y_l;
	robotPsi = ai_waypoint_3d.psi_l;
	radius = sqrt(pow((robotX-oppX),2) + pow((robotY-oppY),2));

	switch(lastCollision){
		case COLLISION_LEFT:
			cout << "COLLISION_LEFT!" << endl;
			if(robotPsi>=-45*M_PI/180 && robotPsi<45*M_PI/180){
				if((TABLE_SIZE_Y - oppY) > WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if((TABLE_SIZE_Y - oppY) <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
			}
			else if(robotPsi>=45*M_PI/180 && robotPsi<135*M_PI/180){
				if(oppX > WAYPOINT_SAFETY_DISTANCE ){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(oppX <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
			}
			else if(robotPsi>=135*M_PI/180 || robotPsi<-135*M_PI/180){
				if(oppY > WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(oppY <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
			}
			else{
				if((TABLE_SIZE_X - oppX) > WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if((TABLE_SIZE_X - oppX) <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
			}
			break;
		case COLLISION_RIGHT:
			cout << "COLLISION_RIGHT!" << endl;
			if(robotPsi>=-45*M_PI/180 && robotPsi<45*M_PI/180){
				if(oppY > WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(oppY <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
			}
			else if(robotPsi>=45*M_PI/180 && robotPsi<135*M_PI/180){
				if((TABLE_SIZE_X - oppX) > WAYPOINT_SAFETY_DISTANCE ){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if((TABLE_SIZE_X - oppX) <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
			}
			else if(robotPsi>=135*M_PI/180 || robotPsi<-135*M_PI/180){
				if((TABLE_SIZE_Y - oppY) > WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if((TABLE_SIZE_Y - oppY) <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
			}
			else{
				if(oppX > WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(oppX <= WAYPOINT_SAFETY_DISTANCE){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
			}
			break;
		case COLLISION_FRONT:
			cout << "COLLISION_FRONT!" << endl;
			if(robotPsi>=-45*M_PI/180 && robotPsi<45*M_PI/180){
				if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(((lastY - oppY) <= 0) && (oppY > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(((lastY - oppY) <= 0) && (oppY <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
			}
			else if(robotPsi>=45*M_PI/180 && robotPsi<135*M_PI/180){
				if(((lastX - oppX) > 0) && ((TABLE_SIZE_X - oppX) > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(((lastX - oppX) > 0) && ((TABLE_SIZE_X - oppX) <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(((lastX - oppX) <= 0) && (oppX > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(((lastX - oppX) <= 0) && (oppX <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
			}
			else if(robotPsi>=135*M_PI/180 || robotPsi<-135*M_PI/180){
				if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(((lastY - oppY) <= 0) && (oppY > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(((lastY - oppY) <= 0) && (oppY <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
			}
			else{
				if(((lastX - oppX) > 0 ) && ((TABLE_SIZE_X - oppX) > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
				else if(((lastX - oppX) > 0 ) && ((TABLE_SIZE_X - oppX) <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(((lastX - oppX) <= 0) && (oppX > WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_RIGHT);
				}
				else if(((lastX - oppX) <= 0) && (oppX <= WAYPOINT_SAFETY_DISTANCE)){
					_subject->CalculateHalfCircleWaypoints(radius, COLLISION_LEFT);
				}
			}
			break;
		case COLLISION_REAR:
			cout << "COLLISION_REAR!" << endl;
			newX = robotX;
			newY = robotY;
			newPsi = robotPsi;
			break;
		default:
			break;
	}
}

void AntiCollisionSupportStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  ){
	Init();
	clearWaypoints( navsysMessageHandler );
	strategyDone = FALSE;
	cout << "Anti collision support strategy" << endl;
	usleep(10000);
	navsysMessageHandler->handleAllMessagesNow();
	cout << timeLeft << endl;
	while(_subject->GetSupportNeeded() && !_subject->GetCollisionValue() && timeLeft>0 && !strategyDone){ 
	//while(_subject->GetSupportNeeded() && timeLeft>0 && !strategyDone){ 
		switch(state){		
			case INIT:				
				requestPosition(navsysMessageHandler);
				usleep(10000);					
				CalculateSupportWaypoint();
				state = SUPPORT;
				break;
			case SUPPORT:
				if(!lastWaypointSent){
					for(int i=0; i<NUMBER_OF_SUPPORT_WAYPOINTS;i++){
						activeWaypoint = _subject->GetSupportWaypoint(i);
						usleep(10000);
						cout << "Support waypoint: x: " << activeWaypoint.x_l << ", y: " << activeWaypoint.y_l << ", psi: " << activeWaypoint.psi_l << endl;
						usleep(10000);
						sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
						usleep(10000);
					}
					usleep(500000);
					_subject->SetAntiCollisionActive(TRUE);
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(collisionStop){
					collisionStop = FALSE;
					clearWaypoints(navsysMessageHandler);
					_subject->SetAntiCollisionActive(FALSE);
					sendReverseWaypoint(navsysMessageHandler, -200, 0, 0, 50, RELATIVE_WAYPOINT);
					usleep(10000);
				}
				if(lastWaypointReached){
					lastWaypointReached = FALSE;
					lastWaypointSent = FALSE;
					state = ANGLE_FIX;
					usleep(10000);
				}
				break;
			case ANGLE_FIX:
				if(!lastWaypointSent){
					requestPosition(navsysMessageHandler);
					usleep(10000);
					activeWaypoint.id = 1;					
					activeWaypoint.x_l = ai_waypoint_3d.x_l;
					activeWaypoint.y_l = ai_waypoint_3d.y_l;
					activeWaypoint.psi_l = atan2((lastActiveWaypoint.y_l - activeWaypoint.y_l), (lastActiveWaypoint.x_l - activeWaypoint.x_l));
					usleep(10000);
					sendRobotWaypoint( navsysMessageHandler, &activeWaypoint );
					usleep(10000);
					lastWaypointSent = TRUE;
				}
				if(lastWaypointReached){
					_subject->SetAntiCollisionActive(TRUE);
					_subject->SetCollision(FALSE);
					lastWaypointReached = FALSE;
					lastWaypointSent = FALSE;	
					_subject->SetSupportNeeded(FALSE);
					usleep(10000);
				}
				break;
			default:
				break;
		}
		usleep(STRATEGY_WHILE_DELAY);
		CheckForCollision(navsysMessageHandler);
		local_time_message = _subject->GetTimeObservation();
		timeLeft = local_time_message.timeLeft;
		//UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
	priority = 0;
	_subject->SetSupportNeeded(FALSE);
}

