#include <math.h>

#include "ContainerStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

ContainerStrategy::ContainerStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	state = INIT;
	strategyDone = FALSE;
	priority = 5;
	internalState = 0;
	timeLeft = 90;
	Init();
	_subject = s;
	_subject->Attach(this);
	cout << "Strategy added" << endl;
}

ContainerStrategy::~ContainerStrategy (){
 	_subject->Detach(this);
}

void ContainerStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	state = INIT;
	strategyDone = FALSE;
	priority = 5;
	internalState = 0;
	timeLeft = 90;
	Init();
	cout << "Strategy reset!" << endl;
}

void ContainerStrategy::Init(){
	leftSwitch = FALSE;
	rightSwitch = FALSE;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	collision = FALSE;
	ballsReleased = FALSE;
	minDistance = FALSE;
	adjustCount = 0;
}

void ContainerStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision = TRUE;
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation(); 
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;	
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_SWITCHES){
		int side = _subject->GetSwitchMessage();
		if(side == 0){
			leftSwitch = TRUE;
		}
		else if(side == 1){
			rightSwitch = TRUE;
		}
	}
	else if( typeOfObservation == MINOR_BALLS_RELEASED ){
		ball_message my_ball_message;
		my_ball_message = _subject->GetBallsReleasedMessage();
		for(int i=0;i<5;i++){
			if(my_ball_message.data[i] == 0){
				balls.RemoveRobotBalls(WHITE_ROBOT_BALL,1);
			}
			else{
				balls.RemoveRobotBalls(RED_BLUE_ROBOT_BALL,1);
			}
		}
		balls.SetLastBallColor(my_ball_message.data[4]);
		ballsReleased = TRUE;
	}
}

int ContainerStrategy::GetPriority(){
	return priority;
}

void ContainerStrategy::UpdatePriority(){
	local_time_message = _subject->GetTimeObservation();
	
	timeLeft = local_time_message.timeLeft;
	//cout << "Time left: " << timeLeft << endl;

	internalState = balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL) + balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL);
	//This strategy should have a priority based only on the content of the robot, and the time left. That means that a priority of 1 or 9 should work. All other strategies will be 0 when time left is below a given value. In this way this is the only possible strategy after the time limit.
	if(internalState == 5){
		priority = 9;
	}
	else if(timeLeft<15){
		priority = 9;
	}
	else if(balls.GetNumberOfRobotBalls(ALL)>=3){
		priority = 5;
	}
	else{
		priority = 1;
	}
}

void ContainerStrategy::CheckForCollision(MessageHandler *navsysMessageHandler){
	int x,y,r;	
	if(collision){
		requestPosition(navsysMessageHandler);
		usleep(10000);
		if(teamColor == TEAM_COLOR_BLUE){
			if((ai_waypoint_3d.x_l > 2000) && (ai_waypoint_3d.y_l > 1400) ){
				collision = FALSE;
				cout << "Collision message ignored!" << endl;
				return;
			}
		}
		else if(teamColor == TEAM_COLOR_RED){
			if((ai_waypoint_3d.x_l < 1000) && (ai_waypoint_3d.y_l > 1400) ){
				collision = FALSE;				
				cout << "Collision message ignored!" << endl;
				return;
			}
		}
		/*x = ai_waypoint_3d.x_l;
		y = ai_waypoint_3d.x_l;
		if(teamColor == TEAM_COLOR_RED){
			r = sqrt(pow((x-50),2)+pow((y-2100),2));
		}
		else if(teamColor == TEAM_COLOR_BLUE){
			r = sqrt(pow((x-2950),2)+pow((y-2100),2));
		}
		if(board4<=DISTANCE_MEASURE_THRESHOLD && board5<=DISTANCE_MEASURE_THRESHOLD){
			minDistance = TRUE;
		}
		else if(board1<=COLLISION_THRESHOLD || board2<=COLLISION_THRESHOLD || board3<=COLLISION_THRESHOLD){
			#ifdef ANTI_COLLISION_ACTIVE
				_subject->SetCollision(TRUE);
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		collision = FALSE;*/
		_subject->GetCollisionValues(navsysMessageHandler,boards);
		cout << "board1: " << boards[0] << ", board2: " << boards[1] << ", board3: " << boards[2] << endl;

		/*x = ai_waypoint_3d.x_l;
		y = ai_waypoint_3d.y_l;
		r = sqrt(pow(x,2)+pow((y-750),2));
		cout << "Radius: " << r << endl;*/

		if(boards[3]<=DISTANCE_MEASURE_THRESHOLD && boards[4]<=DISTANCE_MEASURE_THRESHOLD){
			//minDistance = TRUE;
		}
		if(boards[0]<=COLLISION_THRESHOLD || boards[1]<=COLLISION_THRESHOLD || boards[2]<=COLLISION_THRESHOLD){
			#ifdef ANTI_COLLISION_ACTIVE
				if(_subject->GetAntiCollisionActive()){
					_subject->SetCollision(TRUE);
					clearWaypoints(navsysMessageHandler);
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		collision = FALSE;
		return;
	}
}

void ContainerStrategy::EmptyRobot( MessageHandler *proxysysMessageHandler )
{
	int color;
	ball_message my_ball_message;
	if(balls.GetLastBallColor() == 0){
		color = teamColor;
	}
	else{
		color = 0;
	}
	cout << "Ball color first sorted: " << color << endl;
	my_ball_message.data[0] = color;
	sendReleaseBalls( &my_ball_message, proxysysMessageHandler );
}

void ContainerStrategy::AdjustPosition( MessageHandler *navsysMessageHandler ){
	collision_detection_message adjust_message;
	adjust_message.data[2] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	adjust_message.data[3] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	if(leftSwitch){
		adjust_message.data[0] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	}
	else if(!leftSwitch){
		adjust_message.data[0] = MINIMUM_DISPENSER_STOP_THRESHOLD + 20;
	}
	if(rightSwitch){
		adjust_message.data[1] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	}
	else if(!rightSwitch){
		adjust_message.data[1] = MINIMUM_DISPENSER_STOP_THRESHOLD + 20;
	}
	if(adjustCount == 0){
		adjust_message.data[4] = 1;
	}
	else{
		adjust_message.data[4] = 0;
	}
	sendAdjustPosition( &adjust_message, navsysMessageHandler );
	adjustCount ++;
}

void ContainerStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  ){
	if(strategyDone){
		strategyDone = FALSE;
		state = INIT;
	}
	clearWaypoints( navsysMessageHandler );
	usleep(10000);
	cout << "ContainerStrategy: state = INIT" << endl;
	cout << "Time left: " << timeLeft << endl;
	Init();
	_subject->SetAntiCollisionActive(TRUE);
	while(!strategyDone && !_subject->GetCollisionValue()  && timeLeft>-1){
		switch (state){
			case INIT:
				if(!lastWaypointSent){
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_OUT,proxysysMessageHandler);
					if(teamColor == RED){
						activeWaypoint = GetWaypoint(STANDARD_CONTAINER_RED_WAYPOINT);
					}
					else if(teamColor == BLUE){
						activeWaypoint = GetWaypoint(STANDARD_CONTAINER_BLUE_WAYPOINT);
					}
					lastWaypointReached = FALSE;
					sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
					_subject->SetLastWaypointSent(activeWaypoint);						
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				/*if(_subject->CheckPositionTimeout()){
					cout << "Position timeout" << endl;
					ai_waypoint lastWaypoint;					
					clearWaypoints(navsysMessageHandler);
					lastWaypoint = _subject->GetLastWaypointSent();
					if(lastWaypoint.id == 1){
						//_subject->SetRearAntiCollisionActive(TRUE);
						sendReverseWaypoint(navsysMessageHandler, -400, 0, 0, 50, RELATIVE_WAYPOINT);
					}
					else if(lastWaypoint.id == -1){
						activeWaypoint.id = 1;
						activeWaypoint.x_l = 300;
						activeWaypoint.y_l = 0;
						activeWaypoint.psi_l = 0;
						activeWaypoint.circle_of_acceptance = 50;						
						sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
					}
					strategyDone = TRUE;
				}*/
				if(lastWaypointReached){
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_OUT,proxysysMessageHandler);
					_subject->SetAntiCollisionActive(FALSE);					
					usleep(10000);
					lastWaypointReached = FALSE;
					minDistance = TRUE;
				}
				if(minDistance){
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_OUT,proxysysMessageHandler);
					clearWaypoints( navsysMessageHandler );
					#ifdef SWITCH_ADJUST
						state = ADJUST;
						adjustCount = 0;
					#else
						state = RELEASE_BALLS;
					#endif
					minDistance = FALSE;
					usleep(10000);
				}
				break;
			case ADJUST:
				//cout << "ContainerStrategy: state = ADJUST" << endl;
				AdjustPosition( navsysMessageHandler );
				if(leftSwitch && rightSwitch){
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_OUT,proxysysMessageHandler);
					leftSwitch = FALSE;
					rightSwitch = FALSE;
					lastWaypointSent = FALSE;
					lastWaypointReached = FALSE;

					state = RELEASE_BALLS;
					/*beacon_angles beaconAngles;
					beaconAngles.b1 = 228;
					beaconAngles.b2 = 145;
					beaconAngles.b3 = 271;
					proxysysMessageHandler->sendMessage((char *)&beaconAngles, sizeof(beaconAngles), MINOR_NAVSYS_UPDATE_BEACON_ANGLES, PRIORITY_HIGH);*/
					sendBeaconMeasurementRequest(navsysMessageHandler,BEACON_MEASURE_BLUE_CONTAINER,BEACON_MEASURE_START);
					requestPosition(navsysMessageHandler);
					EmptyRobot(proxysysMessageHandler);
					usleep(10000);
					sendNewPosition(navsysMessageHandler, &ai_waypoint_3d);
					usleep(10000);
				}
				break;
			case RELEASE_BALLS:
				if(ballsReleased){
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_OUT,proxysysMessageHandler);
					cout << "ContainerStrategy: state = RELEASE_BALLS" << endl;
					sendBeaconMeasurementRequest(navsysMessageHandler,BEACON_MEASURE_BLUE_CONTAINER,BEACON_MEASURE_STOP);
					usleep(10000);
					//while(1);
					state = REVERSE;
					_subject->SetRearAntiCollisionActive(TRUE);
					usleep(10000);
				}
				break;
			case REVERSE:
				if(!lastWaypointSent){
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_OUT,proxysysMessageHandler);
					cout << "ContainerStrategy: state = REVERSE" << endl;
					if(teamColor == TEAM_COLOR_RED){					
						sendReverseWaypoint( navsysMessageHandler, -300, 0, -M_PI/4, 50, RELATIVE_WAYPOINT);
					}
					else if(teamColor == TEAM_COLOR_BLUE){
						sendReverseWaypoint( navsysMessageHandler, -300, 0, M_PI*3/4, 50, RELATIVE_WAYPOINT);
					}
					lastWaypointSent = TRUE;
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				if(lastWaypointReached){
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_OUT,proxysysMessageHandler);
					_subject->SetAntiCollisionActive(TRUE);
					strategyDone = TRUE;
					lastWaypointReached = FALSE;
					_subject->SetRearAntiCollisionActive(FALSE);
					usleep(10000);
				}
				break;
			default:
				cout << "WRONG STATE!" << endl;
				break;
		}
		usleep(STRATEGY_WHILE_DELAY);
		//requestPosition(navsysMessageHandler);
		CheckForCollision(navsysMessageHandler); 
		//UpdatePriority();
		local_time_message = _subject->GetTimeObservation();
		timeLeft = local_time_message.timeLeft;
		navsysMessageHandler->handleAllMessagesNow();
	}
	_subject->SetFirstRunVariable(TRUE);
}
