#include <math.h>

#include "DispenserTestStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

DispenserTestStrategy::DispenserTestStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	blueWhiteDispenser = GetWaypoint(BLUE_WHITE_DISPENSER_WAYPOINT);
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = FIRST_RUN;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalState = 5;
	timeLeft = 90;
	//firstRun = TRUE;
	Init();
	_subject = s;
	_subject->Attach(this);
	_subject->SetFirstRunVariable(TRUE);
	cout << "Strategy added" << endl;
}

DispenserTestStrategy::~DispenserTestStrategy (){
 	_subject->Detach(this);
}

void DispenserTestStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	blueWhiteDispenser = GetWaypoint(BLUE_WHITE_DISPENSER_WAYPOINT);
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = FIRST_RUN;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalState = 5;
	timeLeft = 90;
	//firstRun = TRUE;
	_subject->SetFirstRunVariable(TRUE);
	Init();
	cout << "Strategy reset!" << endl;
}

void DispenserTestStrategy::Init(){
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	collision = FALSE;
	newBall = FALSE;
	ballStored = FALSE;
	wheelReady = FALSE;
	firstBall = TRUE;
	minDistanceDispenser = FALSE;
	adjustCount = 0;
}

int DispenserTestStrategy::GetPriority(){
	return priority;
}

void DispenserTestStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		if(!firstRun){		
			collision = TRUE;
		}
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation();
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_BALL_STORED ){
		cout << "Ball stored!" << endl;
		ballStored = TRUE;
	}
	else if( typeOfObservation == MINOR_WHEEL_READY){
		wheelReady = TRUE;
		firstBall = FALSE;
	}
	else if( typeOfObservation == MINOR_COLOR_DETECTED ){
		newBall = TRUE;
	}
}

void DispenserTestStrategy::CheckForCollision(){
	int x,y,r;
	if(collision){
		_subject->GetCollisionValues(boards);
		cout << "board4: " << boards[3] << ", board5: " << boards[4] << endl;

		/*x = ai_waypoint_3d.x_l;
		y = ai_waypoint_3d.y_l;
		r = sqrt(pow(x,2)+pow((y-750),2));
		cout << "Radius: " << r << endl;*/

		if(boards[3]<=DISTANCE_MEASURE_THRESHOLD && boards[4]<=DISTANCE_MEASURE_THRESHOLD){
			minDistanceDispenser = TRUE;
		}
		if(boards[0]<=COLLISION_THRESHOLD || boards[1]<=COLLISION_THRESHOLD || boards[2]<=COLLISION_THRESHOLD){
			#ifdef ANTI_COLLISION_ACTIVE
				_subject->SetCollision(TRUE);
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		collision = FALSE;
	}
}

void DispenserTestStrategy::UpdatePriority(){
	double maxDistance, minValue;
	int whiteBalls;
	local_time_message = _subject->GetTimeObservation();
	timeLeft = local_time_message.timeLeft;

	ai_waypoint_3d = _subject->GetPosObservation();

	internalState = balls.GetNumberOfBalls(BLUE_WHITE_BALL);

	if( !(int)internalState ){
		strategyCompleted = TRUE;
	}

	distance_x = sqrt(pow((blueWhiteDispenser.x_l - ai_waypoint_3d.x_l),2));
	distance_y = sqrt(pow((blueWhiteDispenser.y_l - ai_waypoint_3d.y_l),2));
	distance = sqrt(pow(distance_x,2)+pow(distance_y,2));
	maxDistance = sqrt(pow(3000,2)+pow(2100,2));

	whiteBalls = balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL);
	if(whiteBalls>=3){
		robotState = -5;
	}
	else if(whiteBalls<3 && whiteBalls>1){
		robotState = 3;
	}
	else{
		robotState = 5;
	}

	if( !strategyCompleted ){
		priority = ((((maxDistance-distance)/maxDistance)*0.2 + internalState*0.10 + robotState*0.06)*0.9)*11;
	}
	else{
		priority = 0;
	}
}

/*void DispenserTestStrategy::CheckBall( MessageHandler *proxysysMessageHandler ){
	ball_message my_ball_message;
	my_ball_message = _subject->GetBallMessage();
	ballColor = my_ball_message.data[0];
	cout << "Ball color received: " << ballColor << endl;
	if(ballColor == teamColor || ballColor == 0){
		my_ball_message.data[0] = 1;
	}
	else{
		my_ball_message.data[0] = 1;
	}
	//my_ball_message.data[1] = ballColor;
	my_ball_message.data[1] = 0;
	usleep(1000);
	sendBallAck( &my_ball_message, proxysysMessageHandler );
	cout << "Ball color ACK sent!" << endl;
}*/

void DispenserTestStrategy::AdjustPosition( MessageHandler *navsysMessageHandler ){
	collision_detection_message adjust_message;
	adjust_message.data[0] = boards[3];
	adjust_message.data[1] = boards[4];
	adjust_message.data[2] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	adjust_message.data[3] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	if(adjustCount == 0){
		adjust_message.data[4] = 1;
	}
	else{
		adjust_message.data[4] = 0;
	}
	sendAdjustPosition( &adjust_message, navsysMessageHandler );
	adjustCount ++;
}

void DispenserTestStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler ){
	int startTime = 0;
	if(strategyDone){
		strategyDone = FALSE;
		state = FIRST_RUN;
	}
	Init();
	while(!strategyCompleted && !strategyDone && !_subject->GetCollisionValue() && timeLeft>0){
		switch (state) {
			case FIRST_RUN:
				cout << "WhiteBlueDispenserStrategy: state = FIRST_RUN" << endl;
				if(!_subject->GetFirstRunVariable()){
					if(!lastWaypointSent){
						activeWaypoint = GetWaypoint(BLUE_WHITE_DISPENSER_SUPPORT_WAYPOINT);
						_subject->SetLastWaypointSent(activeWaypoint);
						lastWaypointSent = TRUE;
						lastWaypointReached = FALSE;
						usleep(10000);
					}
					if(lastWaypointReached){
						lastWaypointSent = FALSE;
						lastWaypointReached = FALSE;
						state = INIT;
						usleep(10000);
					}
				}
				else{
					state = INIT;			
				}
				break;
			case INIT:
				cout << "WhiteBlueDispenserStrategy: state = INIT" << endl;
				if(!lastWaypointSent){
					//sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					activeWaypoint = GetWaypoint(BLUE_WHITE_DISPENSER_WAYPOINT);
					activeWaypoint.id = 1;						
					activeWaypoint.x_l = 440;
					activeWaypoint.y_l = 0;
					activeWaypoint.psi_l = 0;
					activeWaypoint.circle_of_acceptance = 30;
					sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint);					
					//sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
					_subject->SetLastWaypointSent(activeWaypoint);
					cout << "First waypoint, x_l: " << activeWaypoint.x_l << endl;
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					//sendPositionArms(ARMS_READY,MOTOR_OFF,BELTS_DIRECTION_IN,proxysysMessageHandler);
					//startTime = timeLeft;
					/*#ifndef ADJUSTING
						state = PICK_UP_BALL;
					#endif*/
					lastWaypointReached = FALSE;
					ballStored = FALSE;
					sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					usleep(100000);
					sendReverseWaypoint( navsysMessageHandler, -100, 0, 0, 30, RELATIVE_WAYPOINT );
					while(!lastWaypointReached){
						navsysMessageHandler->handleAllMessagesNow();
					}
					lastWaypointReached = FALSE;
					state = PICK_UP_BALL;					
					usleep(10000);
				}
				if(minDistanceDispenser && lastWaypointReached){
					clearWaypoints( navsysMessageHandler );
					#ifdef ADJUSTING
						state = DISPENSER;
					#else
						state = PICK_UP_BALL;
					#endif
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				break;
			case DISPENSER:
				cout << "WhiteBlueDispenserStrategy: state = DISPENSER" << endl;
				if(!lastWaypointReached){
					AdjustPosition( navsysMessageHandler );
					usleep(10000);
				}
				else{
					sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					state = PICK_UP_BALL;
					//startTime = timeLeft;
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				break;
			case PICK_UP_BALL:
				cout << "WhiteBlueDispenserStrategy: state = PICK_UP_BALL" << endl;
				if(newBall){
					cout << "New ball picked up!" << endl;
					while(_subject->GetBallStored()<balls.GetNumberOfRobotBalls(ALL)){
						navsysMessageHandler->handleAllMessagesNow();
						usleep(1000);
					}
					/*if(!firstBall){	
						cout << "Not the first ball" << endl;					
						while(!ballStored){
							navsysMessageHandler->handleAllMessagesNow();
							usleep(10000);
						}					
					}*/
					_subject->CheckBall(proxysysMessageHandler, WHITE_BALL, BLUE);
					newBall = FALSE;
					ballStored = FALSE;
					//startTime = timeLeft;
					usleep(10000);
				}
				if(wheelReady){
					while(_subject->GetWheelReady()<balls.GetNumberOfRobotBalls(ALL)+1){
						navsysMessageHandler->handleAllMessagesNow();
						usleep(1000);
					}
					wheelReady = FALSE;
					usleep(BELT_INPUT_TIMER);
					cout << "New ball stored!" << endl;
					balls.AddRobotBalls(WHITE_ROBOT_BALL,1);
					balls.RemoveBall(BLUE_WHITE_BALL, 1);
					cout << "Number of white balls: " << balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL) << endl;
					if(balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL)<balls.GetNumberOfWhiteBallsToPickUp()){
						cout << "Picking up another white ball!" << endl;						
						sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
						//sleep(1);
						activeWaypoint.id = 1;
						activeWaypoint.x_l = 100;
						activeWaypoint.y_l = 0;
						activeWaypoint.psi_l = 0;
						activeWaypoint.circle_of_acceptance = 30;						
						sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
						while(!lastWaypointReached){
							navsysMessageHandler->handleAllMessagesNow();
						}
						lastWaypointReached = FALSE;
						state = PICK_UP_BALL;						
						sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
						usleep(10000);
						sendReverseWaypoint( navsysMessageHandler, -100, 0, 0, 30, RELATIVE_WAYPOINT );
						while(!lastWaypointReached){
							navsysMessageHandler->handleAllMessagesNow();
						}
						lastWaypointReached = FALSE;
					}
					else{
						cout << "Done picking up white balls!" << endl;
						sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_IN,proxysysMessageHandler);
						state = REVERSE;
						lastWaypointSent = FALSE;
						lastWaypointReached = FALSE;
						usleep(10000);
					}
				}
				/*if(startTime-timeLeft>TIME_OUT){
					newBall = FALSE;
					ballStored = FALSE;
					lastWaypointSent = FALSE;
					lastWaypointReached = FALSE;
					state = REVERSE;
					//state = PICK_UP_BALL;
					sendPositionArms(ARMS_READY,MOTOR_OFF,BELTS_DIRECTION_IN,proxysysMessageHandler);
					usleep(10000);
				}*/
				break;
			case REVERSE:
				cout << "WhiteBlueDispenserStrategy: state = REVERSE" << endl;
				if(!lastWaypointSent){
					sendReverseWaypoint( navsysMessageHandler, 700, 750, -M_PI/2, 50, GLOBAL_WAYPOINT);
					_subject->SetLastWaypointSent(activeWaypoint);					
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					strategyDone = TRUE;
					//firstRun = FALSE;
					_subject->SetFirstRunVariable(FALSE);
					//strategyCompleted = TRUE; //HACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACK
					lastWaypointReached = FALSE;
					startTime = timeLeft;
					usleep(10000);
				}
				break;
			default:
				cout << "Out of states!" << endl;
				break;
		}
		usleep(300000);
		requestPosition(navsysMessageHandler);
		CheckForCollision();
		UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
}
