#include <math.h>

#include "EmptyRobotTestStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

EmptyRobotTestStrategy::EmptyRobotTestStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	state = INIT;
	strategyDone = FALSE;
	priority = 5;
	internalState = 0;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	ballsReleased = FALSE;
	minDistance = FALSE;
	adjustCount = 0;
	timeLeft = 90;
	_subject = s;
	_subject->Attach(this);
	cout << "Strategy added" << endl;
}

EmptyRobotTestStrategy::~EmptyRobotTestStrategy (){
 	_subject->Detach(this);
}

void EmptyRobotTestStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	state = INIT;
	strategyDone = FALSE;
	priority = 5;
	internalState = 0;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	ballsReleased = FALSE;
	minDistance = FALSE;
	adjustCount = 0;
	timeLeft = 90;
	cout << "Strategy reset!" << endl;
}

void EmptyRobotTestStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision_message = _subject->GetCollisionMessage();
		board1 = collision_message.data[1];
		board2 = collision_message.data[2];
		board3 = collision_message.data[3];
		board4 = collision_message.data[4];
		board5 = collision_message.data[5];
		cout << "\n\nBoard1: " << board1 << endl;
		cout << "Board2: " << board2 << endl;
		cout << "Board3: " << board3 << endl;
		cout << "Board4: " << board4 << endl;
		cout << "Board5: " << board5 << endl;
		if(board4<=DISTANCE_MEASURE_THRESHOLD && board5<=DISTANCE_MEASURE_THRESHOLD){
			minDistance = TRUE;
		}
		else if(board1<=COLLISION_THRESHOLD || board2<=COLLISION_THRESHOLD || board3<=COLLISION_THRESHOLD){
			//_subject->SetCollision(TRUE);
		}
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation(); 
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;	
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_SWITCHES){
		int side = _subject->GetSwitchMessage();
		if(side == 0){
			leftSwitch = TRUE;
		}
		else if(side == 1){
			rightSwitch = TRUE;
		}
	}
	else if( typeOfObservation == MINOR_BALLS_RELEASED ){
		ball_message my_ball_message;
		my_ball_message = _subject->GetBallMessage();
		for(int i=0;i<5;i++){
			if(my_ball_message.data[i] == 0){
				balls.RemoveRobotBalls(WHITE_ROBOT_BALL,1);
			}
			else{
				balls.RemoveRobotBalls(RED_BLUE_ROBOT_BALL,1);
			}
		}
		balls.SetLastBallColor(my_ball_message.data[4]);
		ballsReleased = TRUE;
	}
}

int EmptyRobotTestStrategy::GetPriority(){
	return priority;
}

void EmptyRobotTestStrategy::UpdatePriority(){
	local_time_message = _subject->GetTimeObservation();
	
	timeLeft = local_time_message.timeLeft;
	cout << "Time left: " << timeLeft << endl;

	internalState = balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL) + balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL);

	if(internalState == 5){
		priority = 9;
	}
	else if(timeLeft<10){
		priority = 9;
	}
	else{
		priority = 1;
	}
}

void EmptyRobotTestStrategy::AdjustPosition( MessageHandler *navsysMessageHandler ){
	collision_detection_message adjust_message;
	adjust_message.data[2] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	adjust_message.data[3] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	if(lastWaypointReached){
		if(leftSwitch){
			adjust_message.data[0] = MINIMUM_DISPENSER_STOP_THRESHOLD;
		}
		else if(!leftSwitch){
			adjust_message.data[0] = MINIMUM_DISPENSER_STOP_THRESHOLD + 5;
		}
		if(rightSwitch){
			adjust_message.data[1] = MINIMUM_DISPENSER_STOP_THRESHOLD;
		}
		else if(!rightSwitch){
			adjust_message.data[1] = MINIMUM_DISPENSER_STOP_THRESHOLD + 5;
		}
	}
	else{
		adjust_message.data[0] = board4;
		adjust_message.data[1] = board5;
	}
	if(adjustCount == 0){
		adjust_message.data[4] = 1;
	}
	else{
		adjust_message.data[4] = 0;
	}
	sendAdjustPosition( &adjust_message, navsysMessageHandler );
	adjustCount ++;
}

void EmptyRobotTestStrategy::PositionArms( int angle, int dir, int onOff, MessageHandler *proxysysMessageHandler ){
	ball_message my_ball_message;
	my_ball_message.data[0] = angle;
	my_ball_message.data[1] = dir;
	my_ball_message.data[2] = onOff;
	sendSetArms( &my_ball_message, proxysysMessageHandler );
}

void EmptyRobotTestStrategy::EmptyRobot( MessageHandler *proxysysMessageHandler )
{
	int color;
	ball_message my_ball_message;
	if(balls.GetLastBallColor() == 0){
		color = 1;
	}
	else{
		color = 0;
	}
	my_ball_message.data[0] = color;
	sendReleaseBalls( &my_ball_message, proxysysMessageHandler );
}

void EmptyRobotTestStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  ){
	strategyDone = FALSE;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	ballsReleased = FALSE;
	minDistance = FALSE;
	adjustCount = 0;
	while(!strategyDone && !_subject->GetCollisionValue()  && timeLeft>0){
		switch (state){
			case INIT:
				cout << "EmptyRobotTestStrategy: state = INIT" << endl;
				if(!lastWaypointSent){
					PositionArms(0,1,0,proxysysMessageHandler);
					activeWaypoint.id = -1;
					activeWaypoint.x_l = -500;
					activeWaypoint.y_l = -500;
					activeWaypoint.psi_l = 0;
					sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					usleep(300000);
					EmptyRobot( proxysysMessageHandler );
					lastWaypointReached = FALSE;
					lastWaypointSent = FALSE;
					state = RELEASE_BALLS;
					usleep(10000);
				}
				if(minDistance){
					clearWaypoints( navsysMessageHandler );
					state = ADJUST;
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				break;
			case ADJUST:
				if(!lastWaypointReached){
					AdjustPosition( navsysMessageHandler );
				}
				else if(lastWaypointReached){
					adjustCount = 0;
					AdjustPosition( navsysMessageHandler );
					if(leftSwitch || rightSwitch){
						AdjustPosition( navsysMessageHandler );
						if(leftSwitch && rightSwitch){
							leftSwitch = FALSE;
							rightSwitch = FALSE;
							lastWaypointSent = FALSE;
							lastWaypointReached = FALSE;
							state = RELEASE_BALLS;
							usleep(10000);
						}
					}
				}
				break;
			case RELEASE_BALLS:
				cout << "EmptyRobotTestStrategy: state = RELEASE_BALLS" << endl;
				if(ballsReleased){
					usleep(50000);
					strategyDone = TRUE;
					//sendReverseCommand(navsysMessageHandler);
				}
				break;
			default:
				cout << "WRONG STATE!" << endl;
				break;
		}
		usleep(300000); 
		UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
}
