#ifndef PLANSYS_OBSERVATION_H
#define PLANSYS_OBSERVATION_H

#include "Observer.h"
#include "planner.h"
#include "observations.h"
#include "waypoint.h"
#include "MessageHandler.h"
#include "posix_message_queue.h"
#include "posix_message_config.h"

class PlansysObservation : public Subject {
  public:
	PlansysObservation();
	ai_waypoint GetPosObservation();
	ai_waypoint GetWaypointReachedObservation();
	cv_observation GetCvObservation();
	cv_observation GetCvColorObservation();
	ball_message GetNewBallMessage();
	ball_message GetBallsReleasedMessage();
	int GetSwitchMessage();
	time_message GetTimeObservation();
	proxy_message GetProxyMessage();
	collision_detection_message GetCollisionMessage();
	int NewObservation(pmq_message pm, int typeOfObservation);
	void ResetStrategies( int teamColor );
	int GetStrategyPriority();
	int GetFirstRunVariable();
	void GetCollisionValues(MessageHandler *navsysMessageHandler, int* boards);
	int GetAntiCollisionActive();
	int GetRearAntiCollisionActive();
	int CheckBall( MessageHandler *proxysysMessageHandler, int nextBallColor, int preferredColor );
	int PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler, int strategy );
	void UpdateStrategyPriority();
	void InitCollisionMessage();
	void SetAntiCollisionActive(int value);
	void SetRearAntiCollisionActive(int value);
	void SetLastCollision(int where);
	void SetLastWaypointSent(ai_waypoint lastWaypoint);
	void SetFirstRunVariable(int value);
	void SetSupportNeeded(int value);
	void SetLastWaypointReachedTag(int value);
	void IncrementLastWaypointSentCounter();
	void AngleWrap();
	ai_waypoint GetLastWaypointSent();
	ai_waypoint GetLocalWaypoint();
	ai_waypoint CheckWaypointFeasibility(ai_waypoint waypoint, ai_waypoint my_position);
	
	int CheckPositionTimeout();

	int GetLastCollision();
	int GetSupportNeeded();
	int GetNewBall();
	int GetWheelReady();
	int GetBallStored();
	int GetLastWaypointSentCounter();
	int GetLastWaypointReachedTag();
	ai_waypoint GetSupportWaypoint(int index);
	void CalculateHalfCircleWaypoints(int radius, int where);

	void SetCollision(int choice);
	int GetCollisionValue();
	/*void SetNewBallFalse();
	void SetWheelReadyFalse();
	void SetBallStoredFalse();*/

	//vector<int> interruptedStrategies;

  private:
	int lastCollision;
	int board1Buffer[NUMBER_OF_BOARDS];
	int board2Buffer[NUMBER_OF_BOARDS];
	int board3Buffer[NUMBER_OF_BOARDS];
	int bufferCounter;

	ai_waypoint positions[POSITION_TIMER_BUFFER_SIZE];

	int board1Average;
	int board2Average;
	int board3Average;

	int supportNeeded;
	int antiCollisionActive;
	int rearAntiCollisionActive;

	int firstRun;
	int ballStored;
	int wheelReady;
	int newBall;

	int index;

	int collision;

	int lastWaypointSentCounter;
	int lastWaypointReachedTag;

	ai_waypoint ai_waypoint_3d, *ai_waypoint_3d_ptr;
	ai_waypoint last_ai_waypoint_3d_reached;
	ai_waypoint lastWaypointSent;
	ai_waypoint localWaypoint;
	ai_waypoint waypointArray[NUMBER_OF_SUPPORT_WAYPOINTS];

	cv_observation my_cv_observation;
	cv_observation color_cv_observation;
	ball_message new_ball_message;
	ball_message balls_released_message;
	proxy_message my_proxy_message;
	collision_detection_message collision_message;
	switch_message my_switch_message;
	time_message main_time;
};

#endif
