#ifndef RED_BLUE_DISPENSER_STRATEGY_H
#define RED_BLUE_DISPENSER_STRATEGY_H

#include <math.h>

#include "Observer.h"
#include "PlansysObservation.h"
#include "planner.h"
#include "waypoint.h"
#include "posix_message_config.h"
#include "init.h"

class RedBlueDispenserStrategy: public Observer {
  public:
	RedBlueDispenserStrategy( PlansysObservation * , int theTeamColor );
	~RedBlueDispenserStrategy();
	void Reset( int theTeamColor );
	void Init();
	void Update( int typeOfObservation );
	int GetPriority();
	void UpdatePriority();
	void pickupBalls(int color);
	void CheckForCollision(MessageHandler *navsysMessageHandler);
	void CheckBall( MessageHandler *proxysysMessageHandler, MessageHandler *navsysMessageHandler, int nextBallColor);
	void AdjustPosition( MessageHandler *navsysMessageHandler );
	void PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  );
  
  private:
	int lastWaypointSent;	
	int lastWaypointReached;
	int newBall;
	int ballStored;
	int wheelReady;
	int firstBall;

	int numberOfBallsToPickUp;
	int firstRun;
	int numberOfTries;
	
	int tryCounter;
	int lastTryOk;
	int tryAgain;
	int ballCheck;

	collision_detection_message collision_message;		
	int strategyDone,strategyCompleted;
	int collision;
	int adjustCount;
	int minDistanceDispenser;

	int board1, board2, board3, board4, board5;
	int boards[NUMBER_OF_BOARDS];
	int ballColor;
	
	int robotState;
	int extraBall;
	double internalState;
	double distance;
	double distance_x;
	double distance_y;
	int timeLeft;
	int priority;
	
	int psi;

	int state;
	enum{
		FIRST_RUN,
		INIT,
		DISPENSER,
		RED_DISPENSER,
		BLUE_DISPENSER,
		PICK_UP_BALL,
		REVERSE,
	};

	PlansysObservation *_subject;
	ai_waypoint ai_waypoint_3d;
	ai_waypoint last_ai_waypoint;
	ai_waypoint activeWaypoint;
	ai_waypoint redDispenser;
	ai_waypoint blueDispenser;
	
	cv_observation my_cv_observation;
	cv_observation battery_bin_cv_observation;
	ball_message my_ball_message;
	proxy_message my_proxy_message;
	time_message local_time_message;
};

#endif
