#include <math.h>

#include "StartStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

extern int gameResetMessageMain;

StartStrategy::StartStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	Init();
	initWaypoints();
	internalState = 1;
	strategyDone = FALSE;	
	state = INIT;
	priority = 9;
	timeLeft = 90;
	_subject = s;
	_subject->Attach(this);
	cout << "Strategy added" << endl;
}

StartStrategy::~StartStrategy (){
 	_subject->Detach(this);
}

void StartStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	Init();
	initWaypoints();
	internalState = 1;
	strategyDone = FALSE;	
	state = INIT;
	priority = 9;
	timeLeft = 90;
	cout << "Strategy reset!" << endl;
}

void StartStrategy::Init(){
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	collision = FALSE;
	newBall = FALSE;
	ballStored = FALSE;
	wheelReady = FALSE;
	firstBall = TRUE;
	ballCheck = 0;
}

void StartStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		//collision = TRUE;
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation(); 
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;	
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_BALL_STORED ){
		ballStored = TRUE;
	}
	else if( typeOfObservation == MINOR_WHEEL_READY){
		wheelReady = TRUE;
		firstBall = FALSE;	
	}
	else if( typeOfObservation == MINOR_COLOR_DETECTED ){
		newBall = TRUE;
	}
}

void StartStrategy::UpdatePriority(){	
	//cout << "Time left: " << timeLeft << endl;
	
	if((balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL)+balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL)) == 5)
	{
		strategyDone = TRUE;
	}
	if( timeLeft<10 ){
		strategyDone = TRUE;
	}
	if( !(int)strategyDone ){
		priority = 9;
	}
	else{
		priority = 0;
	}
}

int StartStrategy::GetPriority(){
	return priority;
}

void StartStrategy::CheckForCollision(MessageHandler *navsysMessageHandler){
	if(collision){	
		_subject->GetCollisionValues(navsysMessageHandler,boards);
		cout << "board4: " << boards[3] << ", board5: " << boards[4] << endl;

		/*x = ai_waypoint_3d.x_l;
		y = ai_waypoint_3d.y_l;
		r = sqrt(pow(x,2)+pow((y-750),2));
		cout << "Radius: " << r << endl;*/

		if(boards[3]<=DISTANCE_MEASURE_THRESHOLD && boards[4]<=DISTANCE_MEASURE_THRESHOLD){
			//minDistanceDispenser = TRUE;
		}
		if(boards[0]<=COLLISION_THRESHOLD || boards[1]<=COLLISION_THRESHOLD){
			#ifdef ANTI_COLLISION_ACTIVE
				if(_subject->GetAntiCollisionActive()){
					_subject->SetCollision(TRUE);
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		else if(boards[2]<=COLLISION_THRESHOLD){
			#ifdef REAR_ANTI_COLLISION_ACTIVE
				if(_subject->GetRearAntiCollisionActive()){
					_subject->SetCollision(TRUE);
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
			
		}
		collision = FALSE;
	}
}

void StartStrategy::CheckBall( MessageHandler *proxysysMessageHandler, MessageHandler *navsysMessageHandler, int nextBallColor){
	int preferredColor;
	if(firstBall){
		if(teamColor == TEAM_COLOR_BLUE){
			preferredColor = BLUE;
		}
		else if(teamColor == TEAM_COLOR_RED){
			preferredColor = RED;	
		}
	}	
	else{
		preferredColor = WHITE;
	}
	ballCheck = _subject->CheckBall(proxysysMessageHandler, nextBallColor, preferredColor);
	if(ballCheck>-1){
		my_ball_message = _subject->GetNewBallMessage();
		my_ball_message.data[1] = preferredColor;
		sendBallAck( &my_ball_message, proxysysMessageHandler );
		if(preferredColor == WHITE){
			balls.AddRobotBalls(WHITE_ROBOT_BALL,1);
		}
		else{
			balls.AddRobotBalls(RED_BLUE_ROBOT_BALL,1);
		}
	}
	else{
		sendRelativeWaypoint(navsysMessageHandler, 0, 0, -45*M_PI/180, 50);
		while(!lastWaypointReached){
			navsysMessageHandler->handleAllMessagesNow();
			usleep(1000);
		
		}
		lastWaypointReached = FALSE;
		sendBallAck( &my_ball_message, proxysysMessageHandler );
		sendRelativeWaypoint(navsysMessageHandler, 0, 0, 45*M_PI/180, 50);
		while(!lastWaypointReached){
			navsysMessageHandler->handleAllMessagesNow();
			usleep(1000);
		}
		lastWaypointReached = FALSE;
	}
}

void StartStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler ){
	int startTime = 0;
	int startTime2 = 0;
	clearWaypoints( navsysMessageHandler );
	cout << "StartStrategy: state = INIT" << endl;
	cout << "Time left: " << timeLeft << endl;
	Init();
	while(!strategyDone && !_subject->GetCollisionValue() && timeLeft>30 && !gameResetMessageMain ){
		switch (state) {
			case INIT:
				cout << "Time left: " << timeLeft << endl;
				if(!lastWaypointSent){
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					if(teamColor == TEAM_COLOR_RED){					
						activeWaypoint = GetWaypoint(FIRST_RED_BALL_WAYPOINT);
					}
					else if(teamColor == TEAM_COLOR_BLUE){
						activeWaypoint = GetWaypoint(FIRST_BLUE_BALL_WAYPOINT);	
					}
					sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					lastWaypointReached = FALSE;
					if(teamColor == TEAM_COLOR_RED){					
						activeWaypoint = GetWaypoint(FIRST_WHITE_BALL_RED_WAYPOINT);
					}
					else if(teamColor == TEAM_COLOR_BLUE){
						activeWaypoint = GetWaypoint(FIRST_WHITE_BALL_BLUE_WAYPOINT);	
					}
					startTime = timeLeft;
					usleep(10000);
				}
				if(newBall){
					CheckBall(proxysysMessageHandler,navsysMessageHandler,WHITE_BALL);
					newBall = FALSE;
					firstBall = FALSE;
					usleep(2*BELT_INPUT_TIMER);			
					state = FIRST_WHITE_BALL;
					sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
					lastWaypointSent = TRUE;
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					usleep(10000);
				}
				if(startTime-timeLeft>TIME_OUT){
					cout << "Timing out!!" << endl;
					newBall = FALSE;
					firstBall = FALSE;
					lastWaypointSent = FALSE;
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					state = FIRST_WHITE_BALL;
					sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
					lastWaypointSent = TRUE;
					usleep(10000);
				}					
				break;
			case FIRST_WHITE_BALL:
				if(lastWaypointReached){
					cout << "StartStrategy: state = WHITE_BALL" << endl;					
					sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					lastWaypointReached = FALSE;
					startTime2 = timeLeft;
					usleep(10000);
				}
				if(newBall){
					cout << "Found a white ball" << endl;
					CheckBall(proxysysMessageHandler,navsysMessageHandler,WHITE_BALL);
					newBall = FALSE;
					strategyDone = TRUE;
					usleep(3*BELT_INPUT_TIMER);
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					usleep(10000);
				}
				if(startTime2-timeLeft>2*TIME_OUT){
					cout << "Timing out..." << endl;
					newBall = FALSE;
					lastWaypointSent = FALSE;
					strategyDone = TRUE;
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					usleep(10000);
				}	
				break;
			default:
				cout << "WRONG STATE!" << endl;
		}
		usleep(STRATEGY_WHILE_DELAY); 
		CheckForCollision(navsysMessageHandler);
		local_time_message = _subject->GetTimeObservation();
		timeLeft = local_time_message.timeLeft;
		//UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
}
