#include <math.h>

#include "TestStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

TestStrategy::TestStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = INIT;
	collision = FALSE;
	minDistanceDispenser = FALSE;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalState = 5;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	newBall = FALSE;
	ballStored = FALSE;
	adjustCount = 0;
	timeLeft = 90;
	_subject = s;
	_subject->Attach(this);
	cout << "Strategy added" << endl;
}

TestStrategy::~TestStrategy (){
 	_subject->Detach(this);
}

void TestStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = INIT;
	minDistanceDispenser = FALSE;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalState = 5;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	newBall = FALSE;
	ballStored = FALSE;
	adjustCount = 0;
	timeLeft = 90;
	cout << "Strategy reset!" << endl;
}

int TestStrategy::GetPriority(){
	return priority;
}

void TestStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision_message = _subject->GetCollisionMessage();
		board1 = collision_message.data[1];
		board2 = collision_message.data[2];
		board3 = collision_message.data[3];
		board4 = collision_message.data[4];
		board5 = collision_message.data[5];
		cout << "\n\nBoard1: " << board1 << endl;
		cout << "Board2: " << board2 << endl;
		cout << "Board3: " << board3 << endl;
		cout << "Board4: " << board4 << endl;
		cout << "Board5: " << board5 << endl;
		if(board4<=DISTANCE_MEASURE_THRESHOLD && board5<=DISTANCE_MEASURE_THRESHOLD){
			minDistanceDispenser = TRUE;
		}
		else if(board1<=COLLISION_THRESHOLD || board2<=COLLISION_THRESHOLD || board3<=COLLISION_THRESHOLD){
			//_subject->SetCollision(TRUE);
		}
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation(); 
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;	
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_BALL_STORED ){
		ballStored = TRUE;
	}
	else if( typeOfObservation == MINOR_COLOR_DETECTED ){
		newBall = TRUE;
	}
}

void TestStrategy::UpdatePriority(){
	double maxDistance;
	int redBlueBalls;
	local_time_message = _subject->GetTimeObservation();
	timeLeft = local_time_message.timeLeft;

	if((balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL)+balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL)) >= 5)
	{
		strategyDone = TRUE;
	}
	if( timeLeft<10 ){
		strategyDone = TRUE;
	}
	if( strategyCompleted )
	{
		strategyDone = TRUE;
		priority = 0;
	}
	if( !(int)strategyDone ){
		priority = 0;
	}
	else{
		priority = 0;
	}
}

void TestStrategy::PositionArms( int angle, int dir, int onOff, MessageHandler *proxysysMessageHandler ){
	ball_message my_ball_message;
	my_ball_message.data[0] = angle;
	my_ball_message.data[1] = dir;
	my_ball_message.data[2] = onOff;
	sendSetArms( &my_ball_message, proxysysMessageHandler );
}

void TestStrategy::CheckBall( MessageHandler *proxysysMessageHandler ){
	ball_message my_ball_message;
	my_ball_message = _subject->GetBallMessage();
	ballColor = my_ball_message.data[0];
	if(ballColor == teamColor || ballColor == 0){
		my_ball_message.data[0] = 1;
	}
	else{
		my_ball_message.data[0] = 0;
	}
	my_ball_message.data[1] = ballColor;
	sendBallAck( &my_ball_message, proxysysMessageHandler );
}

void TestStrategy::AdjustPosition( MessageHandler *navsysMessageHandler ){
	collision_detection_message adjust_message;
	adjust_message.data[0] = board4;
	adjust_message.data[1] = board5;
	cout << "board4: " << board4 << ", board5: " << board5 << endl;
	adjust_message.data[2] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	adjust_message.data[3] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	cout << "board4: " << board4 << ", board5: " << board5 << endl;
	if(adjustCount == 0){
		adjust_message.data[4] = 1;
	}
	else{
		adjust_message.data[4] = 0;
	}
	sendAdjustPosition( &adjust_message, navsysMessageHandler );
	adjustCount ++;
}

void TestStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler ){
	int startTime = 0;
	minDistanceDispenser = FALSE;
	strategyDone = FALSE;
	lastWaypointSent = TRUE;
	lastWaypointReached = FALSE;
	newBall = FALSE;
	ballStored = FALSE;
	adjustCount = 0;
	while(!strategyDone && !_subject->GetCollisionValue() && timeLeft>0){
		switch (state) {
			case INIT:
				cout << "TestStrategy: state = INIT" << endl;
				if(!lastWaypointSent){
					PositionArms(126,1,0,proxysysMessageHandler);
					activeWaypoint.id = 1;
					activeWaypoint.id = 1;
					activeWaypoint.x_l = 500;
					activeWaypoint.y_l = 500;
					activeWaypoint.psi_l = 0;
					sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					PositionArms(243,1,1,proxysysMessageHandler);
					state = PICK_UP_BALL;
					lastWaypointReached = FALSE;
					startTime = timeLeft;
					usleep(10000);
				}
				if(minDistanceDispenser){
					clearWaypoints( navsysMessageHandler );
					state = DISPENSER;
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				break;
			case DISPENSER:
				cout << "TestStrategy: state = DISPENSER" << endl;
				if(!lastWaypointReached){
					AdjustPosition( navsysMessageHandler );
					usleep(10000);
				}
				else{
					PositionArms(90,1,1,proxysysMessageHandler);
					state = PICK_UP_BALL;
					startTime = timeLeft;
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				break;
			case PICK_UP_BALL:
				cout << "TestStrategy: state = PICK_UP_BALL" << endl;
				if(newBall){
					CheckBall(proxysysMessageHandler);
					newBall = FALSE;
					startTime = timeLeft;
					usleep(10000);
				}
				if(ballStored){
					PositionArms(45,1,0,proxysysMessageHandler);
					balls.AddRobotBalls(RED_BLUE_ROBOT_BALL,1);
					balls.RemoveBall(RED_BALL, 1);
					ballStored = FALSE;
					if(balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL)<(5-balls.GetNumberOfWhiteBallsToPickUp())){
						state = PICK_UP_BALL;
						usleep(10000);
						PositionArms(90,1,1,proxysysMessageHandler);
					}
					else{
						strategyDone = TRUE;
						lastWaypointSent = FALSE;
						PositionArms(45,1,0,proxysysMessageHandler);
						//sendReverseCommand(navsysMessageHandler);
						usleep(10000);
					}
				}
				if(startTime-timeLeft>TIME_OUT){
					newBall = FALSE;
					ballStored = FALSE;
					lastWaypointSent = FALSE;
					strategyCompleted = TRUE;
					PositionArms(45,1,0,proxysysMessageHandler);
					//sendReverseCommand(navsysMessageHandler);
					usleep(10000);
				}
				break;
			default:
				cout << "Wrong state!" << endl;
				break;
		}
		usleep(300000); 
		UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
}

