#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <cstdlib>
#include <sys/time.h>
#include <math.h>

#include "posixCommunication.h"
#include "MessageHandler.h"
#include "posix_message_queue.h"
#include "posix_message_config.h"
#include "observations.h"
#include "Observer.h"
#include "PlansysObservation.h"
#include "StartStrategy.h"
#include "AntiCollisionStrategy.h"
#include "ContainerStrategy.h"
#include "SearchForBallStrategy.h"
// #include "CooledContainerStrategy.h"
#include "WhiteRedDispenserStrategy.h"
#include "WhiteBlueDispenserStrategy.h"
#include "RedBlueDispenserStrategy.h"
// #include "HorizontalDispenserStrategy.h"
//#include "DispenserTestStrategy.h"
#include "AntiCollisionSupportStrategy.h"
#include "waypoint.h"
#include "Balls.h"

#include "planner.h"
#include "init.h"

int teamColor = 0;
int timeLeft = 90;
int receivedTeamColor = FALSE;

int collisionDetect = 0;
int askForCvObservationFlag = FALSE;

int receivedGameStartMessage = FALSE;
int gameResetMessageTimer = TRUE;
int gameResetMessageMain = TRUE;
int gameFirstStart = TRUE;
int lowBattery = FALSE;
int quit = FALSE;
int counter = 1;
int timerReady;

time_message main_time_struct, *main_time;

using namespace std;

Balls balls; // PASS PÅ HER!!!!!!!!!!! BlueWhiteBalls blir satt til time viss denne deklarasjonen er plassert under den under
PlansysObservation plansysObs;

extern "C" void* timer_thread(void *arg){
	int m;
	// Timer, turn off actuators after (GAME_TIME) sec
	MessageHandler *proxysysMessageHandler;
	proxysysMessageHandler = (MessageHandler *)arg;

	main_time = &main_time_struct;
	main_time->timeLeft = -1;
	main_time->gameTime = GAME_TIME;

	pmq_message pos_mes;
	pos_mes.to = -1;
	pos_mes.from = -1;
	pos_mes.major = -1;
	pos_mes.minor = -1;
	pos_mes.priority = -1;
	pos_mes.length = sizeof(int);

	int i = 0;
	timeval tim;
	double t1, t2, t;
	gettimeofday(&tim,NULL);
	t1 = tim.tv_sec;
	while(1){
		if(gameResetMessageMain == TRUE){
			usleep(50000);
			while(receivedTeamColor == FALSE){
				usleep(1000);
			}
			usleep(50000);
			while(receivedGameStartMessage == FALSE){
				usleep(1000);
			}
			i = 0;
			gettimeofday(&tim,NULL);
			t1 = tim.tv_sec;
			//proxysysMessageHandler->sendCommand( MINOR_CMD_SYSTEM_GO, PRIORITY_HIGHEST );
		}
		usleep(1000);
		gettimeofday(&tim,NULL);
		t2 = tim.tv_sec;
		t = t2 - t1;
		main_time->timeLeft = (main_time->gameTime - (int) t);
		timerReady = TRUE;		
		// Update time on the Agents
		i++;
		if(i > 10){
			pos_mes.length = main_time->timeLeft;
			plansysObs.NewObservation(pos_mes, GAME_TIME_MESSAGE);
			i = 0;
		}
		if(t2 > t1 + GAME_TIME){
			gameResetMessageTimer = TRUE;
			gameResetMessageMain = TRUE;
			receivedTeamColor = FALSE;
			receivedGameStartMessage = FALSE;
			cout << "Time is up " << endl;
			// Send Command turn off, this command will turn off all actuators
			proxysysMessageHandler->sendCommand( MINOR_CMD_SYSTEM_STOP, PRIORITY_HIGHEST );
			for (m = 0; m < 10; m++) {
				cout << "Time is up " << endl;
				cout << "Time is up " << endl;
			}
			// Set new time when reset
			while( gameResetMessageTimer == FALSE ) {
				i = 0;
				gettimeofday(&tim,NULL);
				t1 = tim.tv_sec;
			}
			break;
		}
	}
}

void shutdownSystem( MessageHandler *proxysysMessageHandler ) {
	proxysysMessageHandler->sendCommand( MINOR_CMD_SHUTDOWN_SYSTEM, PRIORITY_HIGHEST );
}

int initGame() {
	teamColor = 0;
	receivedTeamColor = FALSE;
	receivedGameStartMessage = FALSE;
	gameResetMessageTimer = TRUE;
	gameResetMessageMain = TRUE;
	gameFirstStart = TRUE;
	timerReady = FALSE;
	return 0;
}

int main(int argc, char *argv[]) {
	signal(SIGHUP, catch_sighup);

	int nextStrategy,i;
	timeval cv_timer;

	double cv_t1, cv_t2;
	gettimeofday(&cv_timer,NULL);
	cv_t1 = cv_timer.tv_sec;
	cv_t2 = cv_t1;

	initGame();

	// Define and clear speed limit
	speed_limit max_speed_limit;
	max_speed_limit.u_b = MAX_SPEED_MPS;
	max_speed_limit.r_b = 10.0;

	// Declare Message Handlers
	MessageHandler cvsysMessageHandler(PLANSYS_ID, PMQ_PLANSYS, PMQ_CVSYS, false);
	MessageHandler cvsysColorMessageHandler(PLANSYS_ID, PMQ_PLANSYS, PMQ_CVSYS_COLOR, false);
	MessageHandler proxysysMessageHandler(PLANSYS_ID, PMQ_PLANSYS, PMQ_PROXY, false);
	MessageHandler navsysMessageHandler(PLANSYS_ID, PMQ_PLANSYS, PMQ_NAVSYS, false);

	declareSubMessageHandlers(&navsysMessageHandler, &proxysysMessageHandler, &cvsysMessageHandler, &cvsysColorMessageHandler);

	gameResetMessageTimer = FALSE;

	proxysysMessageHandler.sendCommand( MINOR_CMD_SYSTEM_GO, PRIORITY_HIGHEST );

	gameFirstStart = FALSE;
	gameResetMessageMain = FALSE;

	waitForTeamColorMessage( &proxysysMessageHandler, &navsysMessageHandler );

	/** Initialize strategies */
	AntiCollisionStrategy antiCollisionStrategy( &plansysObs, teamColor );
	AntiCollisionSupportStrategy antiCollisionSupportStrategy( &plansysObs, teamColor );
	StartStrategy startStrategy( &plansysObs, teamColor );
	//TestStrategy testStrategy( &plansysObs, teamColor );
	//EmptyRobotTestStrategy emptyRobotTestStrategy( &plansysObs, teamColor );
	ContainerStrategy containerStrategy( &plansysObs, teamColor );
	WhiteRedDispenserStrategy whiteRedDispenserStrategy( &plansysObs, teamColor );
	WhiteBlueDispenserStrategy whiteBlueDispenserStrategy( &plansysObs, teamColor );
	RedBlueDispenserStrategy redBlueDispenserStrategy( &plansysObs, teamColor );
	SearchForBallStrategy searchForBallStrategy( &plansysObs, teamColor );
// 	CooledContainerStrategy cooledContainerStrategy( &plansysObs, teamColor );
// 	HorizontalDispenserStrategy horizontalDispenserStrategy( &plansysObs, teamColor );
	//DispenserTestStrategy dispenserTestStrategy( &plansysObs, teamColor );

	waitForGameStartMessage( &navsysMessageHandler );

	pthread_t thread1;
	pthread_create(&thread1, NULL, timer_thread, &proxysysMessageHandler);

	while(!timerReady){
		usleep(1000);
	}
	timeLeft = main_time->timeLeft;
	while(timeLeft>0 && quit == FALSE){
		if(gameResetMessageMain == TRUE ) {
			cout << "System reset!" << endl;
			flashTheSensorBoardLeds( &proxysysMessageHandler );
			functionStopRobot(&max_speed_limit, &navsysMessageHandler);
			receivedTeamColor = FALSE;
			proxysysMessageHandler.sendCommand( MINOR_CMD_SYSTEM_GO, PRIORITY_HIGHEST );
			waitForTeamColorMessage( &proxysysMessageHandler, &navsysMessageHandler );
			gameResetMessageMain = FALSE;
			waitForGameStartMessage(&navsysMessageHandler);
			timeLeft = 90;
			SendInitPosition( &navsysMessageHandler );
			usleep(100000);
			plansysObs.ResetStrategies( teamColor );
		}
		usleep(5000);
		// Ask for position
		requestPosition( &navsysMessageHandler );
		// Update the priority of the strategies
		plansysObs.UpdateStrategyPriority();
		// Get the priority from the strategies
		nextStrategy = plansysObs.GetStrategyPriority();
		// Perform the strategy with the highest priority
		plansysObs.PerformStrategy( &navsysMessageHandler, &proxysysMessageHandler, &cvsysMessageHandler, nextStrategy );
		// Check the time
		timeLeft = main_time->timeLeft;
		//cout << "Time left: " << timeLeft << endl;
		if(lowBattery == TRUE) {
			shutdownSystem( &proxysysMessageHandler );
			quit = TRUE;
		}
	}
	sleep(1);
	return 0;
}

void catch_sighup(int signum){
 	signal(SIGHUP, catch_sighup);
}
