#include "sendReceive.h"

extern int teamColor;
extern int receivedTeamColor;

extern int collisionDetect;

extern int askForCvObservationFlag;
extern int receivedGameStartMessage;
extern int gameResetMessageTimer;
extern int gameResetMessageMain;
extern int gameFirstStart;

extern PlansysObservation plansysObs;

/**
 * **************************************************************************** *
 * Request and send functions                                                   *
 * **************************************************************************** *
 */
int SendInitPosition( MessageHandler *navsysMessageHandler ) {
	posix_robot_state_3dof robot_init_state_3dof;
	if( teamColor == TEAM_COLOR_RED ) {
		robot_init_state_3dof.x_l = TABLE_SIZE_X - 250;
		robot_init_state_3dof.y_l = 250;
		robot_init_state_3dof.u_b = 0;
		robot_init_state_3dof.psi_l = ((3*M_PI)/4);
		robot_init_state_3dof.r_b = 0;	
	}
	else if ( teamColor == TEAM_COLOR_BLUE ) {
		robot_init_state_3dof.x_l = 250;
		robot_init_state_3dof.y_l = 250;
		robot_init_state_3dof.u_b = 0;
		robot_init_state_3dof.psi_l = (M_PI/4);
		robot_init_state_3dof.r_b = 0;
	}
	else {
		cout << "ALERT: No teamColor received " << endl;
	}
	navsysMessageHandler->sendMessage( (char *)&robot_init_state_3dof, (int)sizeof(posix_robot_state_3dof), MINOR_NAVSYS_SET_POSITION, PRIORITY_HIGHEST );
}

int flashTheSensorBoardLeds( MessageHandler *proxysysMessageHandler ){
	collision_detection_message collision_message;
	proxysysMessageHandler->sendMessage( (char *)&collision_message, (int)sizeof(collision_detection_message), MINOR_FLASH_SENSOR_BOARD_LED, PRIORITY_NORMAL );
	return 0;
}

int askForCvObservation( MessageHandler *cvsysMessageHandler) {
	cv_observation my_cv_observation;
	my_cv_observation.type = BOTTLE_BOX;
	my_cv_observation.x = 0;
	my_cv_observation.y = 0;
	my_cv_observation.ret = 0;
	cvsysMessageHandler->sendMessage( (char *)&my_cv_observation, (int)sizeof(cv_observation), MINOR_CV_OBSERVATION, PRIORITY_VERY_HIGH );
}
  
/**
* **************************************************************************** *
* Receive functions                                                            *
* **************************************************************************** *
*/

void receiveCollisionDetectionStruct(pmq_message *pm){
	pmq_message my_posix_message;
	memcpy((void *)(&my_posix_message),( void * )pm, sizeof(pmq_message));
	//cout << " \n \n Collision Detection \n \n" << endl;
	collisionDetect++;
	plansysObs.NewObservation(my_posix_message, MINOR_COLLISION_DETECTED);
}

void receiveCvObservaton(pmq_message *pm){
	pmq_message my_posix_message;
	memcpy((void *)(&my_posix_message),( void * )pm, sizeof(pmq_message));
	cout << " \n \n ReceiveCvObservation \n \n" << endl;
	plansysObs.NewObservation(my_posix_message, MINOR_CV_OBSERVATION);
}

void receiveCvColorObservaton(pmq_message *pm){
	pmq_message my_posix_message;
	memcpy((void *)(&my_posix_message),( void * )pm, sizeof(pmq_message));
	cout << " \n \n ReceiveCvColorObservation \n \n" << endl;
	plansysObs.NewObservation(my_posix_message, MINOR_CV_COLOR_OBSERVATION);
}

void receivePositionUpdate(pmq_message *pm){
	pmq_message my_posix_message;
	memcpy((void *)(&my_posix_message),( void * )pm, sizeof(pmq_message));
	plansysObs.NewObservation(my_posix_message, MINOR_NAVSYS_POSITION);
}

void receiveWaypointReached(pmq_message *pm) {
	pmq_message my_posix_message;
	ai_waypoint first_run_waypoint;
	memcpy((void *)(&my_posix_message),( void * )pm, sizeof(pmq_message));
	memcpy((void *)(&first_run_waypoint),(void *)pm->data, sizeof(ai_waypoint));
	plansysObs.NewObservation(my_posix_message, pm->minor);
	if(first_run_waypoint.id == 1975){
		askForCvObservationFlag = TRUE;
	}
}

void receiveProxyMessage(pmq_message *pm) {
	pmq_message my_posix_message;
	memcpy((void *)(&my_posix_message),( void * )pm, sizeof(pmq_message));
}

void receiveCmdBatterySort(pmq_message *pm) {
	if(pm->minor == MINOR_CMD_BATTERY_RELEASED) {
		plansysObs.NewObservation(*pm, pm->minor);
	}
}

void receiveCmdProxyMessage(pmq_message *pm) {
	if(pm->minor == MINOR_CMD_SYSTEM_GO) {
		receivedGameStartMessage = TRUE;
		cout << " " << endl;
		cout << "Recieve CMD START MESSAGE " << endl;
		cout << " " << endl;
	}
	if(pm->minor == MINOR_CMD_SYSTEM_INITIALIZE) {
		cout << "pm->minor == MINOR_CMD_SYSTEM_INITIALIZE" << endl;
		receivedTeamColor = FALSE;
		receivedGameStartMessage = FALSE;
		cout << "recievedGameStartMessage = FALSE" << endl;
		gameResetMessageTimer = TRUE;
		cout << "gameResetMessageTimer = TRUE" << endl;
		gameResetMessageMain = TRUE;
		cout << "gameResetMessageMain = TRUE" << endl;
	}
}

void receiveCmdTeamColor(pmq_message *pm) {
	if(pm->minor == MINOR_CMD_SET_TEAM_COLOR_BLUE) {
		teamColor = TEAM_COLOR_BLUE;
		cout << " " << endl;
		cout << "receiveCmdTeamColor TEAM_COLOR_BLUE" << endl;
		cout << " " << endl;
		receivedTeamColor = TRUE;
	}
	else if(pm->minor == MINOR_CMD_SET_TEAM_COLOR_RED) {
		teamColor = TEAM_COLOR_RED;
		cout << " " << endl;
		cout << "receiveCmdTeamColor TEAM_COLOR_RED" << endl;
		cout << " " << endl;
		receivedTeamColor = TRUE;
	}
	else{
		cout << " " << endl;
		cout << " " << endl;
		cout << "receiveCmdTeamColor ERROR" << endl;
		cout << " " << endl;
		cout << " " << endl;
	}
}

int waitForTeamColorMessage( MessageHandler *proxysysMessageHandler, MessageHandler *navsysMessageHandler ) {
	
	while( receivedTeamColor == FALSE){ 
		cout << "Wait for teamColor " << endl;
		usleep(1000000);
		proxysysMessageHandler->handleAllMessagesNow();
	}
	// Set the team color LED
	if( teamColor == TEAM_COLOR_RED ){
		proxysysMessageHandler->sendCommand(MINOR_SET_LED_TEAM_COLOR_RED ,PRIORITY_HIGH);
	}
	else if( teamColor == TEAM_COLOR_BLUE ){
		proxysysMessageHandler->sendCommand(MINOR_SET_LED_TEAM_COLOR_BLUE ,PRIORITY_HIGH);
	}
	else{
		return -1;
	}
	// Send set init position to navsys.
	// This position is determined by the teamColor.
	SendInitPosition( navsysMessageHandler );
	
	return 0;
}

void waitForGameStartMessage(MessageHandler *navsysMessageHandler){
	while( receivedGameStartMessage == FALSE){ 
		cout << "Wait for game start message " << endl;
		usleep(100000);
		navsysMessageHandler->handleAllMessagesNow();
	}
}

