//****************************************************************************
// INCLUDES

#include <iostream>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <ctype.h>
#include <unistd.h>   // exit
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>    // O_RDWR

#include <libpcan.h>
#include <pcan.h>

#include <common.h>
//#include <parser.h>

#include "can_bus.h"
#include "main.h"

#include "posix_message_config.h"
#include "posix_message_queue.h"
#include "gatewayconfig.h"
#include "gatewayMotorControl.h"
#include "gatewayCollisionDetection.h"
#include "gatewayCommands.h"



//****************************************************************************

// GLOBALS
HANDLE h = NULL;
char *current_release;

//****************************************************************************

int can_init(){

	int nType = HW_TYPE;
	__u32 dwPort = 0;
	__u16 wIrq = 0;
	__u16 wBTR0BTR1 = 0;
	int   nExtended = CAN_INIT_TYPE_ST;
	char  *szDevNode = DEFAULT_NODE;
	bool bDevNodeGiven = false;
	bool bTypeGiven = false;

	// open the CAN port
	if ((bDevNodeGiven) || (!bDevNodeGiven && !bTypeGiven)){
		printf("LINUX_CAN_Open(szDevNode, O_RDWR);   \n");
    		h = LINUX_CAN_Open(szDevNode, O_RDWR);  
	}
	else{
		printf("CAN_Open(nType, dwPort, wIrq); \n");
		h = CAN_Open(nType, dwPort, wIrq);
	}

	if (h){    
		char txt[VERSIONSTRING_LEN];
		CAN_Status(h);

		// get version info
		errno = CAN_VersionInfo(h, txt);
		if (!errno)
			printf("can_init: driver version = %s\n", txt);
		else{
			perror("can_init: CAN_VersionInfo()");
		}

		// init to a user defined bit rate
		if (wBTR0BTR1){
			errno = CAN_Init(h, wBTR0BTR1, nExtended);
			if (errno){
				perror("can_init: CAN_Init()");
			}
		}
	
		current_release = CURRENT_RELEASE;
		disclaimer("Robot Main Program");

		printf("can_init: type=%s", getNameOfInterface(HW_TYPE));
		printf(", Serial Number=default, Device Number=%d\n",NULL );//dwPort); 
	}

	return 0;

}


int can_send_message(TPCANMsg *CAN_SEND_STRUCT){

	// send the message
	if ((errno = CAN_Write(h, CAN_SEND_STRUCT))){
	}
	else{
	}
	return 0;

}

int can_recieve_message( TPCANMsg *CAN_SEND_STRUCT, MessageHandler *navsysMessageHandler, MessageHandler *plansysMessageHandler ){

	
	TPCANMsg m;
	__u32 status;

	if ((errno = CAN_Read(h, &m))){
		perror("can_recieve_message: CAN_Read()");
	}
	else{

	switch(m.ID){

/**
 * CAN-refuse messages
 */
		case CANID_COLLISION_SLAVE_MASTER_COMMANDS:
			break;
		

/**
 * CAN-controll messages
 */
		case CAN_INIT_GAME_TIME:
			proxyInitTimeMessage(plansysMessageHandler);
			break;

		case CAN_RESET_GAME:
			proxySendCommand(plansysMessageHandler, MINOR_CMD_SYSTEM_INITIALIZE, PRIORITY_HIGHEST);
			break;

		case CAN_CMD_SET_TEAM_COLOR_RED:
			proxySendCommand(plansysMessageHandler, MINOR_CMD_SET_TEAM_COLOR_RED, PRIORITY_HIGHEST);
			cout << "CAN_CMD_SET_TEAM_COLOR_RED" << endl;
			break;

		case CAN_CMD_SET_TEAM_COLOR_BLUE:
			proxySendCommand(plansysMessageHandler, MINOR_CMD_SET_TEAM_COLOR_BLUE, PRIORITY_HIGHEST);
			cout << "CAN_CMD_SET_TEAM_COLOR_BLUE" << endl;
			break;
/**
 * Collision Detection
 */
		case CANID_COLLISION_DETECTED:
			proxySendCollisionDetected(m.DATA, plansysMessageHandler, navsysMessageHandler);
			break;
		case CANID_RANGE_MEASUREMENTS:
			proxySendRangeMessurement(m.DATA, plansysMessageHandler);
			break;
/**
 * Motor Control
 */


		case CAN_ADDR_MOTOR_RIGHT_RETURN_QUAD:
			proxyRightReturnQuad(m.DATA, navsysMessageHandler);
			break;

		case CAN_ADDR_MOTOR_LEFT_RETURN_QUAD:
			proxyLeftReturnQuad(m.DATA, navsysMessageHandler);
			break;

		case CAN_ADDR_RETURN_DUAL_QUAD:
			proxyReturnDualQuad(m.DATA, navsysMessageHandler);
 			break;
		
		default:
			printf("Could not recognize CAN-message in can_bus.cpp");
			break;

	}//switch
	

		// don't print any telegram contents for remote frames
		// check if a CAN status is pending	     
		if (m.MSGTYPE & MSGTYPE_STATUS){
			status = CAN_Status(h);
			if ((int)status < 0){
				errno = nGetLastError();
				perror("can_receive_message: CAN_Status()");
			}
			else
				printf("receive: pending CAN status 0x%04x read.\n", (__u16)status);
		} 
	}
	return 0;
}
