// INCLUDES
#include <pthread.h>
#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <ctype.h>
#include <unistd.h>   // exit
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>    // O_RDWR

#include <libpcan.h>
#include <pcan.h>

#include <math.h>

#include "main.h"
#include "can_bus.h"
#include "gatewayMotorControl.h"
#include "gatewayCollisionDetection.h"
#include "gatewayCommands.h"


MessageHandler navsysMessageHandler(PROXY_ID, PMQ_PROXY, PMQ_NAVSYS, true);
MessageHandler plansysMessageHandler(PROXY_ID, PMQ_PROXY, PMQ_PLANSYS, true);


int dirleft = 0;
int dirright = 0;


//Method for CAN server thread
extern "C" void* can_server(void *arg){
	TPCANMsg canReceiveStruct;
	while(1){
		can_recieve_message( &canReceiveStruct, &navsysMessageHandler, &plansysMessageHandler );

	}
	return arg;
}


//Startup method
void proxyReceiveGoCommand(pmq_message *pm) {
	speed_limit max_speed_limit;
    	max_speed_limit.u_b = 10.0;
	max_speed_limit.r_b = 10.0;
    	navsysMessageHandler.sendMessage( (char *)&max_speed_limit, (int)sizeof(speed_limit), MINOR_NAVSYS_SET_SPEED_LIMIT, PRIORITY_HIGHEST );
	
        setStartTimerCollision();
}


//Shutdown method
void proxyReceiveSTOPCommand(pmq_message *pm) {
	speed_limit max_speed_limit;
    	max_speed_limit.u_b = 0.0;
    	max_speed_limit.r_b = 0.0;
    	navsysMessageHandler.sendMessage( (char *)&max_speed_limit, (int)sizeof(speed_limit), MINOR_NAVSYS_SET_SPEED_LIMIT, PRIORITY_HIGHEST );

	cout << " STOPPING ALL ACTUATORS " << endl;
}


int main() {

	int n;
	cout << "Starting PROXY " << endl;
	errno = 0;
	can_init();
	quad_init();

	signal(SIGINT, catch_int);

	signal(SIGHUP, catch_sighup);
	
	TPCANMsg canSendStruct;

	//Create message queues
	navsysMessageHandler.addSubMessageHandler(proxyReceiveMotorControl, MAJOR_STRUCT, MINOR_MOTOR_CONTROL_MESSAGE);
	
	plansysMessageHandler.addSubMessageHandler(proxyReceiveCommands, MAJOR_STRUCT, MINOR_PROXY_COMMAND);
	plansysMessageHandler.addSubMessageHandler(proxyReceiveCommand, MAJOR_COMMAND, MINOR_SET_LED_TEAM_COLOR_RED);
	plansysMessageHandler.addSubMessageHandler(proxyReceiveCommand, MAJOR_COMMAND, MINOR_SET_LED_TEAM_COLOR_BLUE);
	plansysMessageHandler.addSubMessageHandler(proxyReceiveGoCommand, MAJOR_COMMAND, MINOR_CMD_SYSTEM_GO);
	plansysMessageHandler.addSubMessageHandler(proxyReceiveSTOPCommand, MAJOR_COMMAND, MINOR_CMD_SYSTEM_STOP);
	plansysMessageHandler.addSubMessageHandler(proxyRequestDistanceMeasurement, MAJOR_COMMAND, MINOR_REQUEST_MEASUREMENTS);
	plansysMessageHandler.addSubMessageHandler(proxyFlashSensorBordLeds, MAJOR_STRUCT, MINOR_FLASH_SENSOR_BOARD_LED);

	pthread_t thread2;
	
	// Start CAN-server
	pthread_create(&thread2, NULL, can_server, NULL);	

	
	//main loop
	while(1){
				
		//Read from quadrature
		if( MOTORDRIVER_QUAD ) {

			usleep(5000);
			usleep(5000);
			
			canSendStruct.ID = CAN_ADDR_MOTOR_LEFT_READ_QUAD; //0x200;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 0x00;
			n = can_send_message( &canSendStruct );

			usleep(5000);
			usleep(5000);

			canSendStruct.ID = CAN_ADDR_MOTOR_RIGHT_READ_QUAD; //0x201;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 0x00;
			n = can_send_message( &canSendStruct );
		}
		else{
			usleep(50000);
			canSendStruct.ID = CAN_ADDR_READ_DUAL_QUAD; //0x202;
			canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
			canSendStruct.LEN = 0x00;

			n = can_send_message( &canSendStruct );

		}

	}

} // main


void catch_sighup(int signum){
	signal(SIGHUP, catch_sighup);

}

//signal handler 
void catch_int(int sig_num){
	
	signal(SIGINT, catch_int);
	TPCANMsg canSendStruct;

	cout << "Ctrl+c stopping robot " << endl;

	canSendStruct.ID = CAN_ADDR_MOTOR_MAIN;
	canSendStruct.MSGTYPE = MSGTYPE_STANDARD;
	canSendStruct.LEN = 0x04;
	canSendStruct.DATA[0] = 0x00;
	canSendStruct.DATA[1] = 0x00;
	canSendStruct.DATA[2] = 0x00;
	canSendStruct.DATA[3] = 0x00;

	can_send_message( &canSendStruct );

	exit(1);
 
}

