//****************************************************************************
// INCLUDES

#include <iostream>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <ctype.h>
#include <unistd.h>   // exit
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>    // O_RDWR

#include <libpcan.h>
#include <pcan.h>

#include <common.h>
//#include <parser.h>

#include "can_bus.h"
#include "main.h"

#include "posix_message_config.h"
#include "posix_message_queue.h"
#include "gatewayConfig.h"
#include "gatewayMotorControl.h"
#include "gatewayCollisionDetection.h"
#include "gatewayCommands.h"
#include "gatewayBallSorting.h"


//****************************************************************************

// GLOBALS
HANDLE h = NULL;
char *current_release;
pthread_mutex_t CAN_mutex;

//****************************************************************************

int can_init(){

	int nType = HW_TYPE;
	__u32 dwPort = 0;
	__u16 wIrq = 0;
	__u16 wBTR0BTR1 = 0;
	int   nExtended = CAN_INIT_TYPE_ST;
	char  *szDevNode = DEFAULT_NODE;
	bool bDevNodeGiven = false;
	bool bTypeGiven = false;

	// open the CAN port
	if ((bDevNodeGiven) || (!bDevNodeGiven && !bTypeGiven)){
		printf("LINUX_CAN_Open(szDevNode, O_RDWR);   \n");
    		h = LINUX_CAN_Open(szDevNode, O_RDWR);
	}
	else{
		printf("CAN_Open(nType, dwPort, wIrq); \n");
		h = CAN_Open(nType, dwPort, wIrq);
	}

	if (h){
		char txt[VERSIONSTRING_LEN];
		CAN_Status(h);

		// get version info
		errno = CAN_VersionInfo(h, txt);
		if (!errno)
			printf("can_init: driver version = %s\n", txt);
		else{
			perror("can_init: CAN_VersionInfo()");
		}

		// init to a user defined bit rate
		if (wBTR0BTR1){
			errno = CAN_Init(h, wBTR0BTR1, nExtended);
			if (errno){
				perror("can_init: CAN_Init()");
			}
		}

		current_release = CURRENT_RELEASE;
		disclaimer("Robot Main Program");

		printf("can_init: type=%s", getNameOfInterface(HW_TYPE));
		printf(", Serial Number=default, Device Number=%d\n",NULL );//dwPort);
	}

	return 0;

}


int can_send_message(TPCANMsg *CAN_SEND_STRUCT){

	//pthread_mutex_lock(&CAN_mutex);
	// send the message
	if ((errno = CAN_Write(h, CAN_SEND_STRUCT))){
		//printf("if errno: %d\n\r",errno);
	}
	else{
		//printf("else errno: %d\n\r",errno);
	}
	return 0;
	//pthread_mutex_unlock(&CAN_mutex);

}

int can_recieve_message( TPCANMsg *CAN_SEND_STRUCT, MessageHandler *navsysMessageHandler, MessageHandler *plansysMessageHandler ){


	TPCANMsg m;
	__u32 status;

	if ((errno = CAN_Read(h, &m))){
		perror("can_recieve_message: CAN_Read()");
	}
	else{

	//printf("CAN-ID: %X\n", m.ID);

	switch(m.ID){

/**
 * CAN-refuse messages
 */
		case CANID_COLLISION_SLAVE_MASTER_COMMANDS:
			break;
/**
 * CAN-controll messages
 */
		case CAN_INIT_GAME_TIME:
			gatewayInitTimeMessage(plansysMessageHandler);
			break;

		case CAN_RESET_GAME:
			printf("RESET MESSAGE RECEIVED!\n");
			gatewaySendCommand(plansysMessageHandler, MINOR_CMD_SYSTEM_INITIALIZE, PRIORITY_HIGHEST);
			gatewaySendCommand(navsysMessageHandler, MINOR_CMD_SYSTEM_INITIALIZE, PRIORITY_HIGHEST);
			break;

		case CAN_CMD_SET_TEAM_COLOR_RED:
			gatewaySendCommand(plansysMessageHandler, MINOR_CMD_SET_TEAM_COLOR_RED, PRIORITY_HIGHEST);
			cout << "CAN_CMD_SET_TEAM_COLOR_RED" << endl;
			break;

		case CAN_CMD_SET_TEAM_COLOR_BLUE:
			gatewaySendCommand(plansysMessageHandler, MINOR_CMD_SET_TEAM_COLOR_BLUE, PRIORITY_HIGHEST);
			cout << "CAN_CMD_SET_TEAM_COLOR_BLUE" << endl;
			break;
		/**
		 * NEW CODE STARTS HERE
		 */
		case CAN_LOW_BATTERY:
			//gatewayLowBattery(plansysMessageHandler);
			break;
		/**
	 	 * NEW CODE ENDS HERE
	 	 */
/**
 * Collision Detection
 */
		case CANID_COLLISION_DETECTED:
			//printf("Collision message \n\r");
			gatewaySendCollisionDetected(m.DATA, plansysMessageHandler, navsysMessageHandler);
			break;
		case CANID_RANGE_MEASUREMENTS:
			gatewaySendRangeMessurement(m.DATA, plansysMessageHandler);
			break;
/**
 * Motor Control
 */

		case CAN_ADDR_MOTOR_RIGHT_RETURN_QUAD:
			gatewayRightReturnQuad(m.DATA, navsysMessageHandler);
			break;

		case CAN_ADDR_MOTOR_LEFT_RETURN_QUAD:
			gatewayLeftReturnQuad(m.DATA, navsysMessageHandler);
			break;

		case CAN_ADDR_RETURN_DUAL_QUAD:
			gatewayReturnDualQuad(m.DATA, navsysMessageHandler);
 			break;

		case CAN_ADDR_POSITIONING_BEACON_ANGLES:
			beacon_angles beaconAngles;
			beaconAngles.b1 = (double)((m.DATA[0])|(m.DATA[1]<<8));
			beaconAngles.b2 = (double)((m.DATA[2])|(m.DATA[3]<<8));
			beaconAngles.b3 = (double)((m.DATA[4])|(m.DATA[5]<<8));

			printf("BEACON ANGLES RECEIVED: %d, %d, %d\n", (m.DATA[0])|(m.DATA[1]<<8), (m.DATA[2])+(m.DATA[3]<<8),(m.DATA[4])+(m.DATA[5]<<8));
			printf("CONVERTED BEACON ANGLES RECEIVED: %f, %f, %f\n", beaconAngles.b1, (int)beaconAngles.b2,beaconAngles.b3);

			navsysMessageHandler->sendMessage((char *)&beaconAngles, (int)sizeof(beacon_angles), MINOR_NAVSYS_BEACON_MEASUREMENT, PRIORITY_VERY_HIGH );
			break;
		case CAN_ADDR_COLOR_DETECTED:
			gatewayColorDetected(m.DATA, plansysMessageHandler);
			break;
		case CAN_ADDR_BALL_STORED:
			gatewayBallStored(m.DATA, plansysMessageHandler);
			break;
		case CAN_ADDR_WHEEL_READY:
			gatewayWheelReady(plansysMessageHandler);
			break;
		case CAN_ADDR_BALLS_RELEASED:
			gatewayBallsReleased(m.DATA, plansysMessageHandler);
			break;
		case CAN_INTERRUPT_SWITCHES:
			gatewaySendSwitchInterrupt(m.DATA, plansysMessageHandler);
			break;
		default:
			printf("Could not recognize CAN-message (id: %X)in can_bus.cpp\n\r",m.ID);
			break;

	}//switch


		// don't print any telegram contents for remote frames
		// check if a CAN status is pending
		if (m.MSGTYPE & MSGTYPE_STATUS){
			status = CAN_Status(h);
			if ((int)status < 0){
				errno = nGetLastError();
				perror("can_receive_message: CAN_Status()");
			}
			else
				printf("receive: pending CAN status 0x%04x read.\n", (__u16)status);
		}
	}
	return 0;
}
