/*
** ** ** ** ** **  Begin header  ** ** ** ** ** ** **
**
** Project : Eurobot 2007
** Purpose : Easy to use Posix Message Queue implementation.
** Filename : posix_message_config.h
** Version : 1.0
** Date : 18.01.2007
**
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
**
** Copyright © 2007, Kristian M. Knausgård
** All rights reserved.
**
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **

VERSION LOG:
-----------
Version: 1.0
Date: 18.01.2007
Author: Kristian M. Knausgård
Coauthors: <none>
Description : Original version.
Target platform: Linux / Posix

FILE FUNCTIONALITY DESCRIPTION:
-----------

Major an minor numbers are defined in this file.

Needs to be included for all users of PMQ.

** ** ** ** ** ** ** End header ** ** ** ** ** ** ** **
*/


#define PMQ_NAVSYS "/navsys"
#define PMQ_CVSYS "/cvsys"
#define PMQ_CVSYS_COLOR "/cvcolor"
#define PMQ_PLANSYS "/plansys"
#define PMQ_PLANSYS_INT "/plansysint"
#define PMQ_LOGSYS "/logsys"
#define PMQ_PROXY "/proxy"


#define MAX_MAJOR_NUMBER 6
#define MAX_MINOR_NUMBER 120

// Majors (message classes)
#define MAJOR_ALL 0		// Used for convenicence for subscribing to all msgs.
#define MAJOR_COMMAND 1
#define MAJOR_TEXT_MESSAGE 2	// (No subtype neccessary)
#define MAJOR_STRUCT 3
#define MAJOR_CPPCLASS 4
#define MAJOR_SPEAK 5

// Minors (which subtype)
#define MINOR_ALL 0		// Used for convenicence for subscribing to all msgs.

// Command IDs
#define MINOR_CMD_SYSTEM_STOP 1
#define MINOR_CMD_SYSTEM_SHUTDOWN 2
#define MINOR_CMD_SYSTEM_INITIALIZE 3
#define MINOR_CMD_SYSTEM_GO 4
#define MINOR_CMD_LOW_BATTERY 		5
#define MINOR_CMD_SHUTDOWN_SYSTEM	6

#define MINOR_CMD_REQUEST_POSITION 40
#define MINOR_CMD_CLEAR_WAYPOINTS 41
#define MINOR_CMD_STOP_IMMEDIATELY 42
#define MINOR_CMD_SET_TEAM_COLOR_RED 43
#define MINOR_CMD_SET_TEAM_COLOR_BLUE 44
#define MINOR_CMD_RELEASE_BATTERY 45
#define MINOR_CMD_BATTERY_RELEASED 46

#define MINOR_CMD_REVERSE	47
#define MINOR_CMD_SWITCHES	48

#define MINOR_SET_LED_TEAM_COLOR_RED 53
#define MINOR_SET_LED_TEAM_COLOR_BLUE 54

// Collision Detection
#define MINOR_REQUEST_MEASUREMENTS 55

// Struct IDs
#define MINOR_ODOMETRY_OBSERVATION 11
#define MINOR_NAVSYS_BEACON_MEASUREMENT 15
#define MINOR_MOTOR_CONTROL_MESSAGE 12
#define MINOR_CV_OBSERVATION 13
#define MINOR_CV_BOTTLE_OBSERVATION 14
//#define MINOR_CV_BOX_OBSERVATION 15
#define MINOR_CV_BATTERY_OBSERVATION 16
#define MINOR_NAVSYS_POSITION 18
#define MINOR_NAVSYS_SET_POSITION 19
#define MINOR_NAVSYS_SET_WAYPOINT 20
#define MINOR_NAVSYS_SET_WAYPOINT_ROBOT_RELATIVE 21
#define MINOR_NAVSYS_REACHED_WAYPOINT 22
#define MINOR_NAVSYS_SET_SPEED_LIMIT 23
#define MINOR_PLANSYS_FREEWHEEL 24
#define MINOR_PROXY_MESSAGE 25
#define MINOR_PROXY_COMMAND 26
#define MINOR_NAVSYS_WAYPOINT_UNFEASIBLE 27
#define MINOR_NAVSYS_SET_SPEED 37
#define MINOR_NAVSYS_UPDATE_BEACON_ANGLES 90
#define MINOR_BEACON_MEASUREMENT_REQUEST 91

#define MINOR_ROBOT_TO_LOCAL	28
#define MINOR_LOCAL_WAYPOINT	17

// Collision Detection
#define MINOR_COLLISION_DETECTED 29
#define MINOR_RANGE_MEASUREMENTS 30
#define MINOR_FLASH_SENSOR_BOARD_LED 31
#define MINOR_ENABLE_SENSOR_LEDS 32
#define MINOR_SET_COLLISION_THRESHOLDS 33
#define MINOR_SET_INTERRUPT_REFRESH_INTERVAL 34

#define MINOR_CV_COLOR_OBSERVATION 35
#define MINOR_NAVSYS_RELATIVE_INTERRUPT_WAYPOINT 36
#define MINOR_NAVSYS_ADJUST_POSITION 38

#define MINOR_SET_ARMS		80
#define MINOR_COLOR_DETECTED	81
#define MINOR_COLOR_ACK		82
#define MINOR_BALL_STORED	84
#define MINOR_RELEASE_BALLS	85
#define MINOR_BALLS_RELEASED	86
#define MINOR_WHEEL_READY	87


// Speech synthesis
#define MINOR_DEFAULT_VOICE 0


// C++ class IDs
#define MINOR_CPPCLASS_MAPDATA 1


// Posix Message Priorities
#define PRIORITY_HIGHEST 	31	// Max pri. compliant with POSIX.1-2001
#define PRIORITY_VERY_HIGH 	25
#define PRIORITY_HIGH 		20
#define PRIORITY_NORMAL 	15
#define PRIORITY_LOW 		10
#define PRIORITY_VERY_LOW 	5
#define PRIORITY_LOWEST 	0
