#ifndef LOS_GUIDANCE_H
#define LOS_GUIDANCE_H

#include <iostream>
#include <string>
#include <vector>
#include <stdexcept>
#include <errno.h>
#include "config.h"

#ifdef __cplusplus
extern "C"
{
#endif
#include "posix_message_queue.h"
#include "posix_message_config.h"
#ifdef  __cplusplus
}
#endif

#include "waypoint.h"
#include "Controller.h"

using namespace std;


//static void waypointHandler(pmq_message pm);

class LOSGuidance {
    public:
		// Which "channel" to listen to. Must always start with / as it's a file.
        LOSGuidance(MessageHandler *messageHandler, vector<Controller *> controllerList) {
			currentWaypoint = 0;
			this->messageHandler = messageHandler;
			headingAngleController = controllerList.at(HEADING_ANGLE_CONTROLLER);

			lastAngleFix = 0;
			i1 = 0;
			i2 = 0;
		}

		void addWaypoint(const los_waypoint *wp) {
			// Sjekk at det er nok minne tilgjengelig... (det er det nok men)
			waypoints.push_back(*wp);
			//printf("Waypoint added: %f , %f , %f, %f, %d\n", wp->x_l, wp->y_l, wp->psi_l*(180/PI), wp->circle_of_acceptance, wp->id);
		}


		void clearAllWaypoints() {
			waypoints.clear();
			currentWaypoint = 0;
			lastAngleFix = 0;
			i2 = 0;
			i1 = 0;
			//robotModel->setMaxForce(100, 100);
			printf("Clear all waypoints!\n\r");
		}

		void updateSimpel(const robot_state_3dof *currentState, robot_state_3dof *desiredState) {
				double deltaX = (waypoints.at(currentWaypoint).x_l - currentState->x_l);
				double deltaY = (waypoints.at(currentWaypoint).x_l - currentState->y_l);
				double error_u = sqrt(deltaX*deltaX + deltaY*deltaY);
				double error_psi = angleWrap(atan2( deltaY , deltaX ) - currentState->psi_l);
				//double error_psi2 = angleWrap(desiredState->psi_l-currentState->psi_l);
				lastAngleFix = 0;
				if(lastAngleFix){
					error_psi = angleWrap(desiredState->psi_l-currentState->psi_l);
				}

				printf("%f\n\r", error_psi);
				if(error_psi > 5*(PI/360) || error_psi < -2*(PI/360)){
					desiredState->r_b = 100*error_psi;
					desiredState->u_b = 0;
				}
				else if(error_u > 0.1 || error_u < -0.1){
					desiredState->r_b = 0;
					desiredState->u_b = 100*error_u;
					lastAngleFix = 1;
				}
				else{
					desiredState->r_b = 0;
					desiredState->u_b = 0;
				}


		}


		void update(const robot_state_3dof *currentState, robot_state_3dof *desiredState) {

			if(currentWaypoint < (int)waypoints.size()) {
				double x_l_d = waypoints.at(currentWaypoint).x_l;
				double y_l_d = waypoints.at(currentWaypoint).y_l;
				double psi_l_d = waypoints.at(currentWaypoint).psi_l;

				double psi_d = 0;	// Desired heading must be computed from WP.

				// Find distance from WP:
				double deltaX = (x_l_d - currentState->x_l);
				double deltaY = (y_l_d - currentState->y_l);
				double error = sqrt(deltaX*deltaX + deltaY*deltaY);

				psi_d = atan2( deltaY , deltaX );

				double error_psi = angleWrap(psi_d - currentState->psi_l);

				// The error between current robot angle and desired waypoint angle (only used in last wp)
				double error_wp_psi = angleWrap(psi_l_d - currentState->psi_l);

				#define ANGLE_TOLERANCE 2.0

				if(waypoints.at(currentWaypoint).id < 0){
					error_psi = angleWrap(psi_d - currentState->psi_l + PI);
					error_wp_psi = angleWrap(psi_l_d - currentState->psi_l);
				}

				// If within circle of acceptance or lastAnglefix
				if(error<=waypoints.at(currentWaypoint).circle_of_acceptance || lastAngleFix) {
					int lastWaypointAngleOK = 0;
					lastWaypointAngleOK = (fabs(error_wp_psi) <= ANGLE_TOLERANCE*(PI/180.0));

					//printf("Vinkelfeil: %f\n\r",fabs(error_wp_psi));


					if(lastWaypointAngleOK) {

						// Send waypoint reached
						ai_waypoint awp;
						awp.id = waypoints.at(currentWaypoint).id;
						awp.tag = waypoints.at(currentWaypoint).tag;
						awp.x_l = (short)(waypoints.at(currentWaypoint).x_l*1000.0); // From m to mm
						awp.y_l = (short)(waypoints.at(currentWaypoint).y_l*1000.0); // From m to mm
						awp.psi_l = (float)waypoints.at(currentWaypoint).psi_l;	// Already in rads, but decreasing precision
						awp.circle_of_acceptance = (short)(waypoints.at(currentWaypoint).circle_of_acceptance*1000.0); // From m to mm
						messageHandler->sendMessage((char *)&awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
						//robotModel->setMaxForce(100, 100);

						currentWaypoint++;
						lastAngleFix = 0;
						i2 = 0;
						i1 = 0;

						//printf("error: %f, deltaX %f, deltaY %f \n\r", error, deltaX, deltaY);
						printf("Waypoint with tag %d reached! %d, %d, %d, %f\n\r",awp.tag, awp.x_l, awp.y_l, awp.circle_of_acceptance, error);
						printf("Posisjon: %f, %f, %f \n\r", currentState->x_l, currentState->y_l, currentState->psi_l);
						return;	// Don't do anything more on this update

					}
					else {
						// Fix our angle since this is the last wp
						lastAngleFix = 1;
						error_psi = error_wp_psi;
						//printf("fikser vinkel\n\r");
						error = 0;
					}
				}






				double Kp_surge = 0.4;
				double Ti_surge = 10; //5000 funker bra
				//double Td_surge = 0;

				// PID parameters for angular (yaw) rate:
				double Kp_psi = 10; //10 burde funke
				double Ti_psi = 500; //500 funker bra
				double Td_psi = 0;

				if(lastAngleFix){
					Kp_psi = 20;
					Ti_psi = 50;
				}

				static double p1_last = 0;
				double maxSurgeRate = 1.0;
				if(i1>maxSurgeRate) i1 = maxSurgeRate;
				else if(i1<-maxSurgeRate) i1 = -maxSurgeRate;


				// Persistent variables (move initialization to constructor...):
				static double p2_last = 0;


				// Caccia 2000:
				// With Lyapunov function candidate: V=(1/2)error²+(1/2)(psi_d²+h*psi_d_at_waypoint);
				double ksi = 10; // 1 funker bra


				// Translational (body fwd position)
				double error_surge = error;
				double p1 = Kp_surge*error_surge;
				i1 += p1/Ti_surge;//(ksi*error*cos(error_psi)*cos(error_psi)*cos(error_psi)*cos(error_psi))/Ti_surge;//(p1/Ti_surge);
				p1_last = p1;

				double max_u_b = 1;
				// Limit to current limit
				if(i1>max_u_b) i1 = max_u_b;
				else if(i1<-max_u_b) i1 = -max_u_b;
				//if(x_d.r_b>max_r_b) x_d.r_b = max_r_b;
				//else if(x_d.r_b<-max_r_b) x_d.r_b = -max_r_b;


				// Rotation (yaw angle)
				double p2 = Kp_psi*error_psi;
				i2 += (p2/Ti_psi);
				double d2 = (p2-p2_last)*Td_psi;
				p2_last = p2;


				// The wanted speed is then computed (switch to force?):
				desiredState->u_b = ksi*error*cos(error_psi)*cos(error_psi)*cos(error_psi)*cos(error_psi) + i1;	// That is body ref. speed forward (dx_b)
				if(lastAngleFix){
					desiredState->u_b = 0;
				}
				desiredState->r_b = (p2 + i2); // Turn rate.
//printf("i2: %f\n\r", i2);
				if(waypoints.at(currentWaypoint).id < 0){
					desiredState->u_b = -desiredState->u_b;
					//printf("LOS err: %f , Heading err: %f , Desired u: %f , Desired r: %f\n", error, error_psi, desiredState->u_b, desiredState->r_b);
				}

				if(lastAngleFix){
					//printf("Vinkel: feil %f, r_b %f, u_b %f\n\r", error_wp_psi, desiredState->r_b, desiredState->u_b);
				}

				//printf("LOS err: %f , Heading err: %f , Desired u: %f , Desired r: %f\n", error, error_psi, desiredState->u_b, desiredState->r_b);
				//printf("Position: %f, %f, %f\n\r", currentState->x_l, currentState->y_l,currentState->psi_l*(180/PI));
				//printf("Angle: %f, %f, %f, %f, %f\n\r", error_psi, psi_d*(180/PI), psi_l_d*(180/PI), currentState->psi_l*(180/PI), desiredState->r_b);
				//printf("Angle: p %f, i %f, d %f \n\r", p2, i2, d2);
				//printf("Surge: p %f, i %f\n\r", p1, i1);
			}
			else {
				//cout << "Reached end waypoint. LOS stopping robot.\n";
				desiredState->u_b = 0;	// That is body ref. speed forward (dx_b)
				desiredState->r_b = 0;// Turn rate
			}


		}









    private:
		vector<los_waypoint> waypoints;
		int currentWaypoint;
		int lastAngleFix;

		MessageHandler *messageHandler;
		Controller *headingAngleController;
		double i2;
		double i1;
};
#endif


