#include <math.h>

#include "AntiCollisionStrategy.h"
#include "StartStrategy.h"
#include "waypoints.h"
#include "PlansysObservation.h"
#include "observation_structs.h"
#include "posixCommunication.h"
#include "planner.h"

AntiCollisionStrategy::AntiCollisionStrategy ( PlansysObservation *s , int theTeamColor ){
	where = FRONT;	
	teamColor = theTeamColor;
	priority = 5;
	internalState = 1;
	strategyDone = FALSE;
	timeLeft = 90;
	Init();
	balls.InitializeBalls();
	_subject = s;
	_subject->Attach(this);
	_subject->SetCollision(FALSE);
	cout << "Strategy added" << endl;
}

AntiCollisionStrategy::~AntiCollisionStrategy (){
 	_subject->Detach(this);
}

void AntiCollisionStrategy::Reset( int theTeamColor ){
	where = FRONT;	
	_subject->SetCollision(FALSE);
	teamColor = theTeamColor;
	priority = 5;
	internalState = 1;
	strategyDone = FALSE;
	timeLeft = 90;
	Init();	
	balls.InitializeBalls();
	_subject->InitCollisionMessage();
	cout << "Strategy reset!" << endl;
}

void AntiCollisionStrategy::Init(){
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	collision = FALSE;
	collisionStop = FALSE;
	adjustCount = 0;
	state = COLLISION;
}

void AntiCollisionStrategy::UpdatePriority(){
	local_time_message = _subject->GetTimeObservation();
	timeLeft = local_time_message.timeLeft;	
	//cout << "Time left: " << timeLeft << endl;
	if(_subject->GetCollisionValue()){
		priority = 9;
	}
	else{
		priority = 0;
	}
}

void AntiCollisionStrategy::Update( int typeOfObservation ){
	//usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision = TRUE;
	}
	else if(typeOfObservation == MINOR_NAVSYS_POSITION){
		ai_waypoint_3d = _subject->GetPosObservation();
		cout <<  ai_waypoint_3d.x_l << ", " << ai_waypoint_3d.y_l << ", " << ai_waypoint_3d.psi_l << endl;
		positionReceived = TRUE;
	}
	else if(typeOfObservation == MINOR_LOCAL_WAYPOINT){
		opponentWaypoint = _subject->GetLocalWaypoint();
		opponentPositionReceived = TRUE;
	}
	
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		cout << "System reset request" << endl;
		_subject->SetCollision(FALSE);
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
}

int AntiCollisionStrategy::GetPriority(){
	return priority;
}

void AntiCollisionStrategy::CheckForCollision(MessageHandler *navsysMessageHandler){
	if(collision){
		_subject->GetCollisionValues(navsysMessageHandler,boards);
		
		cout << "board1: " << boards[0] << ", board2: " << boards[1] << ", board3: " << boards[2] << endl;

		if(boards[0]<=COLLISION_STOP || boards[1]<=COLLISION_STOP || boards[2]<=COLLISION_STOP){
			#ifdef ANTI_COLLISION_ACTIVE
				if(_subject->GetAntiCollisionActive()){
					//collisionStop = TRUE;
					_subject->SetCollision(TRUE);
				}
				
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		if(boards[0]>COLLISION_THRESHOLD+2 && boards[1]>COLLISION_THRESHOLD+2  && boards[2]>COLLISION_THRESHOLD+2){
			_subject->SetCollision(FALSE);
			//strategyDone = TRUE; //HACKHACKHACKHACKHACKHACKHACKHACKHACKHACKHACK
		}
		collision = FALSE;
	}
}

void AntiCollisionStrategy::CalculateOpponentWaypoint(MessageHandler *navsysMessageHandler, int where){
	int x,y,psi;
	x = ai_waypoint_3d.x_l;
	y = ai_waypoint_3d.y_l;
	psi = ai_waypoint_3d.psi_l;

	switch(where){
		case LEFT:
			relativeOpponentWaypoint.x_l = 200;
			relativeOpponentWaypoint.y_l = -150;
			break;
		case RIGHT:
			relativeOpponentWaypoint.x_l = 200;
			relativeOpponentWaypoint.y_l = 150;
			break;
		case FRONT:
			relativeOpponentWaypoint.x_l = 200;
			relativeOpponentWaypoint.y_l = 0;
			break;
		case REAR:
			relativeOpponentWaypoint.x_l = -200;
			relativeOpponentWaypoint.y_l = 0;
			break;
		default:
			break;
	}
	requestRobotToLocal(navsysMessageHandler,&relativeOpponentWaypoint);
}

ai_waypoint AntiCollisionStrategy::CalculateWaypoint()
{
	int xPos,yPos,x,y,oppX,oppY,lastX,lastY;
	double psiAngle,psi;
	ai_waypoint newAiWaypoint,checkedWaypoint,opponentWaypoint,lastActiveWaypoint;
	opponentWaypoint = _subject->GetLocalWaypoint();
	lastActiveWaypoint = _subject->GetLastWaypointSent();
	oppX = opponentWaypoint.x_l;
	oppY = opponentWaypoint.y_l;
	xPos = ai_waypoint_3d.x_l;
	yPos = ai_waypoint_3d.y_l;
	psiAngle = ai_waypoint_3d.psi_l;
	switch (where)
	{
		case LEFT:
			cout << "LEFT collision!" << endl;
			if((psiAngle <= 45*M_PI/180) && (psiAngle >= -45*M_PI/180)){
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			else if((psiAngle > 45*M_PI/180) && (psiAngle <= 135*M_PI/180)){
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			else if((psiAngle > 135*M_PI/180) || (psiAngle <=-135*M_PI/180)){
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			else{
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			newAiWaypoint.id = -2;
			break;
		case RIGHT:	
			cout << "RIGHT collision!" << endl;
			if((psiAngle <= 45*M_PI/180) && (psiAngle >= -45*M_PI/180)){
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			else if((psiAngle > 45*M_PI/180) && (psiAngle <= 135*M_PI/180)){
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			else if((psiAngle > 135*M_PI/180) || (psiAngle <=-135*M_PI/180)){
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			else{
				newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE;
				newAiWaypoint.y_l = 0;
				newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
			}
			newAiWaypoint.id = -2;
			break;
		case FRONT:
			cout << "FRONT collision!" << endl;
			if(psiAngle>=-45*M_PI/180 && psiAngle<45*M_PI/180){
				if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastY - oppY) <= 0) && (oppY > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastY - oppY) <= 0) && (oppY <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;

				}
			}
			else if(psiAngle>=45*M_PI/180 && psiAngle<135*M_PI/180){
				if(((lastX - oppX) > 0) && ((TABLE_SIZE_X - oppX) > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastX - oppX) > 0) && ((TABLE_SIZE_X - oppX) <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastX - oppX) <= 0) && (oppX > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastX - oppX) <= 0) && (oppX <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
			}
			else if(psiAngle>=135*M_PI/180 || psiAngle<-135*M_PI/180){
				if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastY - oppY) > 0) && ((TABLE_SIZE_Y - oppY) <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastY - oppY) <= 0) && (oppY > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastY - oppY) <= 0) && (oppY <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
			}
			else{
				if(((lastX - oppX) > 0 ) && ((TABLE_SIZE_X - oppX) > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastX - oppX) > 0 ) && ((TABLE_SIZE_X - oppX) <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastX - oppX) <= 0) && (oppX > WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = -ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
				else if(((lastX - oppX) <= 0) && (oppX <= WAYPOINT_SAFETY_DISTANCE)){
					newAiWaypoint.x_l = -ANTI_COLLISION_DISTANCE_FRONT;
					newAiWaypoint.y_l = 0;
					newAiWaypoint.psi_l = ANTI_COLLISON_ANGLE_CHANGE*M_PI/180;
				}
			}
			newAiWaypoint.id = -2;
			break;
		case REAR:
			cout << "REAR collision!" << endl;
			if(xPos<=(TABLE_SIZE_X/2) && yPos<=(TABLE_SIZE_Y/2)){
				newAiWaypoint.x_l = 0;
				if((psiAngle<=45*M_PI/180 && psiAngle>=-135*M_PI/180)){
					newAiWaypoint.psi_l = 90*M_PI/180;
					newAiWaypoint.y_l = ANTI_COLLISION_DISTANCE;
				}
				else{
					newAiWaypoint.psi_l = -90*M_PI/180;
					newAiWaypoint.y_l = -ANTI_COLLISION_DISTANCE;
				}
			}
			else if(xPos>(TABLE_SIZE_X/2) && yPos<=(TABLE_SIZE_Y/2)){
				newAiWaypoint.x_l = 0;
				if((psiAngle<=45*M_PI/180 && psiAngle>=-135*M_PI/180)){
					newAiWaypoint.psi_l = -90*M_PI/180;					
					newAiWaypoint.y_l = -ANTI_COLLISION_DISTANCE;
				}
				else{
					newAiWaypoint.psi_l = 90*M_PI/180;
					newAiWaypoint.y_l = ANTI_COLLISION_DISTANCE;
				}
			}
			else if(xPos>(TABLE_SIZE_X/2) && yPos>(TABLE_SIZE_Y/2)){
				newAiWaypoint.x_l = 0;
				if(psiAngle>=45*M_PI/180 && psiAngle<=225*M_PI/180){
					newAiWaypoint.psi_l = 90*M_PI/180;
					newAiWaypoint.y_l = ANTI_COLLISION_DISTANCE;
				}
				else{
					newAiWaypoint.psi_l = -90*M_PI/180;
					newAiWaypoint.y_l = -ANTI_COLLISION_DISTANCE;
				}
			}
			else if(xPos<=(TABLE_SIZE_X/2) && yPos>(TABLE_SIZE_Y/2)){
				newAiWaypoint.x_l = 0;
				if(psiAngle>=45*M_PI/180 && psiAngle<=225*M_PI/180){
					newAiWaypoint.psi_l = -90*M_PI/180;
					newAiWaypoint.y_l = -ANTI_COLLISION_DISTANCE;
				}
				else{
					newAiWaypoint.psi_l = 90*M_PI/180;
					newAiWaypoint.y_l = ANTI_COLLISION_DISTANCE;
				}
			}
			newAiWaypoint.id = 2;
			break;
		default:
			cout << "Invalid anticollison state" << endl;
			break;
	}
	cout <<  ai_waypoint_3d.x_l << ", " << ai_waypoint_3d.y_l << ", " << ai_waypoint_3d.psi_l << endl;
	cout << "x_l, y_l, psi_l: " << newAiWaypoint.x_l << ", " << newAiWaypoint.y_l << ", " << newAiWaypoint.psi_l << endl;
	//newAiWaypoint.psi_l = 0;
	newAiWaypoint.circle_of_acceptance = 50;
	return newAiWaypoint;
}

void AntiCollisionStrategy::AdjustPosition( MessageHandler *navsysMessageHandler ){
	collision_detection_message adjust_message;
	adjust_message.data[2] = COLLISION_STOP-2;
	adjust_message.data[3] = COLLISION_STOP-2;

	cout << "Start adjusting!" << endl;
	while(!collisionStop && _subject->GetCollisionValue()){
		if(adjustCount == 0){
			adjust_message.data[4] = 1;
		}
		else{
			adjust_message.data[4] = 0;
		}
		adjustCount ++;
		adjust_message.data[0] = COLLISION_STOP + 3;
		adjust_message.data[1] = COLLISION_STOP + 3;
		//cout << "Boards[0]: " << boards[0] << ", boards[1]: " << boards[1] << endl;
		//cout << "Adjusting: " << adjust_message.data[0] << endl;
		sendAdjustPosition(&adjust_message, navsysMessageHandler);
		usleep(STRATEGY_WHILE_DELAY);
		navsysMessageHandler->handleAllMessagesNow();
		CheckForCollision(navsysMessageHandler);	
	}
	if(collisionStop){
		adjust_message.data[4] = 2;
		clearWaypoints( navsysMessageHandler );
		usleep(ADJUST_COLLISION_TIMER);
		CheckForCollision(navsysMessageHandler);
		usleep(10000);
		if(_subject->GetCollisionValue()){
			state = COLLISION;
		}
		else{
			state = ADJUST;
		}
		collisionStop = FALSE;
		cout << "Collision stop!!!" << endl;
	}
	else if(!_subject->GetCollisionValue()){
		adjust_message.data[4] = 2;
		strategyDone = TRUE;
		cout << "Collision avoided!!!" << endl;
	}
	sendAdjustPosition( &adjust_message, navsysMessageHandler );
}

void AntiCollisionStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  ){
	Init();
	clearWaypoints( navsysMessageHandler );
	strategyDone = FALSE;
	cout << "Anti collision strategy" << endl;
	usleep(1000);
	navsysMessageHandler->handleAllMessagesNow();
	_subject->GetCollisionValues(navsysMessageHandler,boards);
	requestPosition(navsysMessageHandler);
	usleep(1000);
	//cout << timeLeft << endl;
	while(timeLeft>0 && !strategyDone){ 
		switch(state){		
			case ADJUST:
				AdjustPosition( navsysMessageHandler );
				break;
			case COLLISION:
				//cout << "Anti collision strategy" << endl;		
				if(!lastWaypointSent){
					if(boards[1]<=COLLISION_THRESHOLD && (boards[0]>COLLISION_THRESHOLD && boards[2]>COLLISION_THRESHOLD)){
						where = RIGHT;
						_subject->SetLastCollision(COLLISION_RIGHT);
					}
					else if(boards[0]<=COLLISION_THRESHOLD && (boards[1]>COLLISION_THRESHOLD && boards[2]>COLLISION_THRESHOLD)){
						where = LEFT;
						_subject->SetLastCollision(COLLISION_LEFT);
					}
					else if(boards[2]<=COLLISION_THRESHOLD){
						where = REAR;
						_subject->SetLastCollision(COLLISION_REAR);
					}
					else{
						where = FRONT;
						_subject->SetLastCollision(COLLISION_FRONT);
					}
					CalculateOpponentWaypoint(navsysMessageHandler,where);
					cout << "Waypoints cleared, prepare for reverse!!!" << endl;
					requestPosition(navsysMessageHandler);
					usleep(1000);					
					activeWaypoint = CalculateWaypoint();
					lastWaypointReached = FALSE;
					_subject->SetAntiCollisionActive(FALSE);
					usleep(100000);
					sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					_subject->SetAntiCollisionActive(TRUE);
					cout << "Reverse waypoint reached!!!" << endl;
					lastWaypointReached = FALSE;
					lastWaypointSent = FALSE;
					_subject->SetCollision(FALSE);
					_subject->SetSupportNeeded(TRUE);
					strategyDone = TRUE;
					usleep(10000);
				}
				break;
			default:
				break;
		}
		usleep(STRATEGY_WHILE_DELAY);
		CheckForCollision(navsysMessageHandler);
		local_time_message = _subject->GetTimeObservation();
		timeLeft = local_time_message.timeLeft;
		//UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
	//AdjustPosition( navsysMessageHandler );	
	priority = 0;
	_subject->SetCollision(FALSE);
	usleep(10000);
	navsysMessageHandler->handleAllMessagesNow();
}

