#ifndef ANTI_COLLISION_SUPPORT_STRATEGY_H
#define ANTI_COLLISION_SUPPORT_STRATEGY_H

#include <math.h>

#include "Observer.h"
#include "PlansysObservation.h"
#include "planner.h"
#include "waypoint.h"
#include "posix_message_config.h"
#include "init.h"
#include "observation_structs.h"

class AntiCollisionSupportStrategy: public Observer {
  public:
	AntiCollisionSupportStrategy( PlansysObservation * , int theTeamColor );
	~AntiCollisionSupportStrategy();
	void Reset( int theTeamColor );
	void Init();
	void Update( int typeOfObservation );
	void UpdatePriority();
	void CheckForCollision(MessageHandler *navsysMessageHandler);
	void CalculateSupportWaypoint();
	int GetPriority();
	void PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  );
	
  private:
	int lastWaypointSent;
	int lastWaypointReached;
	int positionReceived;
	
	int strategyDone;
	int collision;
	int collisionStop;
	int lastCollision;
	int adjustCount;

	int board1, board2, board3;
	int boards[NUMBER_OF_BOARDS];

	collision_detection_message collision_message;
	
	double internalState;
	double distance;
	double distance_x;
	double distance_y;
	int timeLeft;
	int priority;
	
	int waypointCounter;

	int state;
	int where;
	enum{
		INIT,
		SUPPORT,
		ANGLE_FIX,
		};
	
	PlansysObservation *_subject;
	ai_waypoint ai_waypoint_3d;
	ai_waypoint last_ai_waypoint;
	ai_waypoint activeWaypoint;
	ai_waypoint lastActiveWaypoint;
	ai_waypoint opponentWaypoint;
	ai_waypoint checkedWaypoint;
	//ai_waypoint allSupportWaypoints;	

	cv_observation my_cv_observation;
	proxy_message my_proxy_message;
	time_message local_time_message;
};

#endif
