#ifndef CONTAINER_STRATEGY_H
#define CONTAINER_STRATEGY_H

#include <math.h>

#include "Observer.h"
#include "PlansysObservation.h"
#include "planner.h"
#include "waypoint.h"
#include "posix_message_config.h"
#include "init.h"

class ContainerStrategy: public Observer {
  public:
	ContainerStrategy( PlansysObservation * , int theTeamColor );
	~ContainerStrategy();
	void Reset( int theTeamColor );
	void Init();
	void Update( int typeOfObservation );
	void UpdatePriority();
	int GetPriority();
	void CheckForCollision(MessageHandler *navsysMessageHandler);
	void EmptyRobot(MessageHandler *proxysysMessageHandler);
	void AdjustPosition( MessageHandler *navsysMessageHandler );
	void PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  );
  
  private:
	int lastWaypointSent;	
	int lastWaypointReached;

	collision_detection_message collision_message;	
	
	int ballsReleased;
	int strategyDone;
	int collision;
	int leftSwitch;
	int rightSwitch;
	int adjustCount;
	int minDistance;
	
	int board1, board2, board3, board4, board5;
	int boards[NUMBER_OF_BOARDS];

	double internalState;
	double distance;
	double distance_x;
	double distance_y;
	int timeLeft;
	int priority;
	
	int state;
	enum{
		INIT,
		RUNNING,
		ADJUST,
		RELEASE_BALLS,
		REVERSE,
		};
	
	PlansysObservation *_subject;
	ai_waypoint ai_waypoint_3d;
	ai_waypoint last_ai_waypoint;
	ai_waypoint activeWaypoint;

	cv_observation my_cv_observation;
	proxy_message my_proxy_message;
	time_message local_time_message;
};

#endif
