#ifndef DISPENSER_TEST_STRATEGY_H
#define DISPENSER_TEST_STRATEGY_H

#include <math.h>

#include "Observer.h"
#include "PlansysObservation.h"
#include "planner.h"
#include "waypoint.h"
#include "posix_message_config.h"
#include "init.h"

class DispenserTestStrategy: public Observer {
  public:
	DispenserTestStrategy( PlansysObservation * , int theTeamColor );
	~DispenserTestStrategy();
	void Reset( int theTeamColor );
	void Init();
	void Update( int typeOfObservation );
	int GetPriority();
	void UpdatePriority();
	void CheckForCollision();
	void CheckBall( MessageHandler *proxysysMessageHandler );
	void AdjustPosition( MessageHandler *navsysMessageHandler );
	void PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  );
  
  private:
	int lastWaypointSent;	
	int lastWaypointReached;
	int newBall;
	int ballStored;
	int wheelReady;
	int firstBall;

	int firstRun;
	
	collision_detection_message collision_message;		
	int strategyDone,strategyCompleted;
	int collision;
	int minDistanceDispenser;
	int adjustCount;

	int boards[NUMBER_OF_BOARDS];
	int ballColor;
	
	int robotState;
	double internalState;
	double distance;
	double distance_x;
	double distance_y;
	int timeLeft;
	int priority;
	
	int state;
	enum{
		FIRST_RUN,
		INIT,
		DISPENSER,
		PICK_UP_BALL,
		REVERSE,
	};

	PlansysObservation *_subject;
	ai_waypoint ai_waypoint_3d;
	ai_waypoint last_ai_waypoint;
	ai_waypoint activeWaypoint;
	ai_waypoint redWhiteDispenser;
	ai_waypoint blueWhiteDispenser;
	
	cv_observation my_cv_observation;
	cv_observation battery_bin_cv_observation;
	proxy_message my_proxy_message;
	time_message local_time_message;
};

#endif
