#include <math.h>

#include "HorizontalDispenserStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

HorizontalDispenserStrategy::HorizontalDispenserStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	state = INIT;
	strategyDone = FALSE;
	priority = 0;
	internalState = 1;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	timeLeft = 90;
	_subject = s;
	_subject->Attach(this);
	cout << "Strategy added" << endl;
}

HorizontalDispenserStrategy::~HorizontalDispenserStrategy (){
 	_subject->Detach(this);
}

void HorizontalDispenserStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	state = INIT;
	strategyDone = FALSE;
	priority = 0;
	internalState = 1;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	timeLeft = 90;
	cout << "Strategy reset!" << endl;
}

void HorizontalDispenserStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision_message = _subject->GetCollisionMessage();
		board1 = collision_message.data[0];
		board2 = collision_message.data[2];
		board3 = collision_message.data[3];
		cout << "\n\nBoard1: " << board1 << endl;
		cout << "Board2: " << board2 << endl;
		cout << "Board3: " << board3 << endl;
		if(board1<=20 || board2<=20 || board3<=20){
			_subject->SetCollision(TRUE);
			//cout << "\n\nCOLLISION\n\n" << endl;
		}
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation(); 
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;	
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
}

/*void ContainerStrategy::UpdateInternalState(double newInternalState){
	internalState = newInternalState;
}*/

int HorizontalDispenserStrategy::GetPriority(){
	return priority;
}

void HorizontalDispenserStrategy::UpdatePriority(){
	local_time_message = _subject->GetTimeObservation();
	
	timeLeft = local_time_message.timeLeft;
	cout << "Time left: " << timeLeft << endl;

	priority = 0;
}

void HorizontalDispenserStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  ){
	strategyDone = FALSE;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	while(!strategyDone && !_subject->GetCollisionValue()  && timeLeft>0){
		if(teamColor == TEAM_COLOR_RED){
			switch (state){
				case INIT:
					cout << "ContainerStrategy: state = INIT" << endl;
					if(!lastWaypointSent){
						activeWaypoint = GetWaypoint(STANDARD_CONTAINER_RED_WAYPOINT);
						sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
						cout << "Waypoint, red standard container, x_l: " << activeWaypoint.x_l << endl;
						lastWaypointSent = TRUE;
						usleep(10000);
					}
					if(lastWaypointReached){
						usleep(300000);
						lastWaypointReached = FALSE;
						lastWaypointSent = FALSE;
						strategyDone = TRUE;
						usleep(10000);
					}
					break;
				default:
					cout << "WRONG STATE!" << endl;
					break;
			}
		}
		else if(teamColor == TEAM_COLOR_BLUE){
			switch (state){
				case INIT:
					cout << "ContainerStrategy: state = INIT" << endl;
					if(!lastWaypointSent){
						activeWaypoint = GetWaypoint(STANDARD_CONTAINER_BLUE_WAYPOINT);
						sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
						cout << "Waypoint, blue standard container, x_l: " << activeWaypoint.x_l << endl;
						lastWaypointSent = TRUE;
						usleep(10000);
					}
					if(lastWaypointReached){
						usleep(300000);
						lastWaypointReached = FALSE;
						lastWaypointSent = FALSE;
						strategyDone = TRUE;
						usleep(10000);
					}
					break;
				default:
					cout << "WRONG STATE!" << endl;
					break;
			}
		}
		usleep(300000); 
		UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
}
