#include <math.h>

#include "PlansysObservation.h"
#include "posixCommunication.h"
#include "planner.h"

extern int positionReceived;
extern int teamColor;

PlansysObservation::PlansysObservation(){
	ai_waypoint_3d_ptr = &ai_waypoint_3d;
	
	last_ai_waypoint_3d_reached.x_l = -1;
	last_ai_waypoint_3d_reached.y_l = -1;
	last_ai_waypoint_3d_reached.psi_l = 0.0;
	last_ai_waypoint_3d_reached.circle_of_acceptance = 20; 
	
	lastWaypointSent.x_l = 1500;
	lastWaypointSent.y_l = 1050;
	lastWaypointSent.psi_l = 0;
	lastWaypointSent.circle_of_acceptance = 50;

	localWaypoint.x_l = 0;
	localWaypoint.y_l = 0;
	localWaypoint.psi_l = 0;
	localWaypoint.circle_of_acceptance = 0;

	supportNeeded = FALSE;

	antiCollisionActive = FALSE;
	rearAntiCollisionActive = FALSE;
	firstRun = TRUE;

	ballStored = 0;
	wheelReady = 0;
	newBall = 0;

	collision = FALSE;

	lastWaypointSentCounter = 0;
	lastWaypointReachedTag = 0;

	for(int i=0;i<POSITION_TIMER_BUFFER_SIZE;i++){
		positions[i] = localWaypoint;
	}
	index = 0;
}

int PlansysObservation::NewObservation(pmq_message pm, int typeOfObservation) {
	if( typeOfObservation == MINOR_COLLISION_DETECTED ) {
		memcpy((void *)(&collision_message),( void * )pm.data, sizeof(collision_detection_message));
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ) {
		posix_robot_state_3dof pos_robot;
		memcpy((void *)(&pos_robot),( void * )pm.data, sizeof(posix_robot_state_3dof));
		ai_waypoint_3d.x_l = pos_robot.x_l;
		ai_waypoint_3d.y_l = pos_robot.y_l;
		ai_waypoint_3d.psi_l = pos_robot.psi_l;
		ai_waypoint_3d.circle_of_acceptance = 0;
		AngleWrap();
		positionReceived = TRUE;	
	}
	else if( typeOfObservation == MINOR_LOCAL_WAYPOINT ) {
		cout << "Local waypoint received!" << endl;
		posix_robot_state_3dof pos_robot;		
		memcpy((void *)(&pos_robot),( void * )pm.data, sizeof(posix_robot_state_3dof));		
		localWaypoint.x_l = pos_robot.x_l;
		localWaypoint.y_l = pos_robot.y_l;
		localWaypoint.psi_l = pos_robot.psi_l;
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ) {
		memcpy((void *)(&last_ai_waypoint_3d_reached),( void * )pm.data, sizeof(ai_waypoint));
		lastWaypointReachedTag = last_ai_waypoint_3d_reached.tag;
	}
	else if (typeOfObservation == MINOR_CV_OBSERVATION ) {
		memcpy((void *)(&my_cv_observation),( void * )pm.data, sizeof(cv_observation));
	}
	else if (typeOfObservation == MINOR_CV_COLOR_OBSERVATION ) {
		memcpy((void *)(&color_cv_observation),( void * )pm.data, sizeof(cv_observation));
	}
	else if (typeOfObservation == GAME_TIME_MESSAGE ) {
		main_time.timeLeft = pm.length;
	}
	else if (typeOfObservation == MINOR_CMD_LOW_BATTERY ) {
		cout << "Low battery!" << endl;
	}
	else if (typeOfObservation == MINOR_BALL_STORED ) { 
		memcpy((void *)(&new_ball_message),( void * )pm.data, sizeof(ball_message));
		ballStored ++;
	}
	else if(typeOfObservation == MINOR_WHEEL_READY){
		cout << "Wheel ready!" << endl;
		wheelReady ++;
	}
	else if (typeOfObservation == MINOR_BALLS_RELEASED ) {
		memcpy((void *)(&balls_released_message),( void * )pm.data, sizeof(ball_message));
	}
	else if (typeOfObservation == MINOR_COLOR_DETECTED ) {
		memcpy((void *)(&new_ball_message),( void * )pm.data, sizeof(ball_message));
		newBall ++;
	}
	else if(typeOfObservation == MINOR_CMD_SWITCHES ){
		memcpy((void *)(&my_switch_message),( void * )pm.data, sizeof(switch_message));
	}
	if(typeOfObservation != 200){
  		Notify( typeOfObservation );
	}
}

int PlansysObservation::GetStrategyPriority(){
	int i,max,priority,strategy;
	max = 0;
	strategy = 0;
	for(i=0; i<_observers.size(); i++){
		priority = (_observers.at(i))->GetPriority();
		cout << "Strategy: " << i << ", Priority: " << priority << endl;
		if(priority>max){
			max = priority;
			strategy = i;
		}
	}
	cout << "Chosen strategy: " << strategy << endl;
	return strategy;
}

void PlansysObservation::ResetStrategies( int teamColor ){
	int i;
	for(i=0;i<_observers.size();i++){
		(_observers.at(i))->Reset( teamColor );
	}	
}

void PlansysObservation::UpdateStrategyPriority(){
	int i;
	for(i=0; i<_observers.size(); i++){
		(_observers.at(i))->UpdatePriority();
	}
}

void PlansysObservation::GetCollisionValues(MessageHandler *navsysMessageHandler, int* boards){
	int x,y,collision;
	double robotPsi;

	#ifdef FRONT_LEFT_ANTI_COLLISION
		boards[0] = collision_message.data[FRONT_LEFT_ANTI_COLLISION];
	#else
		boards[0] = BOARD_NOT_ACTIVE;
	#endif
	#ifdef FRONT_RIGHT_ANTI_COLLISION
		boards[1] = collision_message.data[FRONT_RIGHT_ANTI_COLLISION];
	#else
		boards[1] = BOARD_NOT_ACTIVE;
	#endif
	#ifdef REAR_ANTI_COLLISION
		boards[2] = collision_message.data[REAR_ANTI_COLLISION];
	#else
		boards[2] = BOARD_NOT_ACTIVE;
	#endif
	#ifdef LEFT_ADJUST
		boards[3] = collision_message.data[LEFT_ADJUST];
	#else
		boards[3] = BOARD_NOT_ACTIVE;
	#endif
	#ifdef RIGHT_ADJUST
		boards[4] = collision_message.data[RIGHT_ADJUST];
	#else
		boards[4] = BOARD_NOT_ACTIVE;
	#endif

	collision = TRUE;

	if((boards[0] <= COLLISION_THRESHOLD) || (boards[1] <= COLLISION_THRESHOLD) || (boards[2] <= COLLISION_THRESHOLD)){
		requestPosition(navsysMessageHandler);
		usleep(1000);
		x = ai_waypoint_3d.x_l;
		y = ai_waypoint_3d.y_l;
		robotPsi = ai_waypoint_3d.psi_l;
		if(robotPsi>=-90*M_PI/180 && robotPsi<0*M_PI/180){
			if( (x > 2700) || (y < 300)){
				collision = FALSE;
				cout << "Collision message ignored!!!" << endl;
				return;
			}
		}
		else if(robotPsi>=0*M_PI/180 && robotPsi<90*M_PI/180){
			if( (x > 2700) || (y > 1800)){
				collision = FALSE;
				cout << "Collision message ignored!!!" << endl;
				return;
			}
		}
		else if(robotPsi>=90*M_PI/180 || robotPsi<180*M_PI/180){
			if( (x < 300) || (y > 1800)){
				collision = FALSE;
				cout << "Collision message ignored!!!" << endl;
				return;
			}
		}
		else{
			if( (x < 300) || (y < 300)){
				collision = FALSE;
				cout << "Collision message ignored!!!" << endl;
				return;
			}
		}
	}

	if(bufferCounter > BUFFER_SIZE-1){
		bufferCounter = 0;
	}
	if(collision){
		if(boards[0] > 50){
			board1Buffer[bufferCounter] = 50;
		}
		else{
			board1Buffer[bufferCounter] = boards[0];
		}
		if(boards[1] > 50){
			board2Buffer[bufferCounter] = 50;
		}
		else{
			board2Buffer[bufferCounter] = boards[1];
		}
		if(boards[2] > 50){
			board3Buffer[bufferCounter] = 50;
		}
		else{
			board3Buffer[bufferCounter] = boards[2];
		}
	}
	else{
		board1Buffer[bufferCounter] = BOARD_NOT_ACTIVE;
		board2Buffer[bufferCounter] = BOARD_NOT_ACTIVE;
		board3Buffer[bufferCounter] = BOARD_NOT_ACTIVE;		
	}
	bufferCounter ++;
	board1Average = 0;	
	board2Average = 0;
	board3Average = 0;
	for(int i=0;i<BUFFER_SIZE;i++){
		board1Average += board1Buffer[i];
		board2Average += board2Buffer[i];
		board3Average += board3Buffer[i];
	}
	board1Average = board1Average/BUFFER_SIZE;
	board2Average = board2Average/BUFFER_SIZE;
	board3Average = board3Average/BUFFER_SIZE;

	boards[0] = board1Average;
	boards[1] = board2Average;
	boards[2] = board3Average;

	cout << "1,0: " << board1Buffer[0] << ", 1,1: " << board1Buffer[1] << ", 1,2: " << board1Buffer[2] << endl;
	cout << "2,0: " << board2Buffer[0] << ", 2,1: " << board2Buffer[1] << ", 2,2: " << board2Buffer[2] << endl;
	cout << "3,0: " << board3Buffer[0] << ", 3,1: " << board3Buffer[1] << ", 3,2: " << board3Buffer[2] << endl;
}

void PlansysObservation::SetCollision(int choice){
	collision = choice;
	if(choice == FALSE){
		InitCollisionMessage();	
	}
}

int PlansysObservation::GetCollisionValue(){
	return collision;
}

int PlansysObservation::CheckBall( MessageHandler *proxysysMessageHandler, int nextBallColor, int preferredColor ){
	int ballColor = new_ball_message.data[0];
	ballColor = preferredColor;
	cout << "Ball color received: " << ballColor << endl;
	if(ballColor == teamColor || ballColor == WHITE){
		new_ball_message.data[0] = 1;
	}
	else{
		new_ball_message.data[0] = 0;
	}
	if(firstRun){
		new_ball_message.data[0] = 1;		
		new_ball_message.data[1] = preferredColor; 
		if(ballColor == WHITE){
			return WHITE;
		}
		else if(ballColor == teamColor){
			return 1;
		}
		else{
			return 1;
		}
	}
	else{
		new_ball_message.data[1] = ballColor;
		if(ballColor == WHITE){
			return WHITE;
		}
		else if(ballColor == teamColor){
			return 1;
		}
		else{
			return -1;
		}
	}
	new_ball_message.data[2] = nextBallColor;
	usleep(1000);
}

int PlansysObservation::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler, int strategy ){
	(_observers.at(strategy))->PerformStrategy(navsysMessageHandler,proxysysMessageHandler,cvsysMessageHandler);
	return 0;
}

ai_waypoint PlansysObservation::GetPosObservation() {
	//cout <<  ai_waypoint_3d.x_l << ", " << ai_waypoint_3d.y_l << ", " << ai_waypoint_3d.psi_l << endl;
  	return ai_waypoint_3d;
}

ai_waypoint PlansysObservation::GetWaypointReachedObservation() {
  	return last_ai_waypoint_3d_reached;
}

cv_observation PlansysObservation::GetCvObservation() {
  	return my_cv_observation;
}

cv_observation PlansysObservation::GetCvColorObservation() {
  	return color_cv_observation;
}

ball_message PlansysObservation::GetNewBallMessage(){
	return new_ball_message;
}

ball_message PlansysObservation::GetBallsReleasedMessage(){
	return balls_released_message;
}

int PlansysObservation::GetSwitchMessage(){
	return my_switch_message.data;
}

time_message PlansysObservation::GetTimeObservation() {
  	return main_time;
}

proxy_message PlansysObservation::GetProxyMessage() {
 	return my_proxy_message; 
}

int PlansysObservation::GetFirstRunVariable(){
	return firstRun;
}

void PlansysObservation::InitCollisionMessage(){
	int i;
	for(i=0;i<8;i++){
		collision_message.data[i] = 50;
	}
	bufferCounter = 0;
	for(int i=0;i<BUFFER_SIZE;i++){
		board1Buffer[i] = 50;
		board2Buffer[i] = 50;
		board3Buffer[i] = 50;
	}
}

collision_detection_message PlansysObservation::GetCollisionMessage() {
  	return collision_message;
}

void PlansysObservation::SetAntiCollisionActive(int value){
	antiCollisionActive = value;
}

void PlansysObservation::SetRearAntiCollisionActive(int value){
	rearAntiCollisionActive = value;
}

int PlansysObservation::GetAntiCollisionActive(){
	return antiCollisionActive;
}

int PlansysObservation::GetRearAntiCollisionActive(){
	return rearAntiCollisionActive;
}

void PlansysObservation::SetLastCollision(int where){
	lastCollision = where;
}

int PlansysObservation::GetLastCollision(){
	return lastCollision;
}

void PlansysObservation::SetLastWaypointSent(ai_waypoint lastWaypoint){
	lastWaypointSent = lastWaypoint;
}

void PlansysObservation::SetFirstRunVariable(int value){
	firstRun = value;
}

void PlansysObservation::SetSupportNeeded(int value){
	supportNeeded = value;
}

ai_waypoint PlansysObservation::GetLastWaypointSent(){
	return lastWaypointSent;
}

ai_waypoint PlansysObservation::GetLocalWaypoint(){
	return localWaypoint;
}

int PlansysObservation::CheckPositionTimeout(){
	ai_waypoint average;
	ai_waypoint current;
	int x = 0;
	int y = 0;
	double psi = 0;
	positions[index] = ai_waypoint_3d;
	index++;
	if(index >= POSITION_TIMER_BUFFER_SIZE){
		index = 0;
	}
	for(int i=0;i<POSITION_TIMER_BUFFER_SIZE;i++){
		current = positions[i];
		x += current.x_l;
		y += current.y_l;
		psi += current.psi_l;
	}
	average.x_l = x/POSITION_TIMER_BUFFER_SIZE;
	average.y_l = y/POSITION_TIMER_BUFFER_SIZE;
	average.psi_l = psi/POSITION_TIMER_BUFFER_SIZE;
	cout << "Average x: " << average.x_l << ", y: " << average.y_l << endl;
	if((fabs(average.x_l - ai_waypoint_3d.x_l) <= 10) && (fabs(average.y_l - ai_waypoint_3d.y_l) <= 10)){
		return TRUE;
	}
	else{
		return FALSE;
	}
}

ai_waypoint PlansysObservation::CheckWaypointFeasibility(ai_waypoint waypoint, ai_waypoint my_position){
	ai_waypoint newWaypoint;
	int newX,newY;
	newWaypoint = waypoint; 
	newWaypoint.x_l = waypoint.x_l;
	newWaypoint.y_l = waypoint.y_l;
	newX = waypoint.x_l+my_position.x_l;
	newY = waypoint.y_l+my_position.y_l;
	if(waypoint.id == 1){
		if(newX >= (TABLE_SIZE_X - 150)){
			newWaypoint.x_l = TABLE_SIZE_X - my_position.x_l;
		}
		else if(newX <= 150){
			newWaypoint.x_l = 150;
		}
		if(newY >= (TABLE_SIZE_Y - 150)){
			newWaypoint.y_l = TABLE_SIZE_Y - my_position.y_l;
		}
		else if(newY <= 150){
			newWaypoint.y_l = 150;
		}
	}
	/*else if(waypoint.id == -1){
		if(newX >= (TABLE_SIZE_X - 150)){
			newX = newX - my_position.x_l;
		}
		else if(newX <= 150){
			newX = -(my_position.x_l - 150);
		}
		if(newY >= (TABLE_SIZE_Y - 150)){
			newY = newY - my_position.y_l;
		}
		else if(newY <= 150){
			newY = -(my_position.y_l - 150);
		}
	}*/
	return newWaypoint;	
}

void PlansysObservation::AngleWrap(){
	if(ai_waypoint_3d.psi_l>180){
		ai_waypoint_3d.psi_l = ai_waypoint_3d.psi_l - 360;
	}
}

int PlansysObservation::GetSupportNeeded(){
	return supportNeeded;
}

int PlansysObservation::GetNewBall(){
	return newBall;
}

int PlansysObservation::GetWheelReady(){
	return wheelReady;
}

int PlansysObservation::GetBallStored(){
	return ballStored;
}

/*void PlansysObservation::SetBallStoredFalse(){
	ballStored[0] = FALSE;
}

void PlansysObservation::SetWheelReadyFalse(){
	wheelReady[0] = FALSE;
}

void PlansysObservation::SetNewBallFalse(){
	newBall[0] = FALSE;
}*/

void PlansysObservation::CalculateHalfCircleWaypoints(int radius, int where){
	int diameter, r, numberOfWaypoints;
	numberOfWaypoints = NUMBER_OF_SUPPORT_WAYPOINTS;
	int xArray[numberOfWaypoints+1];
	int yArray[numberOfWaypoints+1];
	double psiArray[numberOfWaypoints];
	ai_waypoint activeWaypoint;
	r = radius;

	if(r<400){
		r = 400;
	}
	
	diameter = 2*r;

	xArray[0] = 0;
	yArray[0] = 0;

	if(where == COLLISION_LEFT){
		cout << "1" << endl;
		for(int i=numberOfWaypoints;i>0;i--){
			yArray[i] = diameter*(i)/numberOfWaypoints;
			xArray[i] = sqrt(pow(r,2) - pow((r - yArray[i]),2));
		}
		//xArray[numberOfWaypoints+1] = xPos;
		for(int i=0;i<numberOfWaypoints;i++){
			psiArray[i] = atan2((xArray[i+1] - xArray[i]),(yArray[i+1] - yArray[i]));
		}
		psiArray[0] = 90*M_PI/180;
	}
	else if(where == COLLISION_RIGHT){
		cout << "2" << endl;
		for(int i=numberOfWaypoints;i>0;i--){
			yArray[i] = diameter*(i)/numberOfWaypoints;
			xArray[i] = -sqrt(pow(r,2) - pow((r - yArray[i]),2));
		}
		//xArray[numberOfWaypoints+1] = xPos;
		for(int i=0;i<numberOfWaypoints;i++){
			psiArray[i] = atan2((xArray[i+1] - xArray[i]),(yArray[i+1] - yArray[i]));
		}
		psiArray[0] = -90*M_PI/180;
	}
	for(int i=0;i<numberOfWaypoints;i++){
		cout << "x nr: " << i+1 << ", x: " << xArray[i+1] << ", y: " << yArray[i+1] << endl;
		activeWaypoint.id = 2;
		activeWaypoint.x_l = yArray[i];
		activeWaypoint.y_l = xArray[i];
		if(i<numberOfWaypoints-1){
			activeWaypoint.psi_l = psiArray[i]; 
		}
		else{
			activeWaypoint.psi_l = 0;
		}
		activeWaypoint.circle_of_acceptance = 50;
		waypointArray[i] = activeWaypoint;
	}
}

ai_waypoint PlansysObservation::GetSupportWaypoint(int index){
	return waypointArray[index];
}

int PlansysObservation::GetLastWaypointSentCounter(){
	return lastWaypointSentCounter;
}

void PlansysObservation::IncrementLastWaypointSentCounter(){
	lastWaypointSentCounter++;
}

int PlansysObservation::GetLastWaypointReachedTag(){
	return lastWaypointReachedTag;
}

void PlansysObservation::SetLastWaypointReachedTag(int value){
	lastWaypointReachedTag = value;
}
