#include <math.h>

#include "RedBlueDispenserStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

RedBlueDispenserStrategy::RedBlueDispenserStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	redDispenser = GetWaypoint(RED_DISPENSER_WAYPOINT);
	blueDispenser = GetWaypoint(BLUE_DISPENSER_WAYPOINT);
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = FIRST_RUN;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalState = 5;
	extraBall = 0;
	timeLeft = 90;
	numberOfTries = 0;
	Init();
	_subject = s;
	_subject->Attach(this);
	_subject->SetFirstRunVariable(TRUE);
	cout << "Strategy added" << endl;
}

RedBlueDispenserStrategy::~RedBlueDispenserStrategy (){
 	_subject->Detach(this);
}

void RedBlueDispenserStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	redDispenser = GetWaypoint(RED_DISPENSER_WAYPOINT);
	blueDispenser = GetWaypoint(BLUE_DISPENSER_WAYPOINT);
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = FIRST_RUN;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalState = 5;
	extraBall = 0;
	timeLeft = 90;
	numberOfTries = 0;
	_subject->SetFirstRunVariable(TRUE);
	Init();
	cout << "Strategy reset!" << endl;
}

void RedBlueDispenserStrategy::Init(){
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	collision = FALSE;
	newBall = FALSE;
	ballStored = FALSE;
	wheelReady = FALSE;
	firstBall = TRUE;
	minDistanceDispenser = FALSE;
	adjustCount = 0;
	tryCounter = 0;
	tryAgain = FALSE;
	lastTryOk = TRUE;
	ballCheck = 0;
}

int RedBlueDispenserStrategy::GetPriority(){
	return priority;
}

void RedBlueDispenserStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){	
		collision = TRUE;
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation();
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_BALL_STORED ){
		ballStored = TRUE;
	}
	else if( typeOfObservation == MINOR_WHEEL_READY){
		wheelReady = TRUE;
		firstBall = FALSE;
	}
	else if( typeOfObservation == MINOR_COLOR_DETECTED ){
		newBall = TRUE;
	}
}

void RedBlueDispenserStrategy::UpdatePriority(){
	double maxDistance;
	int redBlueBalls;
	local_time_message = _subject->GetTimeObservation();
	timeLeft = local_time_message.timeLeft;

	ai_waypoint_3d = _subject->GetPosObservation();
	if( teamColor == TEAM_COLOR_RED ){
		distance_x = sqrt(pow((redDispenser.x_l - ai_waypoint_3d.x_l),2));
		distance_y = sqrt(pow((redDispenser.y_l - ai_waypoint_3d.y_l),2));
		distance = sqrt(pow(distance_x,2)+pow(distance_y,2));
		internalState = balls.GetNumberOfBalls(RED_BALL);
	}
	else if( teamColor == TEAM_COLOR_BLUE ){
		distance_x = sqrt(pow((blueDispenser.x_l - ai_waypoint_3d.x_l),2));
		distance_y = sqrt(pow((blueDispenser.y_l - ai_waypoint_3d.y_l),2));
		distance = sqrt(pow(distance_x,2)+pow(distance_y,2));
		internalState = balls.GetNumberOfBalls(BLUE_BALL);
	}
	if( !(int)internalState ){
		strategyCompleted = TRUE;
	}
	maxDistance = sqrt(pow(3000,2)+pow(2100,2));

	redBlueBalls = balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL);

	extraBall = balls.GetExtraBall(RED_BLUE_ROBOT_BALL);

	if(redBlueBalls>=3){
		robotState = -1;
	}
	else if(redBlueBalls<3 && redBlueBalls>1){
		robotState = 3;
	}
	else{
		robotState = 5;
	}

	if(numberOfTries > MAX_NUMBER_OF_TRIES){
		strategyCompleted = TRUE;
	}

	if(timeLeft<15){
		strategyCompleted = TRUE;
	}

	if( !strategyCompleted ){
		priority = ((((maxDistance-distance)/maxDistance)*0.2 + internalState*0.10 + robotState*0.06 + extraBall*0.06)*0.9)*11;
		if(priority>9){
			priority = 9;
		}
	}
	else{
		priority = 0;
	}
}

void RedBlueDispenserStrategy::CheckForCollision(MessageHandler *navsysMessageHandler){
	int x,y,r;
	if(collision){
		/*board1 = collision_message.data[1];
		board2 = collision_message.data[5];
		//board3 = collision_message.data[6];
		board3 = 50;
		board4 = collision_message.data[0];
		board5 = collision_message.data[4];
		cout << "board4: " << board4 << ", board5: " << board5 << endl;
		x = ai_waypoint_3d.x_l;
		y = ai_waypoint_3d.x_l;
		if(teamColor == TEAM_COLOR_RED){
			r = sqrt(pow((x-2300),2)+pow(y,2));
		}
		else if(teamColor == TEAM_COLOR_BLUE){
			r = sqrt(pow((x-700),2)+pow(y,2));
		}
		//if(board4<=DISTANCE_MEASURE_THRESHOLD && board5<=DISTANCE_MEASURE_THRESHOLD && board1>=COLLISION_THRESHOLD && board2>=COLLISION_THRESHOLD && board3>=COLLISION_THRESHOLD && r<30){
		if(board4<=DISTANCE_MEASURE_THRESHOLD && board5<=DISTANCE_MEASURE_THRESHOLD){
			minDistanceDispenser = TRUE;
		}
		else if(board1<=COLLISION_THRESHOLD || board2<=COLLISION_THRESHOLD || board3<=COLLISION_THRESHOLD){
			#ifdef ANTI_COLLISION_ACTIVE
				_subject->SetCollision(TRUE);
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		collision = FALSE;*/
		_subject->GetCollisionValues(navsysMessageHandler,boards);
		cout << "board4: " << boards[3] << ", board5: " << boards[4] << endl;

		/*x = ai_waypoint_3d.x_l;
		y = ai_waypoint_3d.y_l;
		r = sqrt(pow(x,2)+pow((y-750),2));
		cout << "Radius: " << r << endl;*/

		if(boards[3]<=DISTANCE_MEASURE_THRESHOLD && boards[4]<=DISTANCE_MEASURE_THRESHOLD){
			minDistanceDispenser = TRUE;
		}
		if(boards[0]<=COLLISION_THRESHOLD || boards[1]<=COLLISION_THRESHOLD){
			#ifdef ANTI_COLLISION_ACTIVE
				if(_subject->GetAntiCollisionActive()){
					_subject->SetCollision(TRUE);
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		else if(boards[2]<=COLLISION_THRESHOLD){
			#ifdef REAR_ANTI_COLLISION_ACTIVE
				if(_subject->GetRearAntiCollisionActive()){
					_subject->SetCollision(TRUE);
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
			
		}
		collision = FALSE;
	}
}

void RedBlueDispenserStrategy::CheckBall( MessageHandler *proxysysMessageHandler, MessageHandler *navsysMessageHandler, int nextBallColor){
	int preferredColor;
	if(teamColor == TEAM_COLOR_RED){
		preferredColor = RED;
	}	
	if(teamColor == TEAM_COLOR_BLUE){
		preferredColor = BLUE;
	}
	ballCheck = _subject->CheckBall(proxysysMessageHandler, nextBallColor, preferredColor);
	if(ballCheck>-1){
		my_ball_message = _subject->GetNewBallMessage();
		my_ball_message.data[1] = preferredColor;
		sendBallAck( &my_ball_message, proxysysMessageHandler );
		balls.AddRobotBalls(RED_BLUE_ROBOT_BALL,1);
		if(teamColor == TEAM_COLOR_RED){						
			balls.RemoveBall(RED_BALL, 1);
		}
		else if(teamColor == TEAM_COLOR_BLUE){
			balls.RemoveBall(BLUE_BALL, 1);
		}
		/*if(ballCheck == WHITE){
			balls.AddRobotBalls(WHITE_ROBOT_BALL,1);
		}
		else{
			balls.AddRobotBalls(RED_BLUE_ROBOT_BALL,1);
			if(teamColor == TEAM_COLOR_RED){						
				balls.RemoveBall(RED_BALL, 1);
			}
			else if(teamColor == TEAM_COLOR_BLUE){
				balls.RemoveBall(BLUE_BALL, 1);
			}
		}*/
	}
	else{
		sendRelativeWaypoint(navsysMessageHandler, 0, 0, -45*M_PI/180, 50);
		while(!lastWaypointReached){
			navsysMessageHandler->handleAllMessagesNow();
			usleep(1000);
		
		}
		lastWaypointReached = FALSE;
		sendBallAck( &my_ball_message, proxysysMessageHandler );
		sendRelativeWaypoint(navsysMessageHandler, 0, 0, 45*M_PI/180, 50);
		while(!lastWaypointReached){
			navsysMessageHandler->handleAllMessagesNow();
			usleep(1000);
		}
		lastWaypointReached = FALSE;
	}
}

void RedBlueDispenserStrategy::AdjustPosition( MessageHandler *navsysMessageHandler ){
	collision_detection_message adjust_message;
	adjust_message.data[0] = boards[3];
	adjust_message.data[1] = boards[4];
	cout << "board4: " << board4 << ", board5: " << board5 << endl;
	adjust_message.data[2] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	adjust_message.data[3] = MINIMUM_DISPENSER_STOP_THRESHOLD;
	if(adjustCount == 0){
		adjust_message.data[4] = 1;
	}
	else{
		adjust_message.data[4] = 0;
	}
	sendAdjustPosition( &adjust_message, navsysMessageHandler );
	adjustCount ++;
}

void RedBlueDispenserStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler ){
	int startTime = 0;
	int nextBall = COLORED_BALL;
	numberOfTries ++;
	if(strategyDone){
		strategyDone = FALSE;
		_subject->SetAntiCollisionActive(TRUE);
		state = FIRST_RUN;
	}
	cout << "RedBlueDispenserStrategy: state = FIRST_RUN" << endl;
	cout << "Time left: " << timeLeft << endl;
	Init();
	while(!strategyCompleted && !strategyDone && !_subject->GetCollisionValue() && timeLeft>20){
		switch (state) {
			case FIRST_RUN:
				if(!_subject->GetFirstRunVariable()){
					numberOfBallsToPickUp = (5-balls.GetNumberOfWhiteBallsToPickUp());
					if(!lastWaypointSent){
						if(teamColor == TEAM_COLOR_RED){
							activeWaypoint = GetWaypoint(RED_DISPENSER_SUPPORT_WAYPOINT);
						}
						else if(teamColor == TEAM_COLOR_BLUE){
							activeWaypoint = GetWaypoint(BLUE_DISPENSER_SUPPORT_WAYPOINT);
						}
						sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
						_subject->SetLastWaypointSent(activeWaypoint);
						lastWaypointSent = TRUE;
						usleep(10000);
					}
					if(lastWaypointReached){
						_subject->SetAntiCollisionActive(FALSE);
						lastWaypointReached = FALSE;
						lastWaypointSent = FALSE;
						state = INIT;
						usleep(10000);
					}
				}
				else{
					numberOfBallsToPickUp = (5-balls.GetNumberOfWhiteBallsToPickUp());
					if(balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL) < balls.GetNumberOfWhiteBallsToPickUp()){
						numberOfBallsToPickUp += balls.GetNumberOfWhiteBallsToPickUp() - balls.GetNumberOfRobotBalls(WHITE_ROBOT_BALL);
					}
					state = INIT;			
				}
				break;
			case INIT:
				if(!lastWaypointSent){
					cout << "RedBlueDispenserStrategy: state = INIT" << endl;
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					if(teamColor == TEAM_COLOR_RED){
						activeWaypoint = GetWaypoint(RED_DISPENSER_WAYPOINT);
					}
					else if(teamColor == TEAM_COLOR_BLUE){
						activeWaypoint = GetWaypoint(BLUE_DISPENSER_WAYPOINT);
					}
					sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
					_subject->SetLastWaypointSent(activeWaypoint);						
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					lastWaypointReached = FALSE;
					sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					usleep(100000);
					sendReverseWaypoint( navsysMessageHandler, -100, 0, 0, 30, RELATIVE_WAYPOINT );
					while(!lastWaypointReached){
						navsysMessageHandler->handleAllMessagesNow();
					}
					lastWaypointReached = FALSE;
					startTime = timeLeft;
					state = PICK_UP_BALL;					
					usleep(10000);
				}
				if(minDistanceDispenser && lastWaypointReached){
					clearWaypoints( navsysMessageHandler );
					#ifdef ADJUSTING
						state = DISPENSER;
					#else
						state = PICK_UP_BALL;
					#endif
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				break;
			case DISPENSER:
				//cout << "RedBlueDispenserStrategy: state = DISPENSER" << endl;
				if(!lastWaypointReached){
					AdjustPosition( navsysMessageHandler );
					usleep(10000);
				}
				else{
					sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					state = PICK_UP_BALL;
					//startTime = timeLeft;
					lastWaypointReached = FALSE;
					usleep(10000);
				}
				break;
			case PICK_UP_BALL:
				//cout << "RedBlueDispenserStrategy: state = PICK_UP_BALL" << endl;
				if(newBall){
					lastTryOk = TRUE;
					startTime = -1;
					cout << "New ball picked up!" << endl;
					while(_subject->GetBallStored()<balls.GetNumberOfRobotBalls(ALL)){
						navsysMessageHandler->handleAllMessagesNow();
						usleep(1000);
					}
					if(balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL) < (numberOfBallsToPickUp-1)){
						nextBall = COLORED_BALL;
					}
					else{
						nextBall = WHITE_BALL;
					}
					if(balls.GetNumberOfRobotBalls(ALL) == MAX_NUMBER_OF_BALLS-1 ){
						if(balls.GetNumberOfWhiteBallsToPickUp()>2){
							nextBall = WHITE_BALL;
						}
						else{
							nextBall = COLORED_BALL;
						}
					}						
					if(teamColor == TEAM_COLOR_RED){
						CheckBall(proxysysMessageHandler, navsysMessageHandler, nextBall);
					}
					else if(teamColor == TEAM_COLOR_BLUE){
						CheckBall(proxysysMessageHandler, navsysMessageHandler, nextBall);
					}
					newBall = FALSE;
					ballStored = FALSE;
					usleep(10000);
				}
				if(wheelReady || tryAgain){
					if(lastTryOk){	
						while(_subject->GetWheelReady()<balls.GetNumberOfRobotBalls(ALL)){
							navsysMessageHandler->handleAllMessagesNow();
							usleep(1000);
						}
						wheelReady = FALSE;
						usleep(BELT_INPUT_TIMER);
						cout << "New ball stored!" << endl;
					}
					cout << "Number of red balls: " << balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL) << endl;
					if(balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL)<2){
						cout << "Picking up another red ball!" << endl;
						sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
						activeWaypoint.id = 1;
						activeWaypoint.x_l = 100;
						activeWaypoint.y_l = 0;
						activeWaypoint.psi_l = 0;
						activeWaypoint.circle_of_acceptance = 30;
						sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
						while(!lastWaypointReached){
							navsysMessageHandler->handleAllMessagesNow();
						}
						lastWaypointReached = FALSE;
						state = PICK_UP_BALL;
						sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
						usleep(200000);
						sendReverseWaypoint( navsysMessageHandler, -100, 0, 0, 30, RELATIVE_WAYPOINT );
						while(!lastWaypointReached){
							navsysMessageHandler->handleAllMessagesNow();
						}
						startTime = timeLeft;
						lastWaypointReached = FALSE;
						tryAgain = FALSE;	
					}
					else{
						cout << "Done picking up red balls!" << endl;
						state = REVERSE;
						_subject->SetRearAntiCollisionActive(TRUE);
						lastWaypointSent = FALSE;
						lastWaypointReached = FALSE;
						sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_IN,proxysysMessageHandler);
						usleep(10000);
					}
				}
				if(startTime-timeLeft>DISPENSER_TIME_OUT){
					cout << "Timing out..." << endl;
					lastTryOk = FALSE;
					startTime = -1;
					newBall = FALSE;
					tryAgain = TRUE;
					lastWaypointSent = FALSE;
					lastWaypointReached = FALSE;
					state = PICK_UP_BALL;
					sendPositionArms(ARMS_CLOSED,MOTOR_OFF,BELTS_DIRECTION_IN,proxysysMessageHandler);
					tryCounter ++;
					if(tryCounter>=2){	
						state = REVERSE;
						if(teamColor == TEAM_COLOR_RED){						
							balls.RemoveBall(RED_BALL, 5);
						}
						else if(teamColor == TEAM_COLOR_BLUE){
							balls.RemoveBall(BLUE_BALL, 5);
						}
						balls.SetExtraBall(WHITE_ROBOT_BALL,(numberOfBallsToPickUp - balls.GetNumberOfRobotBalls(RED_BLUE_ROBOT_BALL)));
						cout << "Dispenser seems to be empty if position is OK" << endl;		
					}				
					usleep(10000);
				}
				break;
			case REVERSE:
				if(!lastWaypointSent){
					cout << "RedBlueDispenserStrategy: state = REVERSE" << endl;
					lastWaypointReached = FALSE;
					if(teamColor == TEAM_COLOR_RED){
						activeWaypoint = GetWaypoint(STANDARD_CONTAINER_BLUE_WAYPOINT);
						psi = M_PI - atan2((activeWaypoint.y_l-750),(2300-activeWaypoint.x_l));
						sendReverseWaypoint( navsysMessageHandler, 2300, 750, psi, 50, GLOBAL_WAYPOINT );
					}
					else if(teamColor == TEAM_COLOR_BLUE){
						activeWaypoint = GetWaypoint(STANDARD_CONTAINER_RED_WAYPOINT);
						psi = atan2((activeWaypoint.y_l-750),(activeWaypoint.x_l-700));
						sendReverseWaypoint( navsysMessageHandler, 700, 750, psi, 50, GLOBAL_WAYPOINT );
					}
					_subject->SetLastWaypointSent(activeWaypoint);						
					lastWaypointSent = TRUE;
					usleep(10000);
				}
				if(lastWaypointReached){
					_subject->SetRearAntiCollisionActive(FALSE);
					strategyCompleted = TRUE;
					_subject->SetFirstRunVariable(FALSE);	
					lastWaypointReached = FALSE;
					startTime = timeLeft;
					balls.SetExtraBall(RED_BLUE_ROBOT_BALL,0);
					usleep(10000);
				}
				break;
			default:
				cout << "Wrong state!" << endl;
				break;
		}
		usleep(STRATEGY_WHILE_DELAY);
		//requestPosition(navsysMessageHandler);
		CheckForCollision(navsysMessageHandler);
		local_time_message = _subject->GetTimeObservation();
		timeLeft = local_time_message.timeLeft;
		//UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
	_subject->SetFirstRunVariable(FALSE);
}

