#include <math.h>

#include "SearchForBallStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

SearchForBallStrategy::SearchForBallStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	strategyDone = FALSE;
	priority = 5;
	internalState = 5;
	timeLeft = 90;
	Init();
	_subject = s;
	_subject->Attach(this);
	cout << "Strategy added" << endl;
}

SearchForBallStrategy::~SearchForBallStrategy (){
 	_subject->Detach(this);
}

void SearchForBallStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	strategyDone = FALSE;
	priority = 5;
	internalState = 5;
	timeLeft = 90;
	Init();
	cout << "Strategy reset!" << endl;
}

void SearchForBallStrategy::Init(){
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	collision = FALSE;
	collisionStop = FALSE;
	cvObsFlag = FALSE;
	newBall = FALSE;
	ballStored = FALSE;
	wheelReady = FALSE;
	lastCvObsOk = FALSE;
	ballCheck = 0;
	numberOfCvObsAsks = 0;
	adjustCount = 0;
	waypointCounter = 0;
	state = WAYPOINTS;
}

void SearchForBallStrategy::InitWaypoints(){
	ai_waypoint searchWaypoint;
	searchWaypoint.id = 3;
	searchWaypoint.circle_of_acceptance = 50;
	
	if(teamColor == TEAM_COLOR_RED){
		/*searchWaypoint.x_l = 1800;
		searchWaypoint.y_l = 1600;
		searchWaypoint.psi_l = -90*M_PI/180;
		searchWaypoints[0] = searchWaypoint;*/

		searchWaypoint.x_l = 1000;
		searchWaypoint.y_l = 750;
		searchWaypoint.id = 3;		
		searchWaypoint.psi_l = 180*M_PI/180;
		searchWaypoints[0] = searchWaypoint;

		searchWaypoint.x_l = 600;
		searchWaypoint.y_l = 750;
		searchWaypoint.id = 3;
		searchWaypoint.psi_l = 90*M_PI/180;
		searchWaypoints[1] = searchWaypoint;

		searchWaypoint.x_l = 700;
		searchWaypoint.y_l = 1600;
		searchWaypoint.id = 3;
		searchWaypoint.psi_l = 90*M_PI/180;
		searchWaypoints[2] = searchWaypoint;
	}
	else if(teamColor == TEAM_COLOR_BLUE){
		/*searchWaypoint.x_l = 1200;
		searchWaypoint.y_l = 1600;
		searchWaypoint.psi_l = -90*M_PI/180;
		searchWaypoints[0] = searchWaypoint;*/

		searchWaypoint.x_l = 2000;
		searchWaypoint.y_l = 750;
		searchWaypoint.id = 3;
		searchWaypoint.psi_l = 0*M_PI/180;
		searchWaypoints[0] = searchWaypoint;

		searchWaypoint.x_l = 2800;
		searchWaypoint.y_l = 750;
		searchWaypoint.id = 3;
		searchWaypoint.psi_l = 90*M_PI/180;
		searchWaypoints[1] = searchWaypoint;

		searchWaypoint.x_l = 2700;
		searchWaypoint.y_l = 1600;
		searchWaypoint.id = 3;
		searchWaypoint.psi_l = 90*M_PI/180;
		searchWaypoints[2] = searchWaypoint;		
	}
}

void SearchForBallStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision_message = _subject->GetCollisionMessage();
		#ifdef ANTI_COLLISION_ACTIVE		
			collision = TRUE;
		#else
			collision = FALSE;
		#endif
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation(); 
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;	
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_CMD_LOW_BATTERY ){
		strategyDone = TRUE;
	}
	else if( typeOfObservation == MINOR_BALL_STORED ){
		ballStored = TRUE;
	}
	else if( typeOfObservation == MINOR_WHEEL_READY){
		wheelReady = TRUE;
	}
	else if( typeOfObservation == MINOR_COLOR_DETECTED ){
		newBall = TRUE;
	}
	else if( typeOfObservation == MINOR_CV_OBSERVATION ){
		my_cv_observation = _subject->GetCvObservation();
		cout << "Ball.x: " << my_cv_observation.x << ", Ball.y: " << my_cv_observation.y << endl;
		cvObsFlag = TRUE;
	}
}

void SearchForBallStrategy::UpdatePriority(){
	local_time_message = _subject->GetTimeObservation();
	timeLeft = local_time_message.timeLeft;	
	//cout << "Time left: " << timeLeft << endl;

	if(teamColor == TEAM_COLOR_RED){
		if(balls.GetNumberOfBalls(RED_BALL) == 0 && balls.GetNumberOfBalls(RED_WHITE_BALL) == 0 && balls.GetNumberOfBalls(BLUE_WHITE_BALL) == 0)
		{
			internalState = 0;
		}
		else
		{
			internalState = 5;
		}
	}
	else if(teamColor == TEAM_COLOR_BLUE){
		if(balls.GetNumberOfBalls(BLUE_BALL) >= 0 && balls.GetNumberOfBalls(RED_WHITE_BALL) >= 0 && balls.GetNumberOfBalls(BLUE_WHITE_BALL) >= 0)
		{
			internalState = 0;
		}
		else
		{
			internalState = 5;
		}
	}
/*	if(((balls.GetNumberOfRobotBalls(ALL))>=5)){
		strategyDone = TRUE;
		priority = 0;
	}
	else if(!_subject->GetFirstRunVariable() && timeLeft>=15){
		priority = 9;	//HACKHACKHACK
		//priority = 9.5-internalState*1.9;
	}
	else{
		priority = 2;	//HACKHACKHACK
	}*/
	if(timeLeft<15){
		strategyDone = TRUE;
		priority = 0;
	}
	else{
		priority = 2;
	}	
}

int SearchForBallStrategy::GetPriority(){
	return priority;
}

void SearchForBallStrategy::CheckForCollision(MessageHandler *navsysMessageHandler){
	if(collision){
		_subject->GetCollisionValues(navsysMessageHandler,boards);
		cout << "board4: " << boards[3] << ", board5: " << boards[4] << endl;

		if(boards[0]<=(COLLISION_THRESHOLD) || boards[1]<=(COLLISION_THRESHOLD)){
			#ifdef ANTI_COLLISION_ACTIVE
				if(_subject->GetAntiCollisionActive()){
					//_subject->SetCollision(TRUE);
					collisionStop = TRUE;
					state = COLLISION;
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
		}
		else if(boards[2]<=COLLISION_THRESHOLD){
			#ifdef REAR_ANTI_COLLISION_ACTIVE
				if(_subject->GetRearAntiCollisionActive()){
					_subject->SetCollision(TRUE);
				}
			#else
				_subject->SetCollision(FALSE);
			#endif
			
		}
		collision = FALSE;
	}
}

ai_waypoint SearchForBallStrategy::GetCvWaypoint(){
	ai_waypoint cvWaypoint;
	int type;
	cvWaypoint.id = 1;
	cvWaypoint.x_l = my_cv_observation.x - 50;
	cvWaypoint.y_l = my_cv_observation.y;
	cvWaypoint.psi_l = atan2( cvWaypoint.y_l , cvWaypoint.x_l );
	cvWaypoint.circle_of_acceptance = 50;
	type = my_cv_observation.type;
	if(type == teamColor || type == WHITE){
		lastCvObsOk = TRUE;
		cout << "cvWaypoint sent: x: " << cvWaypoint.x_l << ", y: " << cvWaypoint.y_l << endl;
	}
	else{
		cout << "No ball found" << endl;
		lastCvObsOk = FALSE;
	}
	return cvWaypoint;
}

void SearchForBallStrategy::CheckBall( MessageHandler *proxysysMessageHandler, MessageHandler *navsysMessageHandler, int nextBallColor){
	ballCheck = _subject->CheckBall(proxysysMessageHandler, nextBallColor, WHITE);
	if(ballCheck>-1){
		my_ball_message = _subject->GetNewBallMessage();
		sendBallAck( &my_ball_message, proxysysMessageHandler );
		if(ballCheck == WHITE){
			balls.AddRobotBalls(WHITE_ROBOT_BALL,1);
		}
		else{
			balls.AddRobotBalls(RED_BLUE_ROBOT_BALL,1);
		}
	}
	else{
		sendRelativeWaypoint(navsysMessageHandler, 0, 0, -45*M_PI/180, 50);
		while(!lastWaypointReached){
			navsysMessageHandler->handleAllMessagesNow();
			usleep(1000);
		
		}
		lastWaypointReached = FALSE;
		sendBallAck( &my_ball_message, proxysysMessageHandler );
		sendRelativeWaypoint(navsysMessageHandler, 0, 0, 45*M_PI/180, 50);
		while(!lastWaypointReached){
			navsysMessageHandler->handleAllMessagesNow();
			usleep(1000);
		}
		lastWaypointReached = FALSE;
	}
}

void SearchForBallStrategy::AdjustPosition( MessageHandler *navsysMessageHandler, MessageHandler *cvsysMessageHandler ){
	collision_detection_message adjust_message;
	adjust_message.data[2] = 0;
	adjust_message.data[3] = 0;

	cout << "Start adjusting!" << endl;
	while(!_subject->GetCollisionValue() && !lastCvObsOk){
		if(adjustCount == 0){
			adjust_message.data[4] = 1;
		}
		else{
			adjust_message.data[4] = 0;
		}
		adjustCount ++;
		if(teamColor == TEAM_COLOR_RED){
			adjust_message.data[0] = 20;
			adjust_message.data[1] = 20;
		}
		else if(teamColor == TEAM_COLOR_BLUE){
			adjust_message.data[0] = 20;
			adjust_message.data[1] = 15;			
		}
		sendAdjustPosition(&adjust_message, navsysMessageHandler);
		askForCvObservation( cvsysMessageHandler );
		cvsysMessageHandler->handleAllMessagesNow();
		CheckForCollision(navsysMessageHandler);
		while(!cvObsFlag){
			usleep(1000);
			cvsysMessageHandler->handleAllMessagesNow();
		}
		cvObsFlag = FALSE;
		activeWaypoint = GetCvWaypoint();		
	}
	if(lastCvObsOk){
		adjust_message.data[4] = 2;
		state = GO_TO_BALL;
		cout << "Ball found!!!" << endl;
	}
	else if(_subject->GetCollisionValue()){
		adjust_message.data[4] = 2;
		strategyDone = TRUE;
		cout << "Collision detected!!!" << endl;
	}
	sendAdjustPosition( &adjust_message, navsysMessageHandler );
}

void SearchForBallStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  ){
	int startTime = 0;
	strategyDone = FALSE;
	cout << "SearchForBallStrategy" << endl;
	cout << "Time left: " << timeLeft << endl;
	_subject->SetAntiCollisionActive(FALSE);
	Init();
	InitWaypoints();
	while(!strategyDone && timeLeft>=15){
		switch (state) {
			case ADJUST:
				AdjustPosition(navsysMessageHandler, cvsysMessageHandler);
				break;
			case WAYPOINTS:
				if(!lastWaypointSent){
					lastCvObsOk = FALSE;
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					if(waypointCounter>2){
						waypointCounter = 0;
					}
					activeWaypoint = searchWaypoints[waypointCounter];
					sendRobotWaypoint( navsysMessageHandler, &activeWaypoint);
					lastWaypointSent = TRUE;	
					usleep(10000);
				}
				askForCvObservation( cvsysMessageHandler );
				while(!cvObsFlag){
					cvsysMessageHandler->handleAllMessagesNow();
					usleep(1000);
				}
				cvObsFlag = FALSE;
				activeWaypoint = GetCvWaypoint();
				if(lastCvObsOk){
					clearWaypoints(navsysMessageHandler);
					lastWaypointSent = FALSE;
					state = GO_TO_BALL;
					cout << "Ball found!!!" << endl;
				}
				else{
					state = WAYPOINTS;
				}
				lastCvObsOk = FALSE;

				if(lastWaypointReached){
					//sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					waypointCounter ++;
					lastWaypointReached = FALSE;
					lastWaypointSent = FALSE;
					//startTime = timeLeft;
					state = WAYPOINTS;
					usleep(10000);
				}
				if(collisionStop){
					state = COLLISION;
				}
				break;
			case GO_TO_BALL:
				if(!lastWaypointSent){
					lastCvObsOk = FALSE;
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					sendRelativeRobotWaypoint( navsysMessageHandler, &activeWaypoint );
					lastWaypointSent = TRUE;	
					usleep(10000);
				}
				if(lastWaypointReached){
					sendPositionArms(ARMS_CLOSED,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					state = PICK_UP_BALL;
					lastWaypointReached = FALSE;
					startTime = timeLeft;
					usleep(10000);
				}
				break;
			case PICK_UP_BALL:
				if(newBall){
					cout << "New ball picked up!" << endl;
					CheckBall(proxysysMessageHandler, navsysMessageHandler, WHITE_BALL);
					newBall = FALSE;
					//startTime = timeLeft;
					usleep(10000);
				}
				if(wheelReady){
					usleep(BELT_INPUT_TIMER);					
					sendPositionArms(ARMS_READY,MOTOR_ON,BELTS_DIRECTION_IN,proxysysMessageHandler);
					wheelReady = FALSE;
					lastWaypointSent = FALSE;





					//state = WAYPOINTS;
					state = COLLISION;




					adjustCount = 0;
					usleep(10000);
					if(balls.GetNumberOfRobotBalls(ALL)>=5){
						strategyDone = TRUE;
					}
				}
				if(startTime-timeLeft>2*TIME_OUT){
					newBall = FALSE;
					ballStored = FALSE;
					lastWaypointSent = FALSE;
					state = WAYPOINTS;
					adjustCount = 0;
					sendPositionArms(ARMS_OPEN,MOTOR_OFF,BELTS_DIRECTION_IN,proxysysMessageHandler);
					usleep(10000);
				}
				break;
			case COLLISION:
				collisionStop = FALSE;
				clearWaypoints(navsysMessageHandler);
				lastWaypointReached = FALSE;
				lastWaypointSent = FALSE;
				if(teamColor == TEAM_COLOR_RED){
					sendReverseWaypoint(navsysMessageHandler, -300, 0, -90*M_PI/180, 50, RELATIVE_WAYPOINT);
				}
				else if(teamColor == TEAM_COLOR_BLUE){
					sendReverseWaypoint(navsysMessageHandler, -300, 0, 90*M_PI/180, 50, RELATIVE_WAYPOINT);
				}
				while(!lastWaypointReached){
					navsysMessageHandler->handleAllMessagesNow();
					usleep(10000);
				}
				waypointCounter = waypointCounter + 2;


				//state = WAYPOINTS;
				strategyDone = TRUE;



				break;
			default:
				cout << "WRONG STATE!" << endl;
		}
		usleep(STRATEGY_WHILE_DELAY);
		//requestPosition(navsysMessageHandler);
		CheckForCollision(navsysMessageHandler);  
		local_time_message = _subject->GetTimeObservation();
		timeLeft = local_time_message.timeLeft;
		//UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
}
