#ifndef SEARCH_FOR_BALL_STRATEGY_H
#define SEARCH_FOR_BALL_STRATEGY_H

#include <math.h>

#include "Observer.h"
#include "PlansysObservation.h"
#include "planner.h"
#include "waypoint.h"
#include "posix_message_config.h"
#include "init.h"
#include "observation_structs.h"

class SearchForBallStrategy: public Observer {
  public:
    SearchForBallStrategy( PlansysObservation * , int theTeamColor );
    ~SearchForBallStrategy();
	void Reset( int theTeamColor );
	void Init();
	void InitWaypoints();
	void Update( int typeOfObservation );
	void UpdatePriority();
	int GetPriority();
	void CheckForCollision(MessageHandler *navsysMessageHandler);
	ai_waypoint GetCvWaypoint();
	void AdjustPosition( MessageHandler *navsysMessageHandler, MessageHandler *cvsysMessageHandler );
	void CheckBall( MessageHandler *proxysysMessageHandler, MessageHandler *navsysMessageHandler, int nextBallColor);
	void PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler  );
  
  private:
	int lastWaypointSent;	
	int lastWaypointReached;
	int collision;
	int collisionStop;
	int newBall;
	int wheelReady;
	int ballStored;
	int ballCheck;

	int numberOfCvObsAsks;
	
	collision_detection_message collision_message;		
	cv_observation my_cv_observation;
	
	int cvObsFlag;
	int lastCvObsOk;
	int strategyDone;

	int adjustCount;
	int waypointCounter;

	int board1, board2, board3, board4, board5;
	int boards[NUMBER_OF_BOARDS];
	int ballColor;

	double internalState;
	double distance;
	double distance_x;
	double distance_y;
	int timeLeft;
	int priority;

	int state;
	enum{
		ADJUST,
		WAYPOINTS,
		GO_TO_BALL,
		PICK_UP_BALL,
		COLLISION,
	};

	PlansysObservation *_subject; 
	ai_waypoint ai_waypoint_3d;
	ai_waypoint last_ai_waypoint;
	ai_waypoint activeWaypoint;
	ai_waypoint searchWaypoints[3];
	
	cv_observation new_cv_observation;
	ball_message my_ball_message;
	proxy_message my_proxy_message;
	time_message local_time_message;
};

#endif
