#include <math.h>

#include "WhiteDispenserStrategy.h"
#include "waypoints.h"
#include "planner.h"
#include "PlansysObservation.h"
#include "observations.h"
#include "posixCommunication.h"

WhiteDispenserStrategy::WhiteDispenserStrategy ( PlansysObservation *s , int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	redWhiteDispenser = GetWaypoint(RED_WHITE_DISPENSER_WAYPOINT);
	blueWhiteDispenser = GetWaypoint(BLUE_WHITE_DISPENSER_WAYPOINT);
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = INIT;
	collision = FALSE;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalStateRed = 5;
	internalStateBlue = 5;
	internalState = 5;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	timeLeft = 90;
	_subject = s;
	_subject->Attach(this);
	cout << "Strategy added" << endl;
}

WhiteDispenserStrategy::~WhiteDispenserStrategy (){
 	_subject->Detach(this);
}

void WhiteDispenserStrategy::Reset( int theTeamColor ){
	teamColor = theTeamColor;
	initWaypoints();
	redWhiteDispenser = GetWaypoint(RED_WHITE_DISPENSER_WAYPOINT);
	blueWhiteDispenser = GetWaypoint(BLUE_WHITE_DISPENSER_WAYPOINT);
	ai_waypoint_3d = GetWaypoint(ZERO_WAYPOINT);
	state = INIT;
	strategyDone = FALSE;
	strategyCompleted = FALSE;
	priority = 5;
	internalState = 5;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	timeLeft = 90;
	cout << "Strategy reset!" << endl;
}

int WhiteDispenserStrategy::GetPriority(){
	return priority;
}

void WhiteDispenserStrategy::Update( int typeOfObservation ){
	usleep(5000);
	if( typeOfObservation == MINOR_COLLISION_DETECTED ){
		collision_message = _subject->GetCollisionMessage();
		board1 = collision_message.data[0];
		board2 = collision_message.data[2];
		board3 = collision_message.data[3];
		cout << "\n\nBoard1: " << board1 << endl;
		cout << "Board2: " << board2 << endl;
		cout << "Board3: " << board3 << endl;
		if(board1<=20 || board2<=20 || board3<=20){
			_subject->SetCollision(TRUE);
			cout << "\n\nCOLLISION\n\n" << endl;
		}
	}
	else if( typeOfObservation == MINOR_NAVSYS_POSITION ){
		ai_waypoint_3d = _subject->GetPosObservation(); 
	}
	else if( typeOfObservation == MINOR_NAVSYS_REACHED_WAYPOINT ){
		lastWaypointReached = TRUE;	
	}
	else if( typeOfObservation == MINOR_CMD_SYSTEM_INITIALIZE ){
		strategyDone = TRUE;
	}
}

void WhiteDispenserStrategy::UpdatePriority(){
	double maxDistance, minValue;
	local_time_message = _subject->GetTimeObservation();
	timeLeft = local_time_message.timeLeft;

	ai_waypoint_3d = _subject->GetPosObservation();
	/*if( teamColor == TEAM_COLOR_RED ){
		distance_x = sqrt(pow((redWhiteDispenser.x_l - ai_waypoint_3d.x_l),2));
		distance_y = sqrt(pow((redWhiteDispenser.y_l - ai_waypoint_3d.y_l),2));
		distance = sqrt(pow(distance_x,2)+pow(distance_y,2));
	}
	else if( teamColor == TEAM_COLOR_BLUE ){
		distance_x = sqrt(pow((blueWhiteDispenser.x_l - ai_waypoint_3d.x_l),2));
		distance_y = sqrt(pow((blueWhiteDispenser.y_l - ai_waypoint_3d.y_l),2));
		distance = sqrt(pow(distance_x,2)+pow(distance_y,2));
		
	}*/
	internalStateRed = balls.GetNumberOfBalls(RED_WHITE_BALL);
	internalStateBlue = balls.GetNumberOfBalls(BLUE_WHITE_BALL);
	if( internalStateRed > internalStateBlue ){
		internalState = internalStateRed;
	}
	else{
		internalState = internalStateBlue;
	}
	if( !(int)internalState ){
		strategyCompleted = TRUE;
	}

	distance_x = sqrt(pow((redWhiteDispenser.x_l - ai_waypoint_3d.x_l),2));
	distance_y = sqrt(pow((redWhiteDispenser.y_l - ai_waypoint_3d.y_l),2));
	distance = sqrt(pow(distance_x,2)+pow(distance_y,2));
	minValue = distance;
	distance_x = sqrt(pow((blueWhiteDispenser.x_l - ai_waypoint_3d.x_l),2));
	distance_y = sqrt(pow((blueWhiteDispenser.y_l - ai_waypoint_3d.y_l),2));
	distance = sqrt(pow(distance_x,2)+pow(distance_y,2));
	if(distance<minValue){
		minValue = distance;
	}

	maxDistance = sqrt(pow(3000,2)+pow(2100,2));
	// Time is weighted half the weight of the distance+internalState, altogether 0.9*11 with a maximum of 9.9, which will give a priority of 9
	if( !strategyCompleted ){
		priority = ((((maxDistance-minValue)/maxDistance)*0.5 + internalState*0.1)*0.9)*11;
	}
	else{
		priority = 0;
	}
}

void WhiteDispenserStrategy::pickupBalls(int color)
{
	int dispenser = 0;
	int numberOfBalls = 0;
	cout << "Ball picked from dispenser? 0=no, 1=yes: " << endl;
	cin >> dispenser;
	cout << "How many balls? " << endl;
	cin >> numberOfBalls;
	switch (color)
	{
		case RED_WHITE_BALL:
			balls.AddRobotBalls(WHITE_ROBOT_BALL,numberOfBalls);
			if(dispenser)
			{
				balls.RemoveBall(RED_WHITE_BALL,numberOfBalls);
			}
			break;
		case BLUE_WHITE_BALL:
			balls.AddRobotBalls(WHITE_ROBOT_BALL,numberOfBalls);
			if(dispenser)
			{
				balls.RemoveBall(BLUE_WHITE_BALL,numberOfBalls);
			}
			break;

		case RED_BALL:
			balls.AddRobotBalls(RED_BLUE_ROBOT_BALL,numberOfBalls);
			if(dispenser)
			{
				balls.RemoveBall(RED_BALL,numberOfBalls);
			}
			break;
		case BLUE_BALL:
			balls.AddRobotBalls(RED_BLUE_ROBOT_BALL,numberOfBalls);
			if(dispenser)
			{
				balls.RemoveBall(BLUE_BALL,numberOfBalls);
			}		
			break;
		default:
			break;
	}
}

void WhiteDispenserStrategy::PerformStrategy( MessageHandler *navsysMessageHandler, MessageHandler *proxysysMessageHandler, MessageHandler *cvsysMessageHandler ){
	strategyDone = FALSE;
	lastWaypointSent = FALSE;
	lastWaypointReached = FALSE;
	while(!strategyCompleted && !strategyDone && !_subject->GetCollisionValue() && timeLeft>0){
		if(teamColor == TEAM_COLOR_RED){
			switch (state) {
				case INIT:
					cout << "WhiteDispenserStrategy: state = INIT" << endl;
					if(!lastWaypointSent){
						activeWaypoint = GetWaypoint(RED_WHITE_DISPENSER_WAYPOINT);
						navsysMessageHandler->sendMessage( (char *)&activeWaypoint, (int)sizeof(ai_waypoint), MINOR_NAVSYS_SET_WAYPOINT, PRIORITY_HIGH );
						cout << "Waypoint, red ball, x_l: " << activeWaypoint.x_l << endl;
						lastWaypointSent = TRUE;
						usleep(10000);
					}
					if(lastWaypointReached){
						pickupBalls(RED_WHITE_BALL);
						lastWaypointReached = FALSE;
						lastWaypointSent = FALSE;
						strategyDone = TRUE;
						usleep(10000);
					}
					break;
				default:
					cout << "Out of states!" << endl;
					break;
			}
		}
		else if(teamColor == TEAM_COLOR_BLUE){	
			switch (state) {
				case INIT:
					cout << "WhiteDispenserStrategy: state = INIT" << endl;
					if(!lastWaypointSent){
						activeWaypoint = GetWaypoint(BLUE_WHITE_DISPENSER_WAYPOINT);
						navsysMessageHandler->sendMessage( (char *)&activeWaypoint, (int)sizeof(ai_waypoint), MINOR_NAVSYS_SET_WAYPOINT, PRIORITY_HIGH );
						cout << "First waypoint, x_l: " << activeWaypoint.x_l << endl;
						lastWaypointSent = TRUE;
						usleep(10000);
					}
					if(lastWaypointReached){
						pickupBalls(BLUE_WHITE_BALL);
						lastWaypointReached = FALSE;
						lastWaypointSent = FALSE;
						strategyDone = TRUE;
						usleep(10000);
					}
				break;
				default:
					cout << "Out of states!" << endl;
					break;
			}

		}
		usleep(300000); 
		UpdatePriority();
		navsysMessageHandler->handleAllMessagesNow();
	}
}
