#include <iostream>
#include <sys/time.h>
#include <math.h>
#include <libplayerc++/playerc++.h>

#include <libpcan.h>
#include <pcan.h>

#include "interface.h"
#include "simtest.h"

using namespace PlayerCc;

MessageHandler *plansysMessageHandler;

double xPos, yPos, yaw, x, y, newYaw;
collision_detection_message collision_message;
double board1,board2,board3;

int waypointReached = FALSE;
int waypointReceived = FALSE;
//player_pose_2dt position;
//player_pose_2dt velocity;

PlayerClient    robot("localhost");
SonarProxy	sp(&robot,0);
Position2dProxy pp(&robot,0);


void returnPosition(posix_robot_state_3dof *pos)
{
	checkPosition();
	pos->x_l = (int) (xPos*750);
	pos->y_l = (int) (yPos*750);
	pos->psi_l = yaw;
	cout << pos->x_l << ", " << pos->y_l << ", " << pos->psi_l << endl;
}

void checkPosition()
{
	xPos = pp.GetXPos();
	yPos = pp.GetYPos();
	yaw = pp.GetYaw();
}

void stopRobot()
{
	checkPosition();
	pp.GoTo(xPos,yPos,yaw);
}

void setWaypoint(double x_l, double y_l, double yaw_l)
{
	pp.GoTo(x_l,y_l,yaw_l);
	std::cout << "x, y, yaw: " << x_l << ", " << y_l << ", " << yaw_l << std::endl;
	x = x_l;
	y = y_l;
	newYaw = yaw_l;
	waypointReceived = TRUE;
}

void setRelativeWaypoint(double x_l, double y_l, double yaw_l)
{
	pp.GoTo(xPos+x_l,yPos+y_l,yaw);
	std::cout << "x, y, yaw: " << x_l << ", " << y_l << ", " << yaw_l << std::endl;
	x = x_l+xPos;
	y = y_l+yPos;
	newYaw = yaw;
	waypointReceived = TRUE;
}

void checkCollision()
{
	board1 = sp[2]*40;
	board2 = ((sp[3]+sp[4])/2)*40;
	board3 = sp[5]*40;
	//std::cout << "Board1: " << board1 << ", Board2: " << board2 << ", Board3: " << board3 << std::endl;
	if(board1<20 || board2<20 || board3<20)
	{
		collision_message.data[1] = board1;
		collision_message.data[2] = board2;
		collision_message.data[3] = board3;
		plansysMessageHandler->sendMessage((char *)&collision_message, (int)sizeof(collision_detection_message), MINOR_COLLISION_DETECTED, PRIORITY_HIGHEST );
		//std::cout << "Collision detected!" << std::endl;
	}
}

int main(int argc, char *argv[])
{	
	plansysMessageHandler = new MessageHandler(NAVSYS_ID, PMQ_NAVSYS, PMQ_PLANSYS, false);
	initMessageHandlers();
	
	pp.ResetOdometry();
	//pp.SetSpeed(2,2,2);

	ai_waypoint awp;

	x = 3.0;
	y = 2.1;
	newYaw = 3.14;
	
	while(1)
	{
		// read from the proxies
		robot.Read();
		//std::cout << sp << std::endl;
		checkCollision();
		plansysMessageHandler->handleAllMessagesNow();
		// print out sonars for fun
		
		/*std::cout << "Enter x-position: " << std::endl;
		std::cin >> x;
		std::cout << "Enter y-position: " << std::endl;
		std::cin >> y;
		std::cout << "Enter yaw-angle: " << std::endl;
		std::cin >> newYaw;*/
		//pp.GoTo(x,y,newYaw);

		checkPosition();
		
		while(!waypointReached || !waypointReceived)
		{	
			robot.Read();
			checkCollision();
			//std::cout << sp << std::endl;
			plansysMessageHandler->handleAllMessagesNow();
			usleep(5000);
			checkPosition();
			/*std::cout << "------------------------------------" << std::endl;
			std::cout << "X-pos, x: " << xPos << ", " << x << std::endl;
			std::cout << "Y-pos, y: " << yPos << ", " << y << std::endl;
			std::cout << "yaw, newYaw: " << yaw << ", " << newYaw << std::endl;
			std::cout << "------------------------------------" << std::endl;*/
			usleep(5000);
			if(fabs((fabs(xPos)-fabs(x)))<0.02 && fabs((fabs(yPos)-fabs(y)))<0.02 && fabs((fabs(yaw)-fabs(newYaw)))<0.1)
			{
				waypointReached = TRUE;
				std::cout << "Waypoint reached!" << std::endl;
				std::cout << (fabs(xPos)-fabs(x)) << std::endl;
				//Hack
				x = xPos + 1000;
				y = yPos + 1000;
				newYaw = newYaw + 1000;
			}
			usleep(300000);
		}
		std::cout << "Done while" << std::endl;
		waypointReached = FALSE;
		waypointReceived = FALSE;
		awp.x_l = (int)xPos*1000;
		awp.y_l = (int)yPos*1000;
		awp.psi_l = yaw;
		plansysMessageHandler->sendMessage((char *)&awp, sizeof(ai_waypoint), MINOR_NAVSYS_REACHED_WAYPOINT, PRIORITY_HIGH);
	}
}
